@@ -1,4 +1,4 @@
-/* @(#)limits.h	1.6 11/08/03 Copyright 2011 J. Schilling */
+/* @(#)limits.h	1.7 11/08/13 Copyright 2011 J. Schilling */
 /*
  *	Copyright (c) 2011 J. Schilling
  */
@@ -9,6 +9,8 @@
  * with the License.
  *
  * See the file CDDL.Schily.txt in this distribution for details.
+ * A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file CDDL.Schily.txt from this distribution.
@@ -36,6 +38,13 @@
 #define	_POSIX2_LINE_MAX	2048
 #endif
 
+/*
+ * Include sys/param.h for PIPE_BUF
+ */
+#ifndef	_SCHILY_PARAM_H
+#include <schily/param.h>
+#endif
+
 #ifndef	PIPE_BUF
 #if	defined(__MINGW32__) || defined(_MSC_VER)
 #define	PIPE_BUF		5120
