@@ -1,4 +1,4 @@
-/* @(#)schily.h	1.100 11/10/21 Copyright 1985-2011 J. Schilling */
+/* @(#)schily.h	1.130 18/10/14 Copyright 1985-2018 J. Schilling */
 /*
  *	Definitions for libschily
  *
@@ -18,7 +18,7 @@
  *	include ctype.h past schily/schily.h as OpenBSD does not follow POSIX
  *	and defines EOF in ctype.h
  *
- *	Copyright (c) 1985-2011 J. Schilling
+ *	Copyright (c) 1985-2018 J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -27,6 +27,8 @@
  * with the License.
  *
  * See the file CDDL.Schily.txt in this distribution for details.
+ * A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file CDDL.Schily.txt from this distribution.
@@ -50,12 +52,12 @@
 extern "C" {
 #endif
 
-#if	defined(_INCL_SYS_TYPES_H) || defined(_INCL_TYPES_) || defined(off_t)
+#if	defined(_INCL_SYS_TYPES_H) || defined(_INCL_TYPES_H) || defined(off_t)
 #	ifndef	FOUND_OFF_T
 #	define	FOUND_OFF_T
 #	endif
 #endif
-#if	defined(_INCL_SYS_TYPES_H) || defined(_INCL_TYPES_) || defined(size_t)
+#if	defined(_INCL_SYS_TYPES_H) || defined(_INCL_TYPES_H) || defined(size_t)
 #	ifndef	FOUND_SIZE_T
 #	define	FOUND_SIZE_T
 #	endif
@@ -165,6 +167,10 @@ extern	int	js_fspawnl __PR((FILE *, FILE *, FILE *, const char *, ...));
 extern	int	js_fspawnv_nowait __PR((FILE *, FILE *, FILE *,
 					const char *, int, char *const*));
 extern	int	js_fgetline __PR((FILE *, char *, int));
+#ifdef	FOUND_SIZE_T
+extern	ssize_t	fgetaline __PR((FILE *, char **, size_t *));
+extern	ssize_t	getaline __PR((char **, size_t *));
+#endif
 extern	int	fgetstr __PR((FILE *, char *, int));
 extern	int	file_getraise __PR((FILE *));
 extern	void	file_raise __PR((FILE *, int));
@@ -214,17 +220,43 @@ extern	int	spawnv_nowait __PR((FILE *, FILE *, FILE *,
 #endif	/* __never_def__ */
 #endif	/* EOF */
 
+/*
+ * Flags for absfpath() and resolvefpath():
+ */
+#define	RSPF_EXIST		0x01	/* All path components must exist    */
+#define	RSPF_NOFOLLOW_LAST	0x02	/* Don't follow link in last pathcomp */
+
 #ifdef	FOUND_SIZE_T
 extern	char	*abspath __PR((const char *relp, char *absp, size_t asize));
+extern	char	*absnpath __PR((const char *relp, char *absp, size_t asize));
+extern	char	*absfpath __PR((const char *relp, char *absp, size_t asize,
+				int __flags));
+#ifndef	HAVE_RESOLVEPATH
+extern	int	resolvepath __PR((const char *__path,
+				char *__buf, size_t __bufsiz));
 #endif
+extern	int	resolvenpath __PR((const char *__path,
+				char *__buf, size_t __bufsiz));
+extern	int	resolvefpath __PR((const char *__path,
+				char *__buf, size_t __bufsiz, int __flags));
+#endif
+
+#ifdef	_SCHILY_TYPES_H
+extern	int	mkdirs __PR((char *, mode_t));
+extern	int	makedirs __PR((char *, mode_t, int __striplast));
+#endif
+
 extern	int	lxchdir	__PR((char *));
 #ifdef	HAVE_FCHDIR
-#define	fdsetname(fd, name)
-#define	fdclosename(fd)
+#define	fdsetname(fd, name)	(0)
+#define	fdclosename(fd)		(0)
 #else
 extern	int	fdsetname __PR((int fd, const char *name));
 extern	int	fdclosename __PR((int fd));
 #endif
+extern	int	diropen __PR((const char *));
+extern	int	dirrdopen __PR((const char *));
+extern	int	dirclose __PR((int));
 
 struct save_wd {
 	int	fd;
@@ -256,6 +288,10 @@ extern	int	gettnum	__PR((char *arg, time_t *valp));
 #endif
 
 #ifdef	_SCHILY_TIME_H
+
+extern	int		getnstimeofday	__PR((struct timespec *__tp));
+extern	int		setnstimeofday	__PR((struct timespec *__tp));
+
 #ifdef	_SCHILY_UTYPES_H
 extern	Llong		mklgmtime	__PR((struct tm *));
 #endif
@@ -272,6 +308,7 @@ extern	time_t		mkgmtime	__PR((struct tm *));
 #define	GP_DOX		1	/* 'X' perm character is valid		  */
 #define	GP_XERR		2	/* 'X' perm characters are invalid	  */
 #define	GP_FPERM	4	/* TRUE if we implement find -perm	  */
+#define	GP_UMASK	8	/* TRUE if we implement umask		  */
 
 extern	int	getperm	__PR((FILE *f, char *perm, char *opname, \
 				mode_t *modep, int smode, int flag));
@@ -290,7 +327,11 @@ extern	int	on_comerr __PR((void (*fun)(int, void *), void *arg));
 /*PRINTFLIKE1*/
 extern	void	comerr __PR((const char *, ...)) __printflike__(1, 2);
 /*PRINTFLIKE2*/
+extern	void	xcomerr	 __PR((int, const char *, ...)) __printflike__(2, 3);
+/*PRINTFLIKE2*/
 extern	void	comerrno __PR((int, const char *, ...)) __printflike__(2, 3);
+/*PRINTFLIKE3*/
+extern	void	xcomerrno __PR((int, int, const char *, ...)) __printflike__(3, 4);
 /*PRINTFLIKE1*/
 extern	int	errmsg __PR((const char *, ...)) __printflike__(1, 2);
 /*PRINTFLIKE2*/
@@ -306,27 +347,99 @@ extern	int	serrmsgno __PR((int, char *, size_t, const char *, ...))
 extern	void	comexit	__PR((int));
 extern	char	*errmsgstr __PR((int));
 
+/*PRINTFLIKE1*/
+extern	void	gtcomerr __PR((const char *, ...)) __printflike__(1, 2);
+/*PRINTFLIKE2*/
+extern	void	gtxcomerr __PR((int, const char *, ...)) __printflike__(2, 3);
+/*PRINTFLIKE2*/
+extern	void	gtcomerrno __PR((int, const char *, ...)) __printflike__(2, 3);
+/*PRINTFLIKE3*/
+extern	void	gtxcomerrno __PR((int, int, const char *, ...)) __printflike__(3, 4);
+/*PRINTFLIKE1*/
+extern	int	gterrmsg __PR((const char *, ...)) __printflike__(1, 2);
+/*PRINTFLIKE2*/
+extern	int	gterrmsgno __PR((int, const char *, ...)) __printflike__(2, 3);
+
+#ifdef	GT_COMERR
+#define	comerr		gtcomerr
+#define	xcomerr		gtxcomerr
+#define	comerrno	gtcomerrno
+#define	xcomerrno	gtxcomerrno
+#define	errmsg		gterrmsg
+#define	errmsgno	gterrmsgno
+#endif
+
 #ifdef	EOF	/* stdio.h has been included */
 /*PRINTFLIKE2*/
 extern	void	fcomerr		__PR((FILE *, const char *, ...))
 					__printflike__(2, 3);
 /*PRINTFLIKE3*/
+extern	void	fxcomerr	__PR((FILE *, int, const char *, ...))
+					__printflike__(3, 4);
+/*PRINTFLIKE3*/
 extern	void	fcomerrno	__PR((FILE *, int, const char *, ...))
 					__printflike__(3, 4);
+/*PRINTFLIKE4*/
+extern	void	fxcomerrno	__PR((FILE *, int, int, const char *, ...))
+					__printflike__(4, 5);
 /*PRINTFLIKE2*/
 extern	int	ferrmsg		__PR((FILE *, const char *, ...))
 					__printflike__(2, 3);
 /*PRINTFLIKE3*/
 extern	int	ferrmsgno	__PR((FILE *, int, const char *, ...))
 					__printflike__(3, 4);
-#ifdef	_SCHILY_VARARGS_H
+
+/*PRINTFLIKE2*/
+extern	void	fgtcomerr	__PR((FILE *, const char *, ...))
+					__printflike__(2, 3);
+/*PRINTFLIKE3*/
+extern	void	fgtxcomerr	__PR((FILE *, int, const char *, ...))
+					__printflike__(3, 4);
+/*PRINTFLIKE3*/
+extern	void	fgtcomerrno	__PR((FILE *, int, const char *, ...))
+					__printflike__(3, 4);
 /*PRINTFLIKE4*/
-extern	int	_comerr		__PR((FILE *, int, int, const char *, va_list));
-#endif
+extern	void	fgtxcomerrno	__PR((FILE *, int, int, const char *, ...))
+					__printflike__(4, 5);
+/*PRINTFLIKE2*/
+extern	int	fgterrmsg	__PR((FILE *, const char *, ...))
+					__printflike__(2, 3);
+/*PRINTFLIKE3*/
+extern	int	fgterrmsgno	__PR((FILE *, int, const char *, ...))
+					__printflike__(3, 4);
+
+#ifdef	GT_COMERR
+#define	fcomerr		fgtcomerr
+#define	fxcomerr	fgtxcomerr
+#define	fcomerrno	fgtcomerrno
+#define	fxcomerrno	fgtxcomerrno
+#define	ferrmsg		fgterrmsg
+#define	ferrmsgno	fgterrmsgno
 #endif
 
+#ifdef	_SCHILY_VARARGS_H
+/*
+ * Definitions for the second _comerr() parameter:
+ */
+#define	COMERR_RETURN	0	/* Do not exit, return only		*/
+#define	COMERR_EXIT	1	/* Exit program, map (X % 256) != X to -64 */
+#define	COMERR_EXCODE	2	/* Exit program, do not map exit codes	*/
+/*PRINTFLIKE5*/
+extern	int	_comerr		__PR((FILE *, int, int, int,
+						const char *, va_list));
+#endif	/* _SCHILY_VARARGS_H */
+#endif	/* EOF */
+
 /*PRINTFLIKE1*/
 extern	int	error __PR((const char *, ...)) __printflike__(1, 2);
+/*PRINTFLIKE1*/
+extern	int	gterror __PR((const char *, ...)) __printflike__(1, 2);
+
+#ifdef	GT_ERROR
+#undef	error
+#define	error		gterror
+#endif
+
 #ifdef	FOUND_SIZE_T
 extern	char	*fillbytes __PR((void *, ssize_t, char));
 extern	char	*zerobytes __PR((void *, ssize_t));
@@ -365,6 +478,7 @@ extern	int	printf __PR((const char *, ...)) __printflike__(1, 2);
 #endif
 #ifdef	FOUND_SIZE_T
 extern	char	*movebytes __PR((const void *, void *, ssize_t));
+extern	char	*movecbytes __PR((const void *, void *, int, size_t));
 #endif
 
 extern	void	save_args __PR((int, char **));
@@ -373,10 +487,12 @@ extern	char	**saved_av __PR((void));
 extern	char	*saved_av0 __PR((void));
 extern	char	*searchfileinpath __PR((char *__name, int __mode,
 					int __file_mode, char *__path));
-#define	SIP_ANY_FILE	0x00
-#define	SIP_PLAIN_FILE	0x01
-#define	SIP_NO_PATH	0x10
-#define	SIP_TYPE_MASK	0x0F
+#define	SIP_ANY_FILE	0x00	/* Search for any file type		*/
+#define	SIP_PLAIN_FILE	0x01	/* Search for plain files - not dirs	*/
+#define	SIP_NO_PATH	0x10	/* Do not do PATH search		*/
+#define	SIP_ONLY_PATH	0x20	/* Do only PATH search			*/
+#define	SIP_NO_STRIPBIN	0x40	/* Do not strip "/bin" from PATH elem.	*/
+#define	SIP_TYPE_MASK	0x0F	/* Mask file type related bits		*/
 
 #ifndef	seterrno
 extern	int	seterrno __PR((int));
@@ -390,17 +506,6 @@ extern	void	setfp __PR((void * const *));
 extern	int	wait_chld __PR((int));		/* for fspawnv_nowait() */
 extern	int	geterrno __PR((void));
 extern	void	raisecond __PR((const char *, long));
-#ifdef	FOUND_SIZE_T
-/*
- * We currently cannot define this here because there IXIX has a definition
- * than violates the standard.
- */
-#ifndef	HAVE_SNPRINTF
-/*PRINTFLIKE3*/
-extern	int	snprintf __PR((char *, size_t, const char *, ...))
-					__printflike__(3, 4);
-#endif
-#endif
 #ifdef	__never__
 /*
  * sprintf() may be declared incorrectly somewhere else
@@ -408,17 +513,26 @@ extern	int	snprintf __PR((char *, size_t, const char *, ...))
  */
 extern	int	sprintf __PR((char *, const char *, ...));
 #endif
+extern	char	*ovstrcpy __PR((char *, const char *));
 extern	char	*strcatl __PR((char *, ...));
+#ifdef	FOUND_SIZE_T
+extern	size_t	strlcatl __PR((char *, size_t, ...));
+#endif
 extern	int	streql __PR((const char *, const char *));
 #ifdef	_SCHILY_WCHAR_H
 extern	wchar_t	*wcscatl __PR((wchar_t *, ...));
+#ifdef	FOUND_SIZE_T
+extern	size_t	wcslcatl __PR((wchar_t *, size_t, ...));
+#endif
 extern	int	wcseql __PR((const wchar_t *, const wchar_t *));
 #endif
 #ifdef	va_arg
-extern	int	format __PR((void (*)(char, long), long, const char *,
+extern	int	format __PR((void (*)(char, void *), void *, const char *,
 							va_list));
+extern	int	fprformat __PR((void *, const char *, va_list));
 #else
-extern	int	format __PR((void (*)(char, long), long, const char *, void *));
+extern	int	format __PR((void (*)(char, void *), void *, const char *, void *));
+extern	int	fprformat __PR((void *, const char *, void *));
 #endif
 
 extern	int	ftoes __PR((char *, double, int, int));
@@ -428,6 +542,11 @@ extern	int	qftoes __PR((char *, long double, int, int));
 extern	int	qftofs __PR((char *, long double, int, int));
 #endif
 
+/*PRINTFLIKE1*/
+extern	int	js_error __PR((const char *, ...)) __printflike__(1, 2);
+/*PRINTFLIKE2*/
+extern	int	js_dprintf	__PR((int, const char *, ...))
+							__printflike__(2, 3);
 #ifdef	EOF	/* stdio.h has been included */
 /*PRINTFLIKE2*/
 extern	int	js_fprintf	__PR((FILE *, const char *, ...))
@@ -444,6 +563,14 @@ extern	int	js_snprintf	__PR((char *, size_t, const char *, ...))
 extern	int	js_sprintf	__PR((char *, const char *, ...))
 							__printflike__(2, 3);
 
+#ifdef	EOF	/* stdio.h has been included */
+/*PRINTFLIKE2*/
+extern	int	fgtprintf	__PR((FILE *, const char *, ...))
+							__printflike__(2, 3);
+#endif	/* EOF */
+/*PRINTFLIKE1*/
+extern	int	gtprintf	__PR((const char *, ...)) __printflike__(1, 2);
+
 #ifdef	FOUND_SIZE_T
 extern	void	swabbytes	__PR((void *, ssize_t));
 #endif
@@ -484,6 +611,14 @@ extern	int	_openfd64	__PR((const char *, int));
 
 #ifndef	NO_SCHILY_PRINT		/* Define to disable *printf() redirects */
 #ifdef	SCHILY_PRINT
+#ifdef	__never__
+#ifndef	GT_ERROR
+#undef	error
+#define	error		js_error
+#endif
+#endif	/* __never__ */
+#undef	dprintf
+#define	dprintf		js_dprintf
 #undef	fprintf
 #define	fprintf		js_fprintf
 #undef	printf
@@ -496,9 +631,9 @@ extern	int	_openfd64	__PR((const char *, int));
 #ifndef	HAVE_SNPRINTF
 #undef	snprintf
 #define	snprintf	js_snprintf
-#endif
-#endif
-#endif
+#endif	/* HAVE_SNPRINTF */
+#endif	/* SCHILY_PRINT */
+#endif	/* NO_SCHILY_PRINT */
 
 #ifndef	NO_SCHILY_GETLINE	/* Define to disable *getline() redirect */
 #undef	getline
@@ -527,6 +662,7 @@ extern	int	_openfd64	__PR((const char *, int));
 #define	fspawnl		js_fspawnl
 #endif
 
+extern	int	js_mexval	__PR((int exval));
 #ifdef	FOUND_SIZE_T
 extern	void	*js_malloc	__PR((size_t size, char *msg));
 extern	void	*js_realloc	__PR((void *ptr, size_t size, char *msg));
@@ -543,6 +679,7 @@ extern	char	*js_savestr	__PR((const char *s));
 #define	JM_EXIT		((sigjmps_t *)-1) /* Call comexit(errno) instead */
 #define	JM_RETURN	((sigjmps_t *)0)  /* Return instead		 */
 
+extern	int	js_jmexval	__PR((int exval));
 #ifdef	FOUND_SIZE_T
 extern	void	*js_jmalloc	__PR((size_t size, char *msg, sigjmps_t *jmp));
 extern	void	*js_jrealloc	__PR((void *ptr, size_t size, char *msg,
@@ -550,6 +687,7 @@ extern	void	*js_jrealloc	__PR((void *ptr, size_t size, char *msg,
 #endif
 extern	char	*js_jsavestr	__PR((const char *s, sigjmps_t *jmp));
 
+extern	int	js_fjmexval	__PR((int exval));
 #ifdef	EOF	/* stdio.h has been included */
 #ifdef	FOUND_SIZE_T
 extern	void	*js_fjmalloc	__PR((FILE *f, size_t size, char *msg,
@@ -561,13 +699,16 @@ extern	char	*js_fjsavestr	__PR((FILE *f, const char *s, sigjmps_t *jmp));
 #endif	/* EOF */
 #endif	/* _SCHILY_JMPDEFS_H */
 
+#define	___mexval	js_mexval
 #define	___malloc	js_malloc
 #define	___realloc	js_realloc
 #define	___savestr	js_savestr
+#define	__jmexval	js_jmexval
 #define	__jmalloc	js_jmalloc
 #define	__jrealloc	js_jrealloc
 #define	__jsavestr	js_jsavestr
 #define	__fjmalloc	js_fjmalloc
+#define	__fjmexval	js_fjmexval
 #define	__fjrealloc	js_fjrealloc
 #define	__fjsavestr	js_fjsavestr
 
