@@ -1,8 +1,8 @@
-/* @(#)utime.h	1.6 06/09/13 Copyright 2001 J. Schilling */
+/* @(#)utime.h	1.9 10/08/24 Copyright 2001-2010 J. Schilling */
 /*
  *	Defines for utimes() / utime()
  *
- *	Copyright (c) 2001 J. Schilling
+ *	Copyright (c) 2001-2010 J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -11,6 +11,8 @@
  * with the License.
  *
  * See the file CDDL.Schily.txt in this distribution for details.
+ * A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file CDDL.Schily.txt from this distribution.
@@ -23,9 +25,8 @@
 #include <schily/mconfig.h>
 #endif
 
-#ifndef	_INCL_SYS_TYPES_H
-#include <sys/types.h>
-#define	_INCL_SYS_TYPES_H
+#ifndef	_SCHILY_TYPES_H
+#include <schily/types.h>
 #endif
 
 #ifdef	HAVE_UTIMES
@@ -46,10 +47,20 @@
 #define	_INCL_SYS_UTIME_H
 #endif
 #else
+
+#ifdef	__cplusplus
+extern "C" {
+#endif
+
 struct utimbuf {
 	time_t	actime;
 	time_t	modtime;
 };
+
+#ifdef	__cplusplus
+}
+#endif
+
 #endif
 #endif
 
@@ -68,5 +79,4 @@ struct utimbuf {
 
 #endif	/* __comment__ */
 
-
 #endif	/* _SCHILY_UTIME_H */
