@@ -24,12 +24,12 @@
  * Use is subject to license terms.
  */
 /*
- * This file contains modifications Copyright 2006-2015 J. Schilling
+ * Copyright 2006-2017 J. Schilling
  *
- * @(#)getopt.c	1.11 15/08/31 J. Schilling
+ * @(#)getopt.c	1.19 19/02/21 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)getopt.c 1.11 15/08/31 J. Schilling"
+#pragma ident "@(#)getopt.c 1.19 19/02/21 J. Schilling"
 #endif
 
 #if defined(sun)
@@ -55,7 +55,10 @@
 /*#include "synonyms.h"*/
 /*#include "_libc_gettext.h"*/
 
-#define	_libc_gettext	gettext
+#if !defined(TEXT_DOMAIN)	/* Should be defined by cc -D */
+#define	TEXT_DOMAIN "SYS_TEST"	/* Use this only if it weren't */
+#endif
+#define	_libc_gettext(s)	dgettext(TEXT_DOMAIN, s)
 
 #include <schily/nlsdefs.h>
 
@@ -67,14 +70,26 @@
 #define	snprintf	js_snprintf
 #endif
 
+#ifndef	__CYGWIN__
+/*
+ *	Cygwin uses a nonstandard:
+ *
+ *		__declspec(dllexport)
+ *	or
+ *		__declspec(dllimport)
+ *
+ *	that is in conflict with our standard definition.
+ */
 extern int optind, opterr, optopt;
 extern char *optarg;
+#endif
 
 static char *parseshort __PR((const char *optstring, const int c));
 #ifdef	DO_GETOPT_LONGONLY
 static int  parseshortval __PR((const char *optstring, const char *cp));
 #endif
-static char *parselong  __PR((const char *optstring, const char *opt, char **longoptarg));
+static char *parselong  __PR((const char *optstring, const char *opt,
+				char **longoptarg));
 
 /*
  * Generalized error processing macro. The parameter i is a pointer to
@@ -102,6 +117,17 @@ static char *parselong  __PR((const char *optstring, const char *opt, char **lon
  * So, why isn't this "static" you ask?  Because the historical Bourne
  * shell has actually latched on to this little piece of private data.
  */
+#if defined(HAVE_PRAGMA_WEAK) && defined(HAVE_LINK_WEAK)
+/*
+ * The name of the variable on SVr4 was _sp, but we enhanced the features
+ * of getopt() to support long options and since recent versions of the
+ * Bourne Shell rely on long option support, the easiest way to signal the
+ * enhanced features of this getopt() implementation is to change the name
+ * of the state variable to opt_sp.
+ */
+#pragma weak _sp = opt_sp	/* Backwards compatibility for old programs */
+#endif
+#define	_sp	opt_sp
 int _sp = 1;
 
 /*
@@ -216,6 +242,8 @@ parselong(optstring, opt, longoptarg)
 
 	cp = ip = (char *)optstring;
 	do {
+		if (*ip == '\0')
+			break;
 		if (*ip != '(' && *++ip == '\0')
 			break;
 		if (*ip == ':' && *++ip == '\0')
@@ -230,10 +258,10 @@ parselong(optstring, opt, longoptarg)
 			if (match && *ip == ')' &&
 			    (*op == '\0' || *op == '=')) {
 				if ((*op) == '=') {
-				    /* may be an empty string - OK */
-				    (*longoptarg) = op + 1;
+					/* may be an empty string - OK */
+					(*longoptarg) = op + 1;
 				} else {
-				    (*longoptarg) = NULL;
+					(*longoptarg) = NULL;
 				}
 				return (cp);
 			}
@@ -247,7 +275,7 @@ parselong(optstring, opt, longoptarg)
 		 * required argument.
 		 */
 		while ((cp > optstring) && ((*cp) == ':')) {
-		    --cp;
+			--cp;
 		}
 	} while (*cp != '\0');
 	return (NULL);
@@ -266,6 +294,7 @@ getopt(argc, argv, optstring)
 	char	*cp;
 	int	longopt;
 	char	*longoptarg = NULL;
+	int	l = 2;
 
 	/*
 	 * Has the end of the options been encountered?  The following
@@ -281,11 +310,11 @@ getopt(argc, argv, optstring)
 	 */
 	if (optind >= argc || argv[optind] == NULL ||
 	    argv[optind][0] != '-' || argv[optind][1] == '\0') {
-		return (EOF);
+		return (-1);
 	} else if (argv[optind][0] == '-' && argv[optind][1] == '-' &&
 	    argv[optind][2] == '\0') {		/* "--" */
 		optind++;
-		return (EOF);
+		return (-1);
 	}
 
 	/*
@@ -310,9 +339,32 @@ getopt(argc, argv, optstring)
 	optopt = c = (unsigned char)argv[optind][_sp];
 	optarg = NULL;
 	longopt = (_sp == 1 && c == '-');
+#ifdef	DO_GETOPT_SDASH_LONG
+	/*
+	 * If optstring starts with "()", traditional UNIX -long options are
+	 * allowed in addition to --long.
+	 */
+	if (optstring[0] == '(' ||
+	    (optstring[0] == ':' && optstring[1] == '(')) {
+		if (!longopt && _sp == 1 &&
+		    c != '\0' && argv[optind][2] != '\0') {
+			longopt = l = 1;
+		}
+	}
+tryshort:
+#endif
 	if (!(longopt ?
-	    ((cp = parselong(optstring, argv[optind]+2, &longoptarg)) != NULL) :
+	    ((cp = parselong(optstring, argv[optind]+l, &longoptarg)) != NULL) :
 	    ((cp = parseshort(optstring, c)) != NULL))) {
+#ifdef	DO_GETOPT_SDASH_LONG
+		if (longopt && optopt != '-') {
+			/*
+			 * In case of "-long" retry as combined short options.
+			 */
+			longopt = 0;
+			goto tryshort;
+		}
+#endif
 		/* LINTED: variable format specifier */
 		ERR(_libc_gettext("%s: illegal option -- %s\n"),
 		    c, (longopt ? optind : 0));
