@@ -1,14 +1,14 @@
-/* @(#)common.h	1.23 15/06/02 2011-2015 J. Schilling */
+/* @(#)common.h	1.29 18/04/10 2011-2018 J. Schilling */
 /*
  *	Copyright (c) 1986, 1988 Larry Wall
- *	Copyright (c) 2011-2015 J. Schilling
+ *	Copyright (c) 2011-2018 J. Schilling
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following condition is met:
  *
  * 1. Redistributions of source code must retain the above copyright notice,
  * this condition and the following disclaimer.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
  * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -39,9 +39,7 @@
 #include <schily/utypes.h>
 #include <schily/errno.h>
 #include <schily/nlsdefs.h>
-#ifndef	NO_SCHILY_PRINT
-#include <schily/schily.h>
-#endif
+#include <schily/schily.h>	/* e.g. ___malloc() */
 
 #define	CH	(char)
 #define	UCH	(unsigned char)
@@ -73,6 +71,7 @@
 #define	Fclose		(void)fclose
 #define	Fflush		(void)fflush
 #define	Sprintf		(void)sprintf
+#define	Snprintf	(void)snprintf
 #define	Mktemp		(void)mktemp
 #define	Strcpy		(void)strcpy
 #define	Strcat		(void)strcat
@@ -82,9 +81,11 @@
 #define	INITHUNKMAX	125		/* initial dynamic allocation size */
 #define	BUFFERSIZE	8192
 #define	SCCSPREFIX	"s."
-#define	GET		"get -e %s"
+#define	GET		"get"		/* use: "get -e %s" */
+#define	GETEDIT		"-e"
 #define	RCSSUFFIX	",v"
-#define	CHECKOUT	"co -l %s"
+#define	CHECKOUT	"co"		/* use: "co -l %s" */
+#define	COEDIT		"-l"
 
 #ifdef FLEXFILENAMES
 #define	ORIGEXT		".orig"
@@ -121,6 +122,7 @@ EXT char **Argv_last;
 
 EXT struct stat file_stat;		/* file statistics area */
 EXT int filemode;
+EXT struct timespec filetime;
 
 EXT char *buf;				/* general purpose buffer */
 EXT size_t bufsize;			/* current size of buf */
@@ -144,6 +146,8 @@ EXT char rejname[128];			/*   -r rejname		*/
 EXT char *origext;			/*   -b ext		*/
 EXT char *origprae;
 
+EXT int	TMPDLEN;
+EXT char *TMPDIR;
 EXT char *TMPOUTNAME;
 EXT char *TMPINNAME;
 EXT char *TMPREJNAME;
@@ -160,6 +164,8 @@ EXT bool verbose;			/* ! -s			*/
 EXT bool reverse;			/*   -R			*/
 EXT bool noreverse;			/*   -N			*/
 EXT bool skip_rest_of_patch;		/*   -S			*/
+EXT bool touch_local;			/*   -T			*/
+EXT bool touch_gmt;			/*   -Z			*/
 EXT int strippath;			/*   -p#		*/
 EXT bool canonicalize;			/*   -l			*/
 
@@ -186,3 +192,7 @@ EXT char *revision;			/* prerequisite revision, if any */
 EXT LINENUM p_repl_lines;		/* From pch.c # of replacement lines */
 
 extern	void	my_exit __PR((int status));
+
+#ifdef	HAVE_GETDELIM
+#define	fgetaline(f, bufp, lenp)	getdelim(bufp, lenp, '\n', f)
+#endif
