@@ -1,19 +1,19 @@
-/* @(#)patch.c	1.40 15/06/03 2011-2015 J. Schilling */
+/* @(#)patch.c	1.48 18/04/10 2011-2018 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)patch.c	1.40 15/06/03 2011-2015 J. Schilling";
+	"@(#)patch.c	1.48 18/04/10 2011-2018 J. Schilling";
 #endif
 /*
  *	Copyright (c) 1984-1988 Larry Wall
- *	Copyright (c) 2011-2015 J. Schilling
+ *	Copyright (c) 2011-2018 J. Schilling
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following condition is met:
  *
  * 1. Redistributions of source code must retain the above copyright notice,
  * this condition and the following disclaimer.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
  * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -35,7 +35,7 @@ static	UConst char sccsid[] =
 #include "pch.h"
 #include "inp.h"
 
-#define	PATCHVERSION	"3.1"
+#define	PATCHVERSION	"3.2"
 
 /* procedures */
 
@@ -132,6 +132,8 @@ main(argc, argv)
 	if (tmpdir == NULL) {
 		tmpdir = "/tmp";
 	}
+	TMPDIR = tmpdir;
+	TMPDLEN = strlen(tmpdir) + 1;
 	tmpname_len = strlen(tmpdir) + 20;
 
 	TMPOUTNAME = (char *) malloc(tmpname_len);
@@ -344,7 +346,7 @@ _("\n\nRan out of memory using Plan A--trying again...\n\n"));
 		if (!skip_rest_of_patch) {
 			if (!failed && wall_plus && !strEQ(outname, "-") &&
 			    (file_stat.st_size == (size_t)0) &&
-			    (remove_empty || is_null_time[reverse])) {
+			    (remove_empty || is_null_time[!reverse])) {
 				say(_(
 "Removing file %s and any empty ancestor directories.\n"), outname);
 				move_file(NULL, outname);
@@ -354,6 +356,8 @@ _("\n\nRan out of memory using Plan A--trying again...\n\n"));
 				chmod(TMPOUTNAME, filemode);
 			} else {
 				chmod(outname, filemode);
+				if (touch_local || touch_gmt)
+					settime(outname, !reverse, failed);
 			}
 		}
 		Fclose(rejfp);
@@ -454,6 +458,11 @@ reinitialize_almost_everything()
 		fatal(_("You may not change to a different patch file.\n"));
 }
 
+/*
+ * If called "patch" we implement strict POSIX.
+ * If called "spatch" or "sccspatch", we implement POSIX + extensions.
+ * If called "opatch" we implement "Larry Wall" + extensions.
+ */
 static void
 init_defaults()
 {
@@ -466,7 +475,7 @@ init_defaults()
 			s = Argv[0];
 		else
 			s++;
-		if (strEQ(s, "sccspatch")) {
+		if (strEQ(s, "spatch") || strEQ(s, "sccspatch")) {
 			do_posix = TRUE;
 			wall_plus = TRUE; /* POSIX + old patch options	*/
 		} else if (!strEQ(s, "opatch")) {
@@ -561,12 +570,16 @@ get_some_switches()
 				if (!isalpha(UCH *s) && '_' != *s)
 					fatal(
 _("Argument to -D not an identifier.\n"));
-				Sprintf(if_defined, "#ifdef %s\n", s);
-				Sprintf(not_defined, "#ifndef %s\n", s);
+				Snprintf(if_defined, sizeof (if_defined),
+				    "#ifdef %s\n", s);
+				Snprintf(not_defined, sizeof (not_defined),
+				    "#ifndef %s\n", s);
 				if (do_posix) {
-					Sprintf(end_defined, "#endif\n");
+					Snprintf(end_defined,
+					    sizeof (end_defined), "#endif\n");
 				} else {
-					Sprintf(end_defined,
+					Snprintf(end_defined,
+						sizeof (end_defined),
 						"#endif /* %s */\n", s);
 				}
 				break;
@@ -641,6 +654,11 @@ _("Argument to -D not an identifier.\n"));
 					goto unknown;
 				skip_rest_of_patch = TRUE;
 				break;
+			case 'T':			/* GNU		*/
+				if (!wall_plus)
+					goto unknown;
+				touch_local = TRUE;
+				break;
 			case 'u':			/* POSIX: OK	*/
 				/*
 				 * Added with patch-2.0.12u5
@@ -656,14 +674,25 @@ _("Argument to -D not an identifier.\n"));
 					verbose = TRUE + 1;
 					break;
 				}
+			case 'Z':			/* GNU		*/
+				if (!wall_plus)
+					goto unknown;
+				touch_gmt = TRUE;
+				break;
 
+			printvers:
 				printf(
 				_("patch %s (%s-%s-%s)\n\n"),
 					PATCHVERSION,
 					HOST_CPU, HOST_VENDOR, HOST_OS);
 				printf(
+				_("Compatibility: %s%s%s\n\n"),
+					do_posix ? "POSIX ":"",
+					do_wall ? "Wall-2.0 ":"",
+					wall_plus ? "Wall+":"");
+				printf(
 				_("Copyright (C) 1984 - 1988 Larry Wall\n"));
-				printf(_("Copyright (C) 2011 - 2015 %s\n"),
+				printf(_("Copyright (C) 2011 - 2018 %s\n"),
 					_("Joerg Schilling"));
 				printf(_(
 "This is free software; see the source for copying conditions.  There is NO\n"
@@ -685,6 +714,9 @@ _("Argument to -D not an identifier.\n"));
 				} else if (strEQ(s, "W-posix")) {
 					do_posix = FALSE;
 					do_wall = FALSE;
+				} else if (strEQ(s, "Wv") ||
+				    strEQ(s, "Wversion")) {
+					goto printvers;
 				}
 				break;
 #ifdef DEBUGGING
@@ -717,11 +749,11 @@ Usage: patch [-blNR] [-c|-e|-n|-u] [-d dir] [-D define] [-i patchfile]\n\
 "));
 				} else {
 					fprintf(stderr, _("\
-Usage: patch [-bEflNRsSv] [-c|-e|-n|-u]\n\
+Usage: patch [-bEflNRsSTvZ] [-c|-e|-n|-u]\n\
 \t[-z backup-ext] [-B backup-prefix] [-d directory]\n\
 \t[-D symbol] [-Fmax-fuzz] [-i patchfile] [-o out-file] [-p[strip-count]]\n\
 \t[-r rej-name] [origfile] [patchfile] [[+] [options] [origfile]...]\n\
-\t[-W+] [-Wall] [-Wposix] [-W-posix]\n\
+\t[-Wv] [-Wversion] [-W+] [-Wall] [-Wposix] [-W-posix]\n\
 "));
 				}
 				my_exit(EXIT_FAIL);
