@@ -1,20 +1,20 @@
-/* @(#)pch.c	1.29 15/06/03 2011-2015 J. Schilling */
+/* @(#)pch.c	1.36 18/10/14 2011-2018 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)pch.c	1.29 15/06/03 2011-2015 J. Schilling";
+	"@(#)pch.c	1.36 18/10/14 2011-2018 J. Schilling";
 #endif
 /*
  *	Copyright (c) 1986-1988 Larry Wall
  *	Copyright (c) 1990 Wayne Davison
- *	Copyright (c) 2011-2015 J. Schilling
+ *	Copyright (c) 2011-2018 J. Schilling
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following condition is met:
  *
  * 1. Redistributions of source code must retain the above copyright notice,
  * this condition and the following disclaimer.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
  * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -65,7 +65,6 @@ static	int	intuit_diff_type __PR((void));
 static	void	next_intuit_at __PR((LINENUM file_pos, LINENUM file_line));
 static	void	skip_to __PR((off_t file_pos, off_t file_line));
 static	void	malformed __PR((void));
-static	void	ovstrcpy __PR((char *p2, char *p1));
 static	ssize_t	pgets __PR((char **bfp, size_t *szp, FILE *fp));
 
 /* Prepare to look for the next patch in the patch file. */
@@ -392,17 +391,22 @@ intuit_diff_type()
 	scan_exit:
 
 	if (no_filearg) {
-		if (indtmp != Nullch)
+		if (indtmp != Nullch) {
 			indname = fetchname(indtmp, strippath,
 						ok_to_create_file, NULL);
-		if (oldtmp != Nullch)
+		}
+		if (oldtmp != Nullch) {
 			oldname = fetchname(oldtmp, strippath,
 						ok_to_create_file,
-						&is_null_time[0]);
-		if (newtmp != Nullch)
+						&file_times[0]);
+			is_null_time[0] = file_times[0].dt_sec == 0;
+		}
+		if (newtmp != Nullch) {
 			newname = fetchname(newtmp, strippath,
 						ok_to_create_file,
-						&is_null_time[1]);
+						&file_times[1]);
+			is_null_time[1] = file_times[1].dt_sec == 0;
+		}
 
 		if (do_wall && oldname && newname) {
 			/*
@@ -937,7 +941,7 @@ _("Replacement text or line numbers mangled in hunk at line %lld\n"),
 		fillsrc = 1;
 		filldst = fillsrc + p_ptrn_lines;
 		p_end = filldst + p_repl_lines;
-		Sprintf(buf, "*** %lld,%lld ****\n",
+		Snprintf(buf, bufsize, "*** %lld,%lld ****\n",
 			(Llong)p_first, (Llong)(p_first + p_ptrn_lines - 1));
 		p_line[0] = savestr(buf);
 		if (out_of_mem) {
@@ -945,7 +949,7 @@ _("Replacement text or line numbers mangled in hunk at line %lld\n"),
 			return (FALSE);
 		}
 		p_char[0] = '*';
-		Sprintf(buf, "--- %lld,%lld ----\n",
+		Snprintf(buf, bufsize, "--- %lld,%lld ----\n",
 			(Llong)p_newfirst, (Llong)(p_newfirst+p_repl_lines-1));
 		p_line[filldst] = savestr(buf);
 		if (out_of_mem) {
@@ -1087,7 +1091,7 @@ _("Unexpected end of file in patch.\n"));
 		}
 		p_newfirst = min;
 		p_repl_lines = max - min + 1;
-		Sprintf(buf, "*** %lld,%lld\n",
+		Snprintf(buf, bufsize, "*** %lld,%lld\n",
 			(Llong)p_first, (Llong)(p_first + p_ptrn_lines - 1));
 		p_line[0] = savestr(buf);
 		if (out_of_mem) {
@@ -1124,7 +1128,7 @@ _("Unexpected end of file in patch at line %lld.\n"),
 				fatal(_("--- expected at line %lld of patch.\n"),
 				(Llong)p_input_line);
 		}
-		Sprintf(buf, "--- %lld,%lld\n", (Llong)min, (Llong)max);
+		Snprintf(buf, bufsize, "--- %lld,%lld\n", (Llong)min, (Llong)max);
 		p_line[i] = savestr(buf);
 		if (out_of_mem) {
 			p_end = i-1;
@@ -1174,18 +1178,6 @@ _("Unexpected end of file in patch at line %lld.\n"),
 	return (TRUE);
 }
 
-/*
- * A strcpy() that works with overlapping buffers
- */
-static void
-ovstrcpy(p2, p1)
-	register char	*p2;
-	register char	*p1;
-{
-	while ((*p2++ = *p1++) != '\0')
-		;
-}
-
 /* Input a line from the patch file, worrying about indentation. */
 
 static ssize_t
@@ -1412,11 +1404,31 @@ do_ed_script()
 	if (!skip_rest_of_patch) {
 		Unlink(TMPOUTNAME);
 		copy_file(filearg[0], TMPOUTNAME);
-		if (verbose)
-			Sprintf(buf, "/bin/ed %s", TMPOUTNAME);
-		else
-			Sprintf(buf, "/bin/ed - %s", TMPOUTNAME);
+		/*
+		 * Warning: "ed" stays in command mode in case there is e.g.
+		 * a wrong line number before an "append" command.
+		 * Using this "feature" could allow to hide a shell command
+		 * from the command filter. To avoid related problems, we
+		 * use the "red" command instead and start "red" from /tmp
+		 * as it does not accept filenames with a '/' inside.
+		 */
+		if (verbose) {
+			Snprintf(buf, bufsize, "cd %s && /bin/red %s",
+						TMPDIR, TMPOUTNAME + TMPDLEN);
+		} else {
+			Snprintf(buf, bufsize, "cd %s && /bin/red - %s",
+						TMPDIR, TMPOUTNAME + TMPDLEN);
+		}
 		pipefp = popen(buf, "w");
+#ifdef	__unsafe__
+		if (pipef == NULL {
+			if (verbose)
+				Snprintf(buf, bufsize, "/bin/ed %s", TMPOUTNAME);
+			else
+				Snprintf(buf, bufsize, "/bin/ed - %s", TMPOUTNAME);
+			pipefp = popen(buf, "w");
+		}
+#endif
 	}
 	for (;;) {
 		beginning_of_this_line = ftell(pfp);
@@ -1431,10 +1443,20 @@ do_ed_script()
 		}
 		this_line_is_command = (isdigit(UCH *buf) &&
 		    (*t == 'd' || *t == 'c' || *t == 'a'));
+		if (!this_line_is_command) {
+			/*
+			 * Check for the diff workaround for "." in a line
+			 * that inserts ".." and then substitites the result.
+			 * Most diff programs emit "s/.//\na\a", but diff from
+			 * OpenBSD emits the substitute program with address.
+			 */
+			if (strEQ(buf, "a\n") || strEQ(t, "s/.//\n"))
+				this_line_is_command = 1;
+		}
 		if (this_line_is_command) {
 			if (!skip_rest_of_patch)
 				fputs(buf, pipefp);
-			if (*t != 'd') {
+			if (*t != 'd' && *t != 's') {
 				while (pgets(&buf, &bufsize, pfp) > 0) {
 					p_input_line++;
 					if (!skip_rest_of_patch)
