@@ -3,7 +3,7 @@
  *
  * The contents of this file are subject to the terms of the
  * Common Development and Distribution License ("CDDL"), version 1.0.
- * You may only use this file in accordance with the terms of version
+ * You may use this file only in accordance with the terms of version
  * 1.0 of the CDDL.
  *
  * A full copy of the text of the CDDL should have accompanied this
@@ -27,12 +27,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2014 J. Schilling
+ * Copyright 2006-2019 J. Schilling
  *
- * @(#)admin.c	1.83 14/08/13 J. Schilling
+ * @(#)admin.c	1.122 19/01/17 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)admin.c 1.83 14/08/13 J. Schilling"
+#pragma ident "@(#)admin.c 1.122 19/01/17 J. Schilling"
 #endif
 /*
  * @(#)admin.c 1.39 06/12/12
@@ -84,51 +84,53 @@ and end of the string MUST be included in the translation.  Formatting
 characters, e.g. "%s" "%c" "%d" "\n" "\t" must appear in the
 translated string exactly as they do in the msgid string.  Spaces
 and/or tabs around these formats must be maintained.
- 
+
 The following are examples of text that should not be translated, but
 should appear exactly as they do in the msgid string:
- 
+
 - Any SCCS error code, which will be one or two letters followed by one
   or two numbers all in parenthesis, e.g. "(ad3)",
 
 - All descriptions of SCCS option flags, e.g. "-r" or " 'e'" , or "f",
   or "-fz"
- 
+
 - ".FRED", "sid", "SID", "MRs", "CMR", "p-file", "CASSI",  "cassi",
-  function names, e.g. "getcwd()", 
+  function names, e.g. "getcwd()",
 */
 
 # define MAXNAMES 9
 
-static char	stdin_file_buf [20];
-static char	*ifile, *tfile;
-static char	*dir_name;
-static struct timespec	ifile_mtime;
-static int	Ncomma = 0;
-static int	Nsdot = 1;
-static int	Nsubd;
-static char	*Nprefix;
-static char	*Nparm;
-static char	*CMFAPPL;	/* CMF MODS */
-static char	*z;			/* for validation program name */
-static char	had_flag[NFLAGS], rm_flag[NFLAGS];
+static char	stdin_file_buf [20];	/* For "/tmp/admin.XXXXXX"	*/
+static char	*ifile;			/* -i argument			*/
+static char	*tfile;			/* -t argument			*/
+static char	*dir_name;		/* directory for -N		*/
+static struct timespec	ifile_mtime;	/* Timestamp for -i -o		*/
+static Nparms	N;			/* Keep -N parameters		*/
+static Xparms	X;			/* Keep -X parameters		*/
+static char	*CMFAPPL;		/* CMF MODS			*/
+static char	*z;			/* for validation program name	*/
+static char	had_flag[NFLAGS];	/* -f seen list			*/
+static char	rm_flag[NFLAGS];	/* -d seen list			*/
 #if	defined(PROTOTYPES) && defined(INS_BASE)
-static char	Valpgmp[]	=	NOGETTEXT(INS_BASE "/ccs/bin/" "val");
+static char	Valpgmp[]	=	NOGETTEXT(INS_BASE "/" SCCS_BIN_PRE "bin/" "val");
 #endif
 static char	Valpgm[]	=	NOGETTEXT("val");
-static int	fexists, num_files;
-static int	VFLAG  =  0;
-static int	versflag = 4;
-static struct sid	new_sid;
-static char	*anames[MAXNAMES], *enames[MAXNAMES];
-static char	*unlock;
-static char	*locks;
-static char	*flag_p[NFLAGS];
-static int	asub, esub;
-static int	check_id;
-static struct	utsname	un;
-static char	*uuname;
-static int 	Encoded = EF_TEXT;	/* Default encoding is '0' */
+static int	fexists;		/* Current file exists		*/
+static int	num_files;		/* Number of file args		*/
+static int	VFLAG  =  0;		/* -v option seen		*/
+static int	versflag = 4;		/* history vers for new files	*/
+static struct sid	new_sid;	/* -r argument			*/
+static char	*anames[MAXNAMES];	/* -a arguments			*/
+static char	*enames[MAXNAMES];	/* -e arguments			*/
+static char	*unlockarg;		/* -dl argument			*/
+static char	*locks;			/* 'l' flag value in file	*/
+static char	*flag_p[NFLAGS];	/* -f arguments			*/
+static int	asub;			/* Index for anames[]		*/
+static int	esub;			/* Index for enames[]		*/
+static int	check_id;		/* To check for Keywds with -i	*/
+static struct	utsname	un;		/* uname for lockit()		*/
+static char	*uuname;		/* un.nodename			*/
+static int 	Encoded = EF_TEXT;	/* Default encoding is '0'	*/
 static off_t	Encodeflag_offset;	/* offset in file where encoded flag is stored */
 static off_t	Checksum_offset;	/* offset in file where g-file hash is stored */
 
@@ -146,7 +148,6 @@ static	int	pos_ser __PR((char *s1, char *s2));
 static	int	range __PR((register char *line));
 static	FILE *	code __PR((FILE *iptr, char *afile, off_t offset, int thash, struct packet *pktp));
 static	void	direrror __PR((char *dir, int keylet));
-static	char *	bulkprepare __PR((char *afile));
 
 extern int	org_ihash;
 extern int	org_chash;
@@ -170,7 +171,7 @@ char *argv[];
 	 * Set locale for all categories.
 	 */
 	setlocale(LC_ALL, NOGETTEXT(""));
-	
+
 	sccs_setinsbase(INS_BASE);
 
 	/*
@@ -178,12 +179,12 @@ char *argv[];
 	 */
 #ifdef	PROTOTYPES
 	(void) bindtextdomain(NOGETTEXT("SUNW_SPRO_SCCS"),
-	   NOGETTEXT(INS_BASE "/ccs/lib/locale/"));
+	   NOGETTEXT(INS_BASE "/" SCCS_BIN_PRE "lib/locale/"));
 #else
 	(void) bindtextdomain(NOGETTEXT("SUNW_SPRO_SCCS"),
 	   NOGETTEXT("/usr/ccs/lib/locale/"));
 #endif
-	
+
 	(void) textdomain(NOGETTEXT("SUNW_SPRO_SCCS"));
 
 	tzset();	/* Set up timezome related vars */
@@ -231,8 +232,8 @@ char *argv[];
 			}
 			no_arg = 0;
 			j = current_optind;
-		        c = getopt(argc, argv, "-i:t:m:y:d:f:r:nN:hzboqa:e:V:(version)");
-		        	/*
+		        c = getopt(argc, argv, "()-i:t:m:y:d:f:r:nN:hzboqkw:a:e:X:V:(version)");
+				/*
 				*  this takes care of options given after
 				*  file names.
 				*/
@@ -260,8 +261,16 @@ char *argv[];
 				   ifile = "";
 				   break;
 				}
+				/*
+				 * Use -i. to tell admin to retrieve the ifile
+				 * name from the g-file name with -N
+				 * We cannot check for "." here as -N may have
+				 * been specified later on the cmd line.
+				 */
 				ifile = p;
-				if (*ifile && exists(ifile)) {
+				if (*ifile &&
+				    !(ifile[0] == '.' && ifile[1] == '\0') &&
+				    exists(ifile)) {
 				   if ((Statbuf.st_mode & S_IFMT) == S_IFDIR) {
 					direrror(ifile, c);
 				   } else {
@@ -297,7 +306,7 @@ char *argv[];
 				if (optarg == argv[j+1]) {
 				   no_arg = 1;
 				   Comments = "";
-				} else {  
+				} else {
 				   Comments = p;
 				}
 				break;
@@ -346,7 +355,7 @@ char *argv[];
 						if (!range(p))
 							fatal(gettext("element in list out of range (ad28)"));
 						if (*p != 'a')
-							unlock = p;
+							unlockarg = p;
 					}
 					break;
 
@@ -472,15 +481,16 @@ char *argv[];
 				 break;
 
 			case 'N':	/* creating new SCCS files */
+				initN(&N);
 				if (optarg == argv[j+1]) {
 				   no_arg = 1;
-				   Nparm = "";
 				   break;
 				}
-				Nparm = p;
+				N.n_parm = p;
 				break;
 			case 'n':	/* creating new SCCS file */
 			case 'h':	/* only check hash of file */
+			case 'k':	/* get(1) without keyword expand */
 			case 'z':	/* zero the input hash */
 			case 'b':	/* force file to be encoded (binary) */
 			case 'o':	/* use original file date */
@@ -496,28 +506,46 @@ char *argv[];
                                 }
                                 break;
 
+			case 'w':
+				if (p)
+					whatsetup(p);
+				break;
+
 			case 'a':	/* user-name allowed to make deltas */
 				testklt = 0;
 				if (!(*p) || *p == '-')
 					fatal(gettext("bad a argument (ad8)"));
-				if (asub > MAXNAMES)
+				if (asub > MAXNAMES) {
 					fatal(gettext("too many 'a' keyletters (ad9)"));
-				anames[asub++] = p;
+					/* NOTREACHED */
+				}
+				anames[asub++] = sccs_user(p);
 				break;
 
 			case 'e':	/* user-name to be removed */
 				testklt = 0;
 				if (!(*p) || *p == '-')
 					fatal(gettext("bad e argument (ad10)"));
-				if (esub > MAXNAMES)
+				if (esub > MAXNAMES) {
 					fatal(gettext("too many 'e' keyletters (ad11)"));
-				enames[esub++] = p;
+					/* NOTREACHED */
+				}
+				enames[esub++] = sccs_user(p);
+				break;
+
+			case 'X':
+				X.x_parm = optarg;
+				X.x_flags = XO_INIT_PATH|XO_URAND;
+				if (!parseX(&X))
+					goto err;
+				had[NLOWER+c-'A'] = 0;	/* Allow mult -X */
 				break;
 
 			case 'V':		/* version */
 				if (optarg == argv[j+1]) {
 				doversion:
-					printf("admin %s-SCCS version %s %s (%s-%s-%s)\n",
+					printf(gettext(
+					    "admin %s-SCCS version %s %s (%s-%s-%s)\n"),
 						PROVIDER,
 						VERSION,
 						VDATE,
@@ -535,6 +563,7 @@ char *argv[];
 				}
 
 			default:
+			err:
 				/*
 				 * Check whether "-V" was last arg...
 				 */
@@ -543,7 +572,7 @@ char *argv[];
 				    argv[argc-1][1] == 'V' &&
 				    argv[argc-1][2] == '\0')
 					goto doversion;
-				fatal(gettext("Usage: admin [ -bhnoz ][ -ausername|groupid ]\n\t[ -dflag ][ -eusername|groupid ]\n\t[ -fflag [value]][ -i [filename]]\n\t[ -m mr-list][ -r release ][ -t [description-file]]\n\t[ -N[bulk-spec]] [ -y[comment]] s.filename ..."));
+				fatal(gettext("Usage: admin [ -bhknoz ][ -ausername|groupid ]\n\t[ -dflag ][ -eusername|groupid ]\n\t[ -fflag [value]][ -i [filename]]\n\t[ -m mr-list][ -r release ][ -t [description-file]]\n\t[ -N[bulk-spec]][ -Xxopts ] [ -y[comment]] s.filename ..."));
 			}
 			/*
 			 * Make sure that we only collect option letters from
@@ -566,52 +595,41 @@ char *argv[];
 		}
 	}
 
-	if (num_files == 0) 
+	if (num_files == 0 && !HADUCN)
 		fatal(gettext("missing file arg (cm3)"));
+	if (num_files > 1 && (X.x_opts & (XO_INIT_PATH|XO_URAND)))
+		fatal(gettext("too many file args (cm18)"));
 
-	if (HADUCN) {
-		HADI = HADN = 1;
-		if (*Nparm == ',') {
-			Ncomma = 1;
-			Nparm++;
-		}
-		Nsdot = sccsfile(Nparm);
-		if (Nsdot) {				/* Nparm ends in s. */
-			if (strlen(sname(Nparm)) > 2)
-				fatal(gettext("bad N argument (ad38)"));
-			Nprefix = Nparm;		/* -Ns. / -NSCCS/s. */
-			Nsubd = Nparm != sname(Nparm);
-		} else if (*Nparm == '\0') {		/* Empty Nparm	*/
-			Nprefix = "s.";
-			Nsubd = 0;
-		} else {				/* -NSCCS	*/
-			size_t	len = strlen(Nparm);
-			int	slseen = 1;
+	setsig();
+	xsethome(NULL);
+	if (HADUCN) {					/* Parse -N args  */
+		/*
+		 * initN() was already called while parsing options.
+		 */
+		if (HADI)
+			N.n_flags |= N_IFILE;
+		if (HADI && ifile[0] == '.' && ifile[1] == '\0')
+			N.n_flags |= N_IDOT;
+		if (HADN)
+			N.n_flags |= N_NFILE;
+		parseN(&N);
+		if (N.n_get)
+			N.n_flags |= N_GETI;
 
-			if (Nparm[len-1] != '/') {
-				slseen = 0;
-				len++;
-			}
-			len += 3;
-			Nprefix = fmalloc(len);			
-			cat(Nprefix, Nparm, slseen?"":"/", "s.", (char *)0);
-			Nsubd = 1;
-		}
+		if (N.n_sdot && (sethomestat & SETHOME_OFFTREE))
+			fatal(gettext("-Ns. not supported in off-tree project mode"));
+
+	} else if (HADI && ifile[0] == '.' && ifile[1] == '\0') {
+		direrror(ifile, 'i');
 	}
+
 	if ((HADY || HADM) && ! (HADI || HADN))
 		fatal(gettext("illegal use of 'y' or 'm' keyletter (ad30)"));
-	if (HADI && num_files > 1) /* only one file allowed with `i' */
+	if (HADI && !HADUCN && num_files > 1) /* only one file allowed with `i' */
 		fatal(gettext("more than one file (ad15)"));
 	if ((HADI || HADN) && ! logname())
 		fatal(gettext("USER ID not in password file (cm9)"));
 
-	setsig();
-
-	errno = 0;
-	i = sethome();
-	if (i < 0)
-		efatal(gettext("cannot get project home directory (cm11)"));
-
 	/*
 	Change flags for 'fatal' so that it will return to this
 	routine (main) instead of terminating processing.
@@ -624,12 +642,14 @@ char *argv[];
 	*/
 	for (j=1; j<argc; j++)
 		if ((p = argv[j]) != NULL)
-			do_file(p, admin, HADN|HADI ? 0 : 1, Nsdot);
+			do_file(p, admin, HADN|HADI ? 0 : 1, N.n_sdot);
+
+	if (num_files == 0 && HADUCN)
+		do_file("-", admin, 0, N.n_sdot);
 
 	return (Fcnt ? 1 : 0);
 }
 
-
 /*
 	Routine that actually does admin's work on SCCS files.
 	Existing s-files are copied, with changes being made, to a
@@ -657,11 +677,11 @@ char	*afile;
 	register char *cp;
 	register signed char *q;
 	char	*in_f;		/* ptr holder for lockflag vals in SCCS file */
-	char	nline[BUFSIZ];
+	char	nline[MAXLINE];
 	char	*p_lval, *tval;
 	char	*lval;
 	char	f;		/* character holder for flag character */
-	char	line[BUFSIZ];
+	char	line[MAXLINE];
 	int	ck_it;		/* used for lockflag checking */
 	off_t	offset;
 	int     thash;
@@ -680,12 +700,28 @@ char	*afile;
 	ck_it = 0;
 	from_stdin = 0;
 	dir_name = "";
+ 	Encoded = EF_TEXT;	/* Default encoding is '0' */
 
 	zero((char *) &stats,sizeof(stats));
 
 	if (HADUCN) {
-		afile = bulkprepare(afile);
+		char	*oafile = afile;
+
+		afile = bulkprepare(&N, afile);
+		if (afile == NULL) {
+			if (N.n_ifile)
+				direrror(N.n_ifile, 'i');
+			else
+				direrror(oafile, 'i');
+		}
+		if (N.n_mtime.tv_sec != 0) {
+			ifile_mtime.tv_sec = N.n_mtime.tv_sec;
+			ifile_mtime.tv_nsec = N.n_mtime.tv_nsec;
+		}
 		had_dir = 0;
+		dir_name = N.n_dir_name;
+		if (N.n_flags & N_IDOT)
+			ifile = N.n_ifile;
 	}
 
 	if (HADI && had_dir) /* directory not allowed with `i' keyletter */
@@ -771,11 +807,11 @@ char	*afile;
 	if (fexists) { /* modifying */
 		int	cklen = 8;
 
-		sinit(&gpkt,afile,1);	/* init pkt & open s-file */
+		sinit(&gpkt, afile, SI_OPEN);	/* init pkt & open s-file */
 
 		if (gpkt.p_flags & PF_V6)
 			cklen = 15;
-	
+
 		/* Modify checksum if corrupted */
 
 		if ((int) strlen(gpkt.p_line) > cklen &&
@@ -814,14 +850,29 @@ char	*afile;
 				fclose(xf);
 		}
 
-		sinit(&gpkt,afile,0);	/* and init pkt */
+		sinit(&gpkt, afile, SI_INIT);	/* and init pkt */
 
 		/*
 		 * Initialize global meta data
 		 */
 		if (versflag == 6) {
-			set_init_path(&gpkt, afile, dir_name);
-			urandom(&gpkt.p_rand);
+			if (X.x_opts & XO_INIT_PATH) {
+				gpkt.p_init_path = X.x_init_path;
+			} else if (HADUCN) {
+				/*
+				 * Only if we have been called with -N..., we
+				 * know the real g-file name. We cannot derive
+				 * the g-file name from the s.file name
+				 * otherwise, since we cannot know about sub
+				 * dirs like "SCCS" in the other cases.
+				 */
+				set_init_path(&gpkt, N.n_ifile, dir_name);
+			}
+
+			if (X.x_opts & XO_URAND)
+				gpkt.p_rand = X.x_rand;
+			else
+				urandom(&gpkt.p_rand);
 		}
 	}
 
@@ -901,6 +952,8 @@ char	*afile;
 		dt.d_serial = 1;
 		dt.d_pred = 0;
 
+		gpkt.p_reqsid = dt.d_sid;	/* set sid for changelog */
+
 		del_ba(&dt,line, gpkt.p_flags);	/* form and write */
 		putline(&gpkt,line);	/* delta-table entry */
 
@@ -1074,7 +1127,7 @@ char	*afile;
 			}
 			if (rm_flag[k]) {
 				if (f == LOCKFLAG) {
-					if (unlock) {
+					if (unlockarg) {
 						in_f = lval;
 						if (((lval = adjust(in_f)) != NULL) &&
 							!had_flag[k])
@@ -1103,12 +1156,18 @@ char	*afile;
 				if (('a' + k) == LOCKFLAG && had_flag[k] == 1) {
 					if ((flag_p[k] && *flag_p[k] == 'a') || (lval && *lval == 'a'))
 						locks = NOGETTEXT("a");
-					else if (lval && flag_p[k])
-						locks =
-						cat(nline,lval," ",flag_p[k], (char *)0);
-					else if (lval)
+					else if (lval && flag_p[k]) {
+						nline[0] = '\0';
+						i = strlcatl(nline, sizeof (nline),
+							lval, " ", flag_p[k], (char *)0);
+						if (i >= sizeof (nline))
+							fatal(gettext("line too long (co31)"));
+						locks = nline;
+					} else if (lval) {
 						locks = lval;
-					else locks = flag_p[k];
+					} else {
+						locks = flag_p[k];
+					}
 					sprintf(line,"%c%c %c %s\n",
 						CTLCHAR,FLAG,'a' + k,locks);
 					locks = 0;
@@ -1156,7 +1215,7 @@ char	*afile;
 			so that if we later discover that the file contains
 			non-ASCII characters we can flag it as encoded
 			by setting the value to 1.
-		 	*/
+			*/
 			Encodeflag_offset = ftell(gpkt.p_xiop);
 			sprintf(line,"%c%c %c %d\n",
 				CTLCHAR, FLAG, ENCODEFLAG, Encoded);
@@ -1166,7 +1225,11 @@ char	*afile;
 		 * Writing out SCCS v6 flags belongs here.
 		 */
 
-		putmeta(&gpkt);
+		/*
+		 * Since we are creating a new history file, everything is from
+		 * us and every meta data needs to be written.
+		 */
+		putmeta(&gpkt, M_ALL);
 
 		/*
 		Beginning of descriptive (user) text.
@@ -1273,12 +1336,12 @@ char	*afile;
 			   } else {
 				/* from standard input */
 				int    err = 0, cnt;
-				char   buf[BUFSIZ];
-				FILE * out;	
+				char   buf[MAXLINE];
+				FILE * out;
 				mode_t cur_umask;
 
 				from_stdin = 1;
-				ifile 	   = stdin_file_buf;
+				ifile	   = stdin_file_buf;
 				strlcpy(stdin_file_buf, "/tmp/admin.XXXXXX", sizeof (stdin_file_buf));
 				cur_umask = umask((mode_t)((S_IRWXU|S_IRWXG|S_IRWXO)&~(S_IRUSR|S_IWUSR)));
 #ifdef	HAVE_MKSTEMP
@@ -1295,7 +1358,7 @@ char	*afile;
 				(void)umask(cur_umask);
 				/*CONSTCOND*/
 				while (1) {
-					if ((cnt = fread(buf, 1, BUFSIZ, stdin))) {
+					if ((cnt = fread(buf, 1, sizeof (buf), stdin))) {
 						if (fwrite(buf, 1, cnt, out) == cnt) {
 							continue;
 						}
@@ -1396,6 +1459,14 @@ char	*afile;
 			while (*q)
 				gpkt.p_nhash += *q++;
 		}
+
+#ifdef	TEST_CHANGE
+		if (versflag == 6) {
+			char	cbuf[MAXPATHLEN];
+			change_ba(&gpkt, cbuf, sizeof (cbuf));
+			fprintf(stderr, "%s\n", cbuf);
+		}
+#endif
 	} else {
 		/*
 		Indicate that EOF at this point is ok, and
@@ -1408,7 +1479,7 @@ char	*afile;
 	/* If encoded file, put change "fe" flag and recalculate
 	   the hash value
 	 */
-	
+
 	if (Encoded & EF_UUENCODE)
 	{
 		strcpy(line,"0");
@@ -1443,17 +1514,53 @@ char	*afile;
 			chmod(gpkt.p_file, sbuf.st_mode);
 			chown(gpkt.p_file,sbuf.st_uid, sbuf.st_gid);
 		}
-		if (HADI && *ifile && Ncomma) {
-			char cfile[FILESIZE];
-			register char *snp;
+		if (HADO) {
+			struct timespec	ts[2];
+			extern dtime_t	Timenow;
 
-			snp = sname(ifile);
+			ts[0].tv_sec = Timenow.dt_sec;
+			ts[0].tv_nsec = Timenow.dt_nsec;
+			ts[1].tv_sec = ifile_mtime.tv_sec;
+			ts[1].tv_nsec = ifile_mtime.tv_nsec;
 
-			strlcpy(cfile, ifile, sizeof (cfile));
-			cfile[snp-ifile] = '\0';
-			strlcat(cfile, ",", sizeof (cfile));
-			if (strlcat(cfile, snp, sizeof (cfile)) < sizeof (cfile))
-				rename(ifile, cfile);
+			/*
+			 * As SunPro make and gmake call sccs get when the time
+			 * if s.file equals the time stamp of the g-file, make
+			 * sure the s.file is a bit older.
+			 */
+			if (!(gpkt.p_flags & PF_V6)) {
+				struct timespec	tn;
+
+				getnstimeofday(&tn);
+				ts[1].tv_nsec = tn.tv_nsec;
+			}
+			if (ts[1].tv_nsec > 500000000)
+				ts[1].tv_nsec -= 500000000;
+
+			utimensat(AT_FDCWD, gpkt.p_file, ts, 0);
+		}
+		if (HADI && *ifile) {
+			if (N.n_comma) {
+				char cfile[FILESIZE];
+				register char *snp;
+
+				snp = sname(ifile);
+
+				strlcpy(cfile, ifile, sizeof (cfile));
+				cfile[snp-ifile] = '\0';
+				strlcat(cfile, ",", sizeof (cfile));
+				if (strlcat(cfile, snp, sizeof (cfile)) <
+				    sizeof (cfile)) {
+					rename(ifile, cfile);
+				}
+			} else if(N.n_unlink) {
+				unlink(ifile);
+			} else if (N.n_get) {
+				unlink(ifile);
+				doget(gpkt.p_file, ifile, 1);
+				if (HADO)
+					dogtime(&gpkt, ifile, &ifile_mtime);
+			}
 		}
 		xrm(&gpkt);
 		uname(&un);
@@ -1468,9 +1575,12 @@ char	*afile;
 		stdin_file_buf[0] = '\0';
 	}
 	if (gpkt.p_init_path) {
-		ffree(gpkt.p_init_path);
+		if (gpkt.p_init_path != X.x_init_path)
+			ffree(gpkt.p_init_path);
 		gpkt.p_init_path = NULL;
 	}
+	sclose(&gpkt);
+	sfree(&gpkt);
 }
 
 static int
@@ -1487,7 +1597,7 @@ struct	packet	*pkt;		/* struct paket for output	*/
 	char	lastchar;
 #ifndef	RECORD_IO
 	char	*p = NULL;	/* Intialize to make gcc quiet */
-	char	*pn =  NULL;
+	char	*pn =  NULL;	/* Pointer to nul character */
 	char	line[VBUF_SIZE+1];
 	char	*lastline = line; /* Init to make GCC quiet */
 #else
@@ -1501,11 +1611,11 @@ struct	packet	*pkt;		/* struct paket for output	*/
 
 	/*
 	 * This gives the illusion that a zero-length file ends
-	 * in a newline so that it won't be mistaken for a 
+	 * in a newline so that it won't be mistaken for a
 	 * binary file.
 	 */
 	lastchar = '\n';
-	
+
 	nline = 0;
 	(void)memset(line, '\377', sizeof (line));
 #ifndef	RECORD_IO
@@ -1547,7 +1657,6 @@ struct	packet	*pkt;		/* struct paket for output	*/
 
 			if (p[1] == CTLCHAR) {
 				chkflags |= CK_CTLCHAR;
-	err:
 				if (fflag && (pkt->p_flags & PF_V6) &&
 				    (chkflags & CK_NULL) == 0) {
 					if (!warned) {
@@ -1561,6 +1670,7 @@ struct	packet	*pkt;		/* struct paket for output	*/
 					putctl(pkt);
 					continue;
 				}
+	err:
 				if (fflag) {
 					return(-1);
 				} else {
@@ -1605,7 +1715,7 @@ struct	packet	*pkt;		/* struct paket for output	*/
 		    if (fflag) {
 		       return(-1);
 		    } else {
-		       sprintf(SccsError, 
+		       sprintf(SccsError,
 			 gettext("file '%s' contains illegal data on line %jd (ad21)"),
 			 file, (intmax_t)nline);
 		       fatal(SccsError);
@@ -1624,7 +1734,7 @@ struct	packet	*pkt;		/* struct paket for output	*/
 		    ibase += idx;
 		 }
 	      }
-	   }   
+	   }
 	   if (check_id && pkt->p_did_id == 0) {
 		pkt->p_did_id =
 			chkid(line, flag_p[IDFLAG - 'a'], flag_p);
@@ -1640,7 +1750,7 @@ struct	packet	*pkt;		/* struct paket for output	*/
 
 	if (chkflags & CK_NONL) {
 #ifndef	RECORD_IO
-		if (pn && nline == 0)	/* Found null byte but no newline */
+		if (pn)		/* Found null byte but no newline past null */
 			goto err;
 #endif
 		if (fflag && (pkt->p_flags & PF_V6)) {
@@ -1648,6 +1758,10 @@ struct	packet	*pkt;		/* struct paket for output	*/
 			fseek(pkt->p_xiop, ioff + soff, SEEK_SET);
 
 			putctlnnl(pkt);
+			/*
+			 * Write again already written text without recomputing
+			 * the checksum for this part of the text.
+			 */
 			while ((idx =
 			    fread(line, 1, sizeof (line) - 1, inptr)) > 0) {
 				if (fwrite(line, 1, idx, pkt->p_xiop) <= 0)
@@ -1686,7 +1800,7 @@ warnctl(file, nline)
 		*dir_name?"/":"",
 		file, (intmax_t)nline);
 }
- 
+
 static void
 clean_up()
 {
@@ -1703,7 +1817,8 @@ clean_up()
 			unlink(gpkt.p_file);
 	}
 	if (gpkt.p_init_path) {
-		ffree(gpkt.p_init_path);
+		if (gpkt.p_init_path != X.x_init_path)
+			ffree(gpkt.p_init_path);
 		gpkt.p_init_path = NULL;
 	}
 	if (!HADH) {
@@ -1779,10 +1894,10 @@ char	*line;
 	register int k;
 	register int i;
 	char	*t_unlock;
-	char	t_line[BUFSIZ];
+	char	t_line[MAXLINE];
 	char	rel[5];
 
-	t_unlock = unlock;
+	t_unlock = unlockarg;
 	while(*t_unlock) {
 		NONBLANK(t_unlock);
 		t_unlock = getval(t_unlock,rel);
@@ -1796,21 +1911,28 @@ char	*line;
 				else --i;
 			}
 			k = 0;
-			for(i = 0; i < (int) length(line); i++)
+			for(i = 0; i < (int) length(line); i++) {
 				if (line[i] == '+')
 					continue;
 				else if (k == 0 && line[i] == ' ')
 					continue;
 				else t_line[k++] = line[i];
-			if (t_line[(int) strlen(t_line) - 1] == ' ')
-				t_line[(int) strlen(t_line) - 1] = '\0';
+			}
 			t_line[k] = '\0';
+			if (k > 0 &&
+			    t_line[(int) strlen(t_line) - 1] == ' ')
+				t_line[(int) strlen(t_line) - 1] = '\0';
 			line = t_line;
 		}
 	}
-	if (length(line))
-		return(line);
-	else return(0);
+	if (length(line) == 0)
+		return (0);
+	if (line == t_line) {
+		t_unlock = fmalloc(size(line));
+		copy(line, t_unlock);
+		return (t_unlock);
+	}
+	return (line);
 }
 
 static char*
@@ -1872,7 +1994,7 @@ range(line)
 register char *line;
 {
 	register char *p;
-	char	rel[BUFSIZ];
+	char	rel[MAXLINE];
 
 	p = line;
 	while(*p) {
@@ -1897,7 +2019,7 @@ struct packet *pktp;
 
 	/* issue a warning that file is non-text */
 	if (!HADB) {
-		fprintf(stderr, "WARNING [%s%s%s]: %s",
+		fprintf(stderr, gettext("WARNING [%s%s%s]: %s"),
 			dir_name,
 			*dir_name?"/":"",
 			ifile,
@@ -1930,142 +2052,3 @@ direrror(dir, keylet)
 		dir, (char)keylet);
 	fatal(SccsError);
 }
-
-/*
- * Compute path names for the bulk enter mode that is selected by -N
- *
- * admin -Ns.		Arguments are aaa/s.xxx files with matching aaa/xxx files
- * admin -N		Arguments are aaa/xxx files, aaa/s.xxx is created
- * admin -NSCCS/s.	Arguments are aaa/SCCS/s.xxx with matching aaa/xxx files
- * admin -NSCCS		Arguments are aaa/xxx Files, aaa/SCCS/s.xxx is created
- */
-static char *
-bulkprepare(afile)
-	char	*afile;
-{
-static int	dfd = -1;
-static char	Dir[MAXPATHLEN];
-static char	Nhold[MAXPATHLEN];
-
-#ifdef	HAVE_FCHDIR
-	if (dfd < 0) {
-		dfd = open(".", O_SEARCH);	/* on failure use full path */
-	} else {
-		if (fchdir(dfd) < 0)
-			xmsg(".",NOGETTEXT("admin"));
-	}
-#endif
-
-	Dir[0] = '\0';
-	if (!Nsdot) {				/* afile is ifile name */
-		char	*sn = sname(afile);
-
-		if (exists(afile)) {		/* must exist */
-			if ((Statbuf.st_mode & S_IFMT) == S_IFDIR) {
-				direrror(afile, 'i');
-			} else {
-				ifile_mtime.tv_sec = Statbuf.st_mtime;
-				ifile_mtime.tv_nsec = stat_mnsecs(&Statbuf);
-			}
-		} else {
-			xmsg(afile,NOGETTEXT("admin"));
-		}
-		if (sn == afile) {		/* No dir for short names */
-			Dir[0] = '\0';
-		} else if (*afile == '/' && &afile[1] == sn) {
-			copy("/", Dir);
-		} else {			/* Get dir part from afile */
-			size_t	len = sn - afile;
-			if (len > sizeof (Dir))
-				len = sizeof (Dir);
-			strlcpy(Dir, afile, len); /* replace last '/' by '\0' */
-			/*
-			 * We need a path free of symlink components.
-			 * resolvepath() is available as syscall on Solaris,
-			 * or as user space implementation in libschily.
-			 */
-			if ((len = resolvepath(Dir, Dir, sizeof (Dir))) == -1)
-				efatal(gettext("path conversion error (cm12)"));
-			else if (len >= sizeof (Dir))
-				fatal(gettext("resolved path too long (cm11)"));
-		}
-		if (Dir[0] != '\0' && (dfd < 0 || chdir(Dir) < 0)) {
-			cat(Nhold, Dir, *Dir?"/":"", Nprefix, sn, (char *)0);
-			ifile = afile;
-		} else {					/* Did chdir  */
-			cat(Nhold, Nprefix, sn, (char *)0); /* use short name */
-			ifile = sn;
-			dir_name = Dir;
-			if (dir_name[0] == '.' && dir_name[1] == '/')
-				dir_name += 2;
-		}
-		afile = Nhold;			/* Use computed s.file name */
-
-	} else {				/* afile is s.file name */
-		char	*np;
-		size_t	plen = strlen(Nprefix);
-
-		if (!sccsfile(afile))
-			fatal(gettext("not an SCCS file (co1)"));
-
-		np = sname(afile);
-		np = np + 2 - plen;
-		if (np < afile ||
-		    ((np > afile) && (np[-1] != '/')) ||
-		    strncmp(np, Nprefix, plen) != 0)
-			fatal(gettext("not in specified sub directory (ad37)"));
-
-		if (np > afile) {
-			np[-1] = '\0';
-			if (dfd >= 0) {
-				int	len;
-
-				if (afile[0] == '\0')
-					copy("/", Dir);
-				else
-					copy(afile, Dir);
-
-				if ((len = resolvepath(Dir, Dir, sizeof (Dir))) == -1)
-					efatal(gettext("path conversion error (cm12)"));
-				else if (len >= sizeof (Dir))
-					fatal(gettext("resolved path too long (cm11)"));
-
-				if (chdir(Dir) < 0)
-					efatal("Chdir");
-				dir_name = Dir;
-				if (dir_name[0] == '.' && dir_name[1] == '/')
-					dir_name += 2;
-				afile = np;
-				copy(auxf(np,'g'), Nhold);
-			} else {
-				cat(Nhold, afile, "/", auxf(np,'g'), (char *)0);
-			}
-			np[-1] = '/';
-		} else {
-			copy(auxf(np,'g'), Nhold);
-		}
-		ifile = Nhold;
-
-		if (exists(ifile)) {
-			if ((Statbuf.st_mode & S_IFMT) == S_IFDIR) {
-				direrror(ifile, 'i');
-			} else {
-				ifile_mtime.tv_sec = Statbuf.st_mtime;
-				ifile_mtime.tv_nsec = stat_mnsecs(&Statbuf);
-			}
-		} else {
-			xmsg(afile,NOGETTEXT("admin"));
-		}
-	}
-	if (Nsubd) {			/* Want to put s.file in subdir */
-		char	dbuf[MAXPATHLEN];
-
-		copy (afile, dbuf);	/* Copy as dname() modifies arg */
-		dname(dbuf);
-		if (!exists(dbuf)) {
-			if (mkdir(dbuf, 0777) < 0)
-				xmsg(dbuf, NOGETTEXT("admin"));
-		}
-	}
-	return (afile);
-}
