@@ -3,7 +3,7 @@
  *
  * The contents of this file are subject to the terms of the
  * Common Development and Distribution License ("CDDL"), version 1.0.
- * You may only use this file in accordance with the terms of version
+ * You may use this file only in accordance with the terms of version
  * 1.0 of the CDDL.
  *
  * A full copy of the text of the CDDL should have accompanied this
@@ -27,12 +27,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2014 J. Schilling
+ * Copyright 2006-2018 J. Schilling
  *
- * @(#)delta.c	1.61 14/08/09 J. Schilling
+ * @(#)delta.c	1.85 18/12/17 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)delta.c 1.61 14/08/09 J. Schilling"
+#pragma ident "@(#)delta.c 1.85 18/12/17 J. Schilling"
 #endif
 /*
  * @(#)delta.c 1.40 06/12/12
@@ -60,33 +60,41 @@
 # define	LENMR	60
 
 static FILE	*Diffin, *Gin;
+static Nparms	N;			/* Keep -N parameters		*/
+static Xparms	X;			/* Keep -X parameters		*/
 static struct packet	gpkt;
 static struct utsname	un;
 static int	num_files;
 static int	number_of_lines;
 static off_t	size_of_file;
 static off_t	Szqfile;
-#ifdef	PROTOTYPES
-static char	BDiffpgm[]  =   NOGETTEXT(INS_BASE "/ccs/bin/" "bdiff");
+static off_t	Checksum_offset;
+#if	defined(PROTOTYPES) && defined(INS_BASE)
+static char	BDiffpgmp[]  =   NOGETTEXT(INS_BASE "/" SCCS_BIN_PRE "bin/" "bdiff");
 #else
 /*
  * XXX If you are using a K&R compiler and like to install to a path
  * XXX different from "/usr/ccs/bin/", you need to edit this string.
  */
-static char	BDiffpgm[]  =   NOGETTEXT("/usr/ccs/bin/bdiff");
+static char	BDiffpgmp[]  =   NOGETTEXT("/usr/ccs/bin/bdiff");
 #endif
-static char	BDiffpgm2[]  =   NOGETTEXT("bdiff");
+static char	BDiffpgm[]  =   NOGETTEXT("/usr/bin/bdiff");
+static char	BDiffpgm2[]  =   NOGETTEXT("/bin/bdiff");
 #if	defined(PROTOTYPES) && defined(INS_BASE)
-static char	Diffpgmp[]  =   NOGETTEXT(INS_BASE "/ccs/bin/" "diff");
+static char	Diffpgmp[]  =   NOGETTEXT(INS_BASE  "/" SCCS_BIN_PRE "bin/" "diff");
+#else
+static char	Diffpgmp[]  =   NOGETTEXT("/usr/ccs/bin/diff");
 #endif
 static char	Diffpgm[]   =   NOGETTEXT("/usr/bin/diff");
 static char	Diffpgm2[]   =   NOGETTEXT("/bin/diff");
+static char	*diffpgm = "";
 static char	*ilist, *elist, *glist, Cmrs[300], *Nsid;
 static char	Pfilename[FILESIZE];
 static char	*uuname;
 static char	*Cwd = "";
+static char	*Dfilename;
 
-static	time_t	gfile_mtime;
+static struct timespec	gfile_mtime;	/* Timestamp for -o		*/
 static	time_t	cutoff = MAX_TIME;
 
 static struct sid sid;
@@ -96,14 +104,15 @@ static	void	enter	__PR((struct packet *pkt, int ch, int n, struct sid *sidp));
 
 	int	main __PR((int argc, char **argv));
 static void	delta __PR((char *file));
-static int	mkdelt __PR((struct packet *pkt, struct sid *sp, struct sid *osp, int diffloop, int orig_nlines));
+static int	mkdelt __PR((struct packet *pkt, struct sid *sp, struct sid *osp,
+				int diffloop, int orig_nlines));
 static void	mkixg __PR((struct packet *pkt, int reason, int ch));
 static void	putmrs __PR((struct packet *pkt));
 static void	putcmrs __PR((struct packet *pkt));
 static struct pfile *rdpfile __PR((struct packet *pkt, struct sid *sp));
 static FILE *	dodiff __PR((char *newf, char *oldf, int difflim));
 static int	getdiff __PR((char *type, int *plinenum));
-static void	insert __PR((struct packet *pkt, int linenum, int n, int ser));
+static void	insert __PR((struct packet *pkt, int linenum, int n, int ser, int off));
 static void	delete __PR((struct packet *pkt, int linenum, int n, int ser));
 static void	after __PR((struct packet *pkt, int n));
 static void	before __PR((struct packet *pkt, int n));
@@ -112,6 +121,7 @@ static void	skipline __PR((char *lp, int num));
 static char *	rddiff __PR((char *s, int n));
 static void	fgetchk __PR((char *file, struct packet *pkt));
 static void	warnctl __PR((char *file, off_t nline));
+static void 	fixghash __PR((struct packet *pkt, int ser));
 
 extern int	org_ihash;
 extern int	org_chash;
@@ -140,7 +150,7 @@ register char *argv[];
 	 */
 #ifdef	PROTOTYPES
 	(void) bindtextdomain(NOGETTEXT("SUNW_SPRO_SCCS"),
-	   NOGETTEXT(INS_BASE "/ccs/lib/locale/"));
+	   NOGETTEXT(INS_BASE "/" SCCS_BIN_PRE "lib/locale/"));
 #else
 	(void) bindtextdomain(NOGETTEXT("SUNW_SPRO_SCCS"),
 	   NOGETTEXT("/usr/ccs/lib/locale/"));
@@ -178,7 +188,7 @@ register char *argv[];
 			}
 			no_arg = 0;
 			i = current_optind;
-		        c = getopt(argc, argv, "-r:dpsnm:g:y:fhoqzC:V(version)");
+		        c = getopt(argc, argv, "()-r:dpsnm:g:y:fhoqkzC:D:N:X:V(version)");
 
 				/* this takes care of options given after
 				** file names.
@@ -235,6 +245,7 @@ register char *argv[];
 				}
 				break;
 			case 'h': /* allow diffh for large files (NSE only) */
+			case 'k': /* get(1) without keyword expand */
 			case 'o': /* use original file date */
                                 break;
                         case 'q': /* enable NSE mode */
@@ -251,9 +262,29 @@ register char *argv[];
 			case 'C':
 				Cwd = p;
 				break;
+			case 'D':
+				Dfilename = p;
+				break;
+
+			case 'N':	/* Bulk names */
+				initN(&N);
+				if (optarg == argv[i+1]) {
+				   no_arg = 1;
+				   break;
+				}
+				N.n_parm = p;
+				break;
+
+			case 'X':
+				X.x_parm = optarg;
+				X.x_flags = XO_PREPEND_FILE;
+				if (!parseX(&X))
+					goto err;
+				break;
 
 			case 'V':		/* version */
-				printf("delta %s-SCCS version %s %s (%s-%s-%s)\n",
+				printf(gettext(
+				    "delta %s-SCCS version %s %s (%s-%s-%s)\n"),
 					PROVIDER,
 					VERSION,
 					VDATE,
@@ -261,7 +292,8 @@ register char *argv[];
 				exit(EX_OK);
 
 			default:
-				fatal(gettext("Usage: delta [ -dnops ][ -g sid-list ][ -m mr-list ]\n\t[ -r SID ][ -y[comment] ] s.filename... "));
+			err:
+				fatal(gettext("Usage: delta [ -dknops ][ -g sid-list ][ -m mr-list ][ -r SID ]\n\t[ -y[comment] ][ -D diff-file ][ -N[bulk-spec]][ -Xxopts ] s.filename..."));
 			}
 
 			/*
@@ -291,16 +323,28 @@ register char *argv[];
  	   fatal(gettext("missing file arg (cm3)"));
  	   exit(2);
  	}
+
 	setsig();
+	xsethome(NULL);
+	if (HADUCN) {					/* Parse -N args  */
+		parseN(&N);
+
+		if (N.n_sdot && (sethomestat & SETHOME_OFFTREE))
+			fatal(gettext("-Ns. not supported in off-tree project mode"));
+	}
+
 	Fflags &= ~FTLEXIT;
 	Fflags |= FTLJMP;
 	for (i=1; i<argc; i++)
 		if ((p=argv[i]) != NULL)
-			do_file(p, delta, 1, 1);
+			do_file(p, delta, 1, N.n_sdot);
 
 	return (Fcnt ? 1 : 0);
 }
 
+/*
+ * Create the delta for one file
+ */
 static void
 delta(file)
 char *file;
@@ -311,12 +355,13 @@ char *file;
 	char type;
 	register int ser;
 	extern char had_dir, had_standinp;
-	char nsid[50];
+	char nsid[SID_STRSIZE];
 	char dfilename[FILESIZE];
 	char gfilename[FILESIZE];
-	char line[BUFSIZ];
+	char *ifile;
+	char line[BUFSIZ];		/* Buffer for uuencoded IO, see below */
 	struct stats stats;
-	struct pfile *pp;
+	struct pfile *pp = NULL;
 	struct stat sbuf;
 	int inserted, deleted, orig;
 	int newser;
@@ -331,18 +376,32 @@ char *file;
 	if (setjmp(Fjmp))
 		return;
 
+	if (HADUCN) {
+		char	*ofile = file;
+
+		file = bulkprepare(&N, file);
+		if (file == NULL) {
+			if (N.n_ifile)
+				ofile = N.n_ifile;
+			fatal(gettext("directory specified as s-file (cm14)"));
+		}
+		ifile = N.n_ifile;
+	} else {
+		ifile = NULL;
+	}
+
 	/*
 	 * Init and check for validity of file name but do not open the file.
 	 * This prevents us from potentially damaging files with lockit().
 	 */
-	sinit(&gpkt, file, 0);
+	sinit(&gpkt, file, SI_INIT);
 
 	uname(&un);
 	uuname = un.nodename;
 	if (lockit(auxf(gpkt.p_file,'z'),SCCS_LOCK_ATTEMPTS,getpid(),uuname))
 		efatal(gettext("cannot create lock file (cm4)"));
 
-	sinit(&gpkt,file,1);
+	sinit(&gpkt, file, SI_OPEN);
 	gpkt.p_enter = enter;
 	if ((gpkt.p_flags & PF_V6) == 0)
 		gpkt.p_flags |= PF_GMT;
@@ -352,29 +411,24 @@ char *file;
 	}
 	gpkt.p_reopen = 1;
 	gpkt.p_stdout = stdout;
-	cat(gfilename,Cwd,auxf(gpkt.p_file,'g'), (char *)0);
+	gfilename[0] = '\0';
+	strlcatl(gfilename, sizeof (gfilename),
+		Cwd,
+		ifile ? ifile :
+			auxf(gpkt.p_file, 'g'),
+		(char *)0);
 	Gin = xfopen(gfilename, O_RDONLY|O_BINARY);
 #ifdef	USE_SETVBUF
 	setvbuf(Gin, NULL, _IOFBF, VBUF_SIZE);
 #endif
-	pp = rdpfile(&gpkt,&sid);
+	Pfilename[0] = '\0';
+	if (!HADF || exists(auxf(gpkt.p_file,'p')))
+		pp = rdpfile(&gpkt, &sid);
 
-	if((pp->pf_cmrlist)==0)
-		Cmrs[0] = 0;
-	else
+	Cmrs[0] = 0;
+	if (pp != NULL && pp->pf_cmrlist != NULL)
 		strlcpy(Cmrs, pp->pf_cmrlist, sizeof (Cmrs));
 
-	if(!pp->pf_nsid.s_br) {
-		sprintf(nsid, NOGETTEXT("%d.%d"),
-		  pp->pf_nsid.s_rel, pp->pf_nsid.s_lev);
-	}
-	else {
-		sprintf(nsid, NOGETTEXT("%d.%d.%d.%d"),
-		  pp->pf_nsid.s_rel, pp->pf_nsid.s_lev,
-		  pp->pf_nsid.s_br,  pp->pf_nsid.s_seq);
-	}
-	Nsid=nsid;
-
 	if (dodelt(&gpkt,&stats,(struct sid *) 0,0) == 0)
 		fmterr(&gpkt);
 
@@ -386,11 +440,11 @@ char *file;
                 gpkt.p_reqsid.s_lev = 0;
                 gpkt.p_reqsid.s_br = 0;
                 gpkt.p_reqsid.s_seq = 0;
-                gpkt.p_cutoff = time(0);
+                gpkt.p_cutoff = cutoff;
                 ilist = 0;
                 elist = 0;
                 ser = getser(&gpkt);
-                newsid(&gpkt, 0);
+                newsid(&gpkt, 0);	/* sets gpkt.p_reqsid */
         } else {        
                 gpkt.p_cutoff = pp->pf_date;
                 ilist = pp->pf_ilist;
@@ -400,6 +454,9 @@ char *file;
                                 fatal(gettext("invalid sid in p-file (de3)"));
 		gpkt.p_reqsid = pp->pf_nsid;
         }
+	sid_ba(&gpkt.p_reqsid, nsid);
+	Nsid=nsid;
+	gfile_mtime.tv_sec = gfile_mtime.tv_nsec = 0;
         if ((HADO || HADQ) && stat(gfilename, &sbuf) == 0) {
                 /*
 		 * When specifying -o (original date) and when
@@ -410,16 +467,18 @@ char *file;
 		 * In non-original date mode, sccs is vulnerable to a mis-set of
 		 * the local clock while calling 'delta'.
                  */
-                gfile_mtime = sbuf.st_mtime;
+                gfile_mtime.tv_sec = sbuf.st_mtime;
+                gfile_mtime.tv_nsec = stat_mnsecs(&sbuf);
 	}
 
 	doie(&gpkt,ilist,elist,glist);
 	setup(&gpkt,ser);
 	finduser(&gpkt);
 	doflags(&gpkt);
-	gpkt.p_reqsid = pp->pf_nsid;
 	permiss(&gpkt);
-	flushto(&gpkt,EUSERTXT,1);
+	donamedflags(&gpkt);
+	dometa(&gpkt);
+	flushto(&gpkt, EUSERTXT, FLUSH_NOCOPY);
 	gpkt.p_chkeof = 1;
 	/* if encode flag is set, encode the g-file before diffing it
 	 * with the s.file
@@ -438,27 +497,39 @@ char *file;
 		setvbuf(Gin, NULL, _IOFBF, VBUF_SIZE);
 #endif
 	}
-	copy(auxf(gpkt.p_file,'d'),dfilename);
-	gpkt.p_gout = xfcreat(dfilename,(mode_t)0444);
+
+	dfilename[0] = '\0';
+	if ((X.x_opts & XO_PREPEND_FILE) == 0) {
+		copy(auxf(gpkt.p_file,'d'),dfilename);
+		gpkt.p_gout = xfcreat(dfilename,(mode_t)0444);
 #ifdef	USE_SETVBUF
-	setvbuf(gpkt.p_gout, NULL, _IOFBF, VBUF_SIZE);
+		setvbuf(gpkt.p_gout, NULL, _IOFBF, VBUF_SIZE);
 #endif
-	while(readmod(&gpkt)) {
-		if (gpkt.p_flags & PF_NONL)
-			gpkt.p_line[gpkt.p_line_length-1] = '\0';
-		if(fputs(gpkt.p_lineptr, gpkt.p_gout) == EOF)
+		while(readmod(&gpkt)) {
+			if (gpkt.p_flags & PF_NONL)
+				gpkt.p_line[gpkt.p_line_length-1] = '\0';
+			if(fputs(gpkt.p_lineptr, gpkt.p_gout) == EOF)
+				xmsg(dfilename, NOGETTEXT("delta"));
+		}
+		if (fflush(gpkt.p_gout) == EOF)
 			xmsg(dfilename, NOGETTEXT("delta"));
-	}
-	if (fflush(gpkt.p_gout) == EOF)
-		xmsg(dfilename, NOGETTEXT("delta"));
 #ifdef	HAVE_FSYNC
-	if (fsync(fileno(gpkt.p_gout)) < 0)
-		xmsg(dfilename, NOGETTEXT("delta"));
+		if (fsync(fileno(gpkt.p_gout)) < 0)
+			xmsg(dfilename, NOGETTEXT("delta"));
 #endif
-	if (fclose(gpkt.p_gout) == EOF)
-		xmsg(dfilename, NOGETTEXT("delta"));
-	gpkt.p_gout = NULL;
-	orig = gpkt.p_glnno;
+		if (fclose(gpkt.p_gout) == EOF)
+			xmsg(dfilename, NOGETTEXT("delta"));
+		gpkt.p_gout = NULL;
+		orig = gpkt.p_glnno;
+	} else {
+		/*
+		 * We don't count the previous version and we
+		 * do not have real diffs, so we need to use
+		 * an estimation.
+		 */
+		orig = stats.s_ins + stats.s_unc - stats.s_del;
+	}
+
 	gpkt.p_glnno = 0;
 	gpkt.p_verbose = (HADS) ? 0 : 1;
 	gpkt.p_did_id = 0;
@@ -471,9 +542,16 @@ char *file;
 		gpkt.p_did_id = 1;	/* No need to scan for keywds */
 	}
 	if ((gpkt.p_encoding & EF_UUENCODE) == 0) {
+		/*
+		 * Compute the checksum and scan for unsupported characters.
+		 * This method supports nul bytes.
+		 */
 		fgetchk(gfilename, &gpkt);
 		number_of_lines = gpkt.p_glines;
 	} else {
+		/*
+		 * This does not support nul bytes, but this is uuencoded text.
+		 */
 	 	while (fgets(line,sizeof(line),Gin) != NULL) {
 			register int	len = strlen(line);
 			register char	**sflags = gpkt.p_sflags;
@@ -493,9 +571,11 @@ char *file;
 	if (stat(gfilename, &Statbuf) == 0) {
 		size_of_file = Statbuf.st_size;
 	}
-	if (Gin)
-		fclose(Gin);
-	Gin = NULL;
+	if ((X.x_opts & XO_PREPEND_FILE) == 0) {
+		if (Gin)
+			fclose(Gin);
+		Gin = NULL;
+	}
 	if (gpkt.p_verbose && (num_files > 1 || had_dir || had_standinp))
  	   fprintf(gpkt.p_stdout,"\n%s:\n",gpkt.p_file);
  	if (HADD != 0) {
@@ -531,16 +611,42 @@ char *file;
 	*/
 	difflim = 24000;
 	diffloop = 0;
-	ghash = gpkt.p_ghash;		/* Save ghash value */
+	ghash = gpkt.p_ghash;			/* Save ghash value */
+
+	if (X.x_opts & XO_PREPEND_FILE) {
+		int	oihash = gpkt.p_ihash;	/* Remember hash from sinit() */
+
+		grewind(&gpkt);
+		gpkt.do_chksum = 1;		/* No old g-file, do it now */
+		gpkt.p_ihash = oihash;		/* Restore hash */
+		if (gpkt.p_flags & PF_V6) {
+			/*
+			 * We do not read the "current" file as it does not yet
+			 * exist, but we compute the sum of the hashes from the
+			 * old file and the new beginning that is a file with
+			 * the name of the g-file.
+			 */
+			if (gpkt.p_hash != NULL) {
+				ghash += gpkt.p_hash[ser];
+				ghash &= 0xFFFF;
+			}
+		}
+	}
+
 	/*CONSTCOND*/
 	while (1) {
 		inserted = deleted = 0;
 		gpkt.p_glnno = 0;
 		gpkt.p_upd = 1;
 		gpkt.p_wrttn = 1;
-		getline(&gpkt);
+		getline(&gpkt);		/* Read the magic line */
+		gpkt.p_chash = 0; 	/* Reset signed hash */
+		gpkt.p_uchash = 0; 	/* Reset unsigned hash */
 		gpkt.p_wrttn = 1;
 		gpkt.p_ghash = ghash;	/* ghash may be destroyed in loop */
+		if (glist && gpkt.p_flags & PF_V6) {
+			gpkt.p_ghash = 0; /* write 00000 before correcting */
+		}
         	if (HADF) {
                 	newser = mkdelt(&gpkt, &gpkt.p_reqsid, &gpkt.p_gotsid, 
 				diffloop, orig);
@@ -549,21 +655,34 @@ char *file;
 				diffloop, orig);
         	}
 		diffloop = 1;
-		flushto(&gpkt,EUSERTXT,0);
-		if (gpkt.p_encoding & EF_UUENCODE)
-			Diffin = dodiff(auxf(gpkt.p_file,'e'),dfilename,difflim);
-		else
-			Diffin = dodiff(gfilename, dfilename, difflim);
+		flushto(&gpkt, EUSERTXT, FLUSH_COPY);
 
-		type = 0;				/* Make GCC quiet */
-		while ((n = getdiff(&type,&linenum)) != 0) {
-			if (type == INS) {
-				inserted += n;
-				insert(&gpkt,linenum,n,newser);
+		if (X.x_opts & XO_PREPEND_FILE) {
+			/*
+			 * Since we do not call "diff", we come here only once.
+			 */
+			Diffin = Gin;
+			rewind(Diffin);
+			Gin = NULL;
+			inserted += number_of_lines;
+			insert(&gpkt, 0, number_of_lines, newser, 0);
+			status = 0;		/* Causes a break from while */
+		} else {
+			if (gpkt.p_encoding & EF_UUENCODE) {
+				Diffin = dodiff(auxf(gpkt.p_file,'e'),
+						dfilename, difflim);
+			} else {
+				Diffin = dodiff(gfilename, dfilename, difflim);
 			}
-			else {
-				deleted += n;
-				delete(&gpkt,linenum,n,newser);
+			type = 0;			/* Make GCC quiet */
+			while ((n = getdiff(&type,&linenum)) != 0) {
+				if (type == INS) {
+					inserted += n;
+					insert(&gpkt, linenum , n, newser, 1);
+				} else {
+					deleted += n;
+					delete(&gpkt,linenum,n,newser);
+				}
 			}
 		}
 		if (Diffin)
@@ -572,20 +691,18 @@ char *file;
 		if (gpkt.p_iop)
 			while (readmod(&gpkt))
 				;
-		wait(&status);
+		if ((X.x_opts & XO_PREPEND_FILE) == 0)
+			wait(&status);
  		/*
  		 Check top byte (exit code of child).
  		*/
 
  		/*
- 		 * The logical shift operator construct ">>" below
- 		 * does not violate i18n rules, since "status" is
- 		 * an internally used variable and is not used
- 		 * for i/o.
+		 * 32 is the exit code below after a failed execlp() call.
  		 */
- 		if (((status >> 8) & 0377) == 32) { /* 'execl' failed */
+ 		if (WEXITSTATUS(status) == 32) { /* 'execl' failed */
  		   sprintf(SccsError,
- 		      gettext("cannot execute '%s' (de12)"), BDiffpgm);
+ 		      gettext("cannot execute '%s' (de12)"), diffpgm);
  		   fatal(SccsError);
  		}
  		if ((status != 0) && (HADD == 0)) { /* diff failed */
@@ -595,15 +712,13 @@ char *file;
 			if (difflim -= 3000) {	/* reduce segmentation */
 				fprintf(stderr,
 					gettext("'%s' failed, re-trying, segmentation = %d (de13)\n"),
- 					BDiffpgm,
+ 					diffpgm,
 					difflim);
-				if (gpkt.p_xiop)
-					fclose(gpkt.p_xiop); /* set up */
-				gpkt.p_xiop = 0;	/* for new x-file */
-				gpkt.p_xcreate = 0;
+				xrm(&gpkt);		/* Close x-file */
 				/*
-				Re-open s-file.
-				*/
+				 * Re-open s-file.
+				 */
+				sclose(&gpkt);
 				gpkt.p_iop = xfopen(gpkt.p_file, O_RDONLY|O_BINARY);
 #ifdef	USE_SETVBUF
 				setvbuf(gpkt.p_iop, NULL, _IOFBF, VBUF_SIZE);
@@ -631,15 +746,25 @@ TRANSLATION_NOTE
 command, to check the differences found between two files.
 */
 				fatal(gettext("diff failed (de4)"));
-		}
-		else {		/* no need to try again, worked */
+		} else {		/* no need to try again, worked */
 			break;			/* exit while loop */
 		}
 	}
 	if (gpkt.p_encoding & EF_UUENCODE) {
 		unlink(auxf(gpkt.p_file,'e'));
 	}
-	unlink(dfilename);
+	if (dfilename[0]) {
+		unlink(dfilename);
+		dfilename[0] = '\0';
+	}
+	/*
+	 * If we had a glist, the checked out file will differ from the checked
+	 * in file. Check out the real new content and recompute + correct the
+	 * ghash.
+	 */
+	if (glist && gpkt.p_flags & PF_V6)
+		fixghash(&gpkt, newser);
+
 	stats.s_ins = inserted;
 	stats.s_del = deleted;
 	stats.s_unc = orig - deleted;
@@ -655,7 +780,32 @@ command, to check the differences found between two files.
 
 	chown(gpkt.p_file, (unsigned int)sbuf.st_uid,
 			(unsigned int)sbuf.st_gid);
-	if (!HADF) {
+	if (HADO) {
+		struct timespec	ts[2];
+		extern dtime_t	Timenow;
+
+		ts[0].tv_sec = Timenow.dt_sec;
+		ts[0].tv_nsec = Timenow.dt_nsec;
+		ts[1].tv_sec = gfile_mtime.tv_sec;
+		ts[1].tv_nsec = gfile_mtime.tv_nsec;
+
+		/*
+		 * As SunPro make and gmake call sccs get when the time
+		 * if s.file equals the time stamp of the g-file, make
+		 * sure the s.file is a bit older.
+		 */
+		if (!(gpkt.p_flags & PF_V6)) {
+			struct timespec	tn;
+
+			getnstimeofday(&tn);
+			ts[1].tv_nsec = tn.tv_nsec;
+		}
+		if (ts[1].tv_nsec > 500000000)
+			ts[1].tv_nsec -= 500000000;
+
+		utimensat(AT_FDCWD, gpkt.p_file, ts, 0);
+	}
+	if (!HADF || Pfilename[0] != '\0') {
 		char	*qfile;
 		
 		if (exists(qfile = auxf(gpkt.p_file, 'q'))) {
@@ -668,6 +818,7 @@ command, to check the differences found between two files.
 			xunlink(Pfilename);
 		}
 	}
+	sclose(&gpkt);
 	clean_up();
 	if (!HADN) {
 		fflush(gpkt.p_stdout);
@@ -676,11 +827,19 @@ command, to check the differences found between two files.
 		setuid(getuid());
 		setgid(getgid());
 		unlink(gfilename);
+		if (N.n_get) {
+			doget(gpkt.p_file, gfilename, newser);
+			if (HADO)
+				dogtime(&gpkt, gfilename, &gfile_mtime);
+		}
 		setuid(holduid);
 		setgid(holdgid);
 	}
 }
 
+/*
+ * Make the delta table for the current file
+ */
 static int
 mkdelt(pkt,sp,osp,diffloop,orig_nlines)
 struct packet *pkt;
@@ -690,7 +849,7 @@ int orig_nlines;
 {
 	extern dtime_t Timenow;
 	struct deltab dt;
-	char str[BUFSIZ];
+	char str[max(BUFSIZ, SID_STRSIZE)];	/* Buffer for delta table IO */
 	int newser;
 	register char *p;
 	int ser_inc, opred, nulldel;
@@ -747,8 +906,8 @@ int orig_nlines;
          * supported), the delta time is set to be the mtime of the clear
          * file.
          */
-        if ((HADO || HADQ) && (gfile_mtime != 0)) {
-		time2dt(&dt.d_dtime, gfile_mtime, 0);
+        if ((HADO || HADQ) && (gfile_mtime.tv_sec != 0)) {
+		time2dt(&dt.d_dtime, gfile_mtime.tv_sec, gfile_mtime.tv_nsec);
         }
 	strncpy(dt.d_pgmr,logname(),LOGSIZE-1);
 	dt.d_type = 'D';
@@ -784,8 +943,10 @@ int orig_nlines;
 		}
 		putcmrs(pkt); /* this routine puts cmrs on the out put file */
 	}
-	if (pkt->p_flags & PF_V6)
+	if (pkt->p_flags & PF_V6) {
+		Checksum_offset = ftell(gpkt.p_xiop);
 		sidext_ba(pkt, &dt);
+	}
 
 	sprintf(str, NOGETTEXT("%c%c "), CTLCHAR, COMMENTS);
 	putline(pkt,str);
@@ -820,6 +981,9 @@ int orig_nlines;
 	return(newser);
 }
 
+/*
+ * Write include/exclude/ignore table for delta table
+ */
 static void
 mkixg(pkt,reason,ch)
 struct packet *pkt;
@@ -827,7 +991,7 @@ int reason;
 char ch;
 {
 	int n;
- 	char str[BUFSIZ];
+ 	char str[BUFSIZ];	/* Only limits the size of a single entry */
 
 	sprintf(str, NOGETTEXT("%c%c"), CTLCHAR, ch);
 	putline(pkt,str);
@@ -875,6 +1039,9 @@ struct packet *pkt;
 
 static char ambig[] = NOGETTEXT("ambiguous `r' keyletter value (de15)");
 
+/*
+ * Read the p-file and write a new version as q-file.
+ */
 static struct pfile *
 rdpfile(pkt,sp)
 register struct packet *pkt;
@@ -883,7 +1050,7 @@ struct sid *sp;
 	char *user;
 	struct pfile pf;
 	static struct pfile goodpf;
-	char line[BUFSIZ];
+	char line[BUFSIZ];		/* Limits the line length of a p-file */
 	int cnt, uniq, fd;
 	FILE *in, *out;
 	char *outname;
@@ -966,7 +1133,9 @@ struct sid *sp;
 	return(&goodpf);
 }
 
-
+/*
+ * Open a FILE * to the diff output.
+ */
 static FILE *
 dodiff(newf,oldf,difflim)
 char *newf, *oldf;
@@ -979,6 +1148,8 @@ int difflim;
 	char num[10];
 	struct rlimit	rlim;
 
+	if (HADUCD)
+		return  (xfopen(Dfilename, O_RDONLY|O_BINARY));
 	xpipe(pfd);
 	if ((i = vfork()) < 0) {
 		int	errsav = errno;
@@ -1022,13 +1193,22 @@ int difflim;
 		sprintf(num, NOGETTEXT("%d"), difflim);
  		if (HADD) {
 #if	defined(PROTOTYPES) && defined(INS_BASE)
+		   diffpgm = Diffpgmp;
  		   execl(Diffpgmp,Diffpgmp,oldf,newf, (char *)0);
 #endif
+		   diffpgm = Diffpgm;
  		   execl(Diffpgm,Diffpgm,oldf,newf, (char *)0);
+		   diffpgm = Diffpgm2;
  		   execl(Diffpgm2,Diffpgm2,oldf,newf, (char *)0);
  		} else {
+#if	defined(PROTOTYPES) && defined(INS_BASE)
+		   diffpgm = BDiffpgmp;
+ 		   execl(BDiffpgmp,BDiffpgmp,oldf,newf,num,"-s", (char *)0);
+#endif
+		   diffpgm = BDiffpgm;
  		   execl(BDiffpgm,BDiffpgm,oldf,newf,num,"-s", (char *)0);
- 		   execl(BDiffpgm2,BDiffpgm,oldf,newf,num,"-s", (char *)0);
+		   diffpgm = BDiffpgm2;
+ 		   execl(BDiffpgm2,BDiffpgm2,oldf,newf,num,"-s", (char *)0);
  		}
 		close(1);
 		_exit(32);	/* tell parent that 'execl' failed */
@@ -1042,7 +1222,10 @@ int difflim;
 	return (0);	/* fake for gcc */
 }
 
-
+/*
+ * Parse the part of the diff output that contains the line numbers and the
+ * delete/append/change instructions.
+ */
 static int
 getdiff(type,plinenum)
 register char *type;
@@ -1093,10 +1276,15 @@ register int *plinenum;
 	return(num_lines);
 }
 
+/*
+ * Skip the next chunk of kept lines from the old file and then
+ * insert the new lines from the diff output.
+ */
 static void
-insert(pkt, linenum, n, ser)
+insert(pkt, linenum, n, ser, off)
 struct	packet	*pkt;
 int	linenum, n, ser;
+	int	off;
 {
  	char	str[BUFSIZ];
  	int 	first;
@@ -1115,6 +1303,8 @@ int	linenum, n, ser;
 	after(pkt, linenum);
 	sprintf(str, NOGETTEXT("%c%c %d\n"), CTLCHAR, INS, ser);
 	putline(pkt, str);
+	if (off)
+		off = 2;			/* strlen("> ") from diff */
 	while (--n >= 0) {
  		first = 1;
  		for (;;) {			/* Loop over partial line */
@@ -1125,9 +1315,9 @@ int	linenum, n, ser;
 				first = 0;
 				if (n == 0 && nonl) {	/* No newline at end */
 					putctlnnl(pkt);	/* ^AN escape	    */
-				} else if (str[2] == CTLCHAR) /* ^A escape? */
+				} else if (str[off] == CTLCHAR) /* ^A escape? */
 					putctl(pkt);
-				putline(pkt, str+2);	/* Skip diff's "> " */
+				putline(pkt, str+off);	/* Skip diff's "> " */
 			} else {
 				putline(pkt, str);
 			}
@@ -1140,12 +1330,15 @@ int	linenum, n, ser;
 	putline(pkt, str);
 }
 
+/*
+ * Add delete markers in the weave data.
+ */
 static void
 delete(pkt, linenum, n, ser)
 struct	packet	*pkt;
 int	linenum, n, ser;
 {
-	char str[BUFSIZ];
+	char str[BUFSIZ];	/* Only used for ^A lines in the weave */
 
 	before(pkt, linenum);
 	sprintf(str, NOGETTEXT("%c%c %d\n"), CTLCHAR, DEL, ser);
@@ -1185,7 +1378,9 @@ int	n;
 	}
 }
 
-
+/*
+ * Parse the line range from the diff output
+ */
 static char *
 linerange(cp, low, high)
 char	*cp;
@@ -1200,6 +1395,9 @@ int	*low, *high;
 	return(cp);
 }
 
+/*
+ * Skip lines from the diff outout, e.g. lines that start with "< ".
+ */
 static void
 skipline(lp, num)
 char	*lp;
@@ -1212,6 +1410,10 @@ int	num;
  	}
 }
 
+/*
+ * This is the central read routine.
+ * As long as we use fgets() here, we cannot support nul bytes in the files.
+ */
 static char *
 rddiff(s, n)
 char	*s;
@@ -1236,7 +1438,7 @@ char ch;
 int n;
 struct sid *sidp;
 {
-	char str[32];
+	char str[SID_STRSIZE];
 	register struct apply *ap;
 
 	sid_ba(sidp,str);
@@ -1275,10 +1477,8 @@ clean_up()
 	uname(&un);
 	uuname = un.nodename;
 	if (mylock(auxf(gpkt.p_file,'z'), getpid(),uuname)) {
-		if (gpkt.p_iop) {
-			fclose(gpkt.p_iop);
-			gpkt.p_iop = NULL;
-		}
+		sclose(&gpkt);
+		sfree(&gpkt);
 		if (gpkt.p_xiop) {
 			fclose(gpkt.p_xiop);
 			gpkt.p_xiop = NULL;
@@ -1298,6 +1498,22 @@ clean_up()
 	}
 }
 
+/*
+ * Compute the checksum for the new version and check for characters that
+ * are not allowed in the file.
+ *
+ * SCCSv4 disallows ^A (0x01) at the start of a line, nul bytes and requires a
+ *	newline at the end of the file.
+ *
+ * SCCSv6 currently disallows nul bytes in the file.
+ *	Since our diff(1) implementation supports to treat even files with
+ *	nul bytes as text files, we could support nul bytes in the future.
+ *
+ * Both versions allow an unlimited line length.
+ *
+ * Since fgets() does not report the amount of bytes read, we cannot support
+ * nul bytes on platforms with record oriented IO (VMS).
+ */
 /*ARGSUSED*/
 static void
 fgetchk(file, pkt)
@@ -1469,7 +1685,51 @@ warnctl(file, nline)
 		"WARNING [%s]: line %jd begins with ^A\n"),
 		file, (intmax_t)nline);
 }
- 
+
+static void 
+fixghash(pkt, ser)
+	struct	packet	*pkt;
+	int		ser;
+{
+	char		ghbuf[10];
+	signed char	*q;
+	struct packet	pk2;
+	struct stats	stats;
+
+	putline(pkt,(char *) 0);	/* Flush last unwritten line	*/
+	fflush(pkt->p_xiop);		/* Flush stdio buffers		*/
+
+	sinit(&pk2, auxf(pkt->p_file,'x'), SI_OPEN|SI_FORCE);
+
+	pk2.do_chksum = 0;		/* Checksums are still wrong	*/
+	pk2.p_stdout = stderr;
+	pk2.p_cutoff = MAX_TIME;
+
+	if (dodelt(&pk2, &stats, (struct sid *) 0, 0) == 0)
+		fmterr(&pk2);
+	flushto(&pk2, EUSERTXT, FLUSH_NOCOPY);
+
+	pk2.p_chkeof = 1;
+	pk2.p_gotsid = pk2.p_idel[ser].i_sid;
+	pk2.p_reqsid = pk2.p_gotsid;
+
+	setup(&pk2, ser);
+
+	pk2.p_ghash = 0;
+	while (readmod(&pk2))		/* Compute ghash for gotten content */
+		;
+	pk2.p_ghash &= 0xFFFF;
+
+	fseek(pkt->p_xiop, Checksum_offset, SEEK_SET);
+	fprintf(pkt->p_xiop, "%c%c s %5.5d\n",
+		CTLCHAR, SIDEXTENS, pk2.p_ghash);
+	pkt->p_nhash -= 5 * '0';
+	sprintf(ghbuf, "%5.5d", pk2.p_ghash);
+	q = (signed char *) ghbuf;
+	while (*q)
+		pkt->p_nhash += *q++;
+}
+
 /* SVR4.0 does not support getdtablesize().				  */
 
 #ifndef	HAVE_GETDTABLESIZE
