@@ -2,14 +2,13 @@
  * CDDL HEADER START
  *
  * The contents of this file are subject to the terms of the
- * Common Development and Distribution License, Version 1.0 only
- * (the "License").  You may not use this file except in compliance
- * with the License.
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may use this file only in accordance with the terms of version
+ * 1.0 of the CDDL.
  *
- * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
- * See the License for the specific language governing permissions
- * and limitations under the License.
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
@@ -37,12 +36,12 @@
  * contributors.
  */
 /*
- * This file contains modifications Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2018 J. Schilling
  *
- * @(#)diff.c	1.37 11/08/13 J. Schilling
+ * @(#)diff.c	1.73 18/12/17 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)diff.c 1.37 11/08/13 J. Schilling"
+#pragma ident "@(#)diff.c 1.73 18/12/17 J. Schilling"
 #endif
 
 #if defined(sun)
@@ -134,7 +133,7 @@
  *	If the code is changed to use "long", it would be possible to have
  *	real large file mode in case that diff is compiled in 64 bit mode.
  *	In 32 bit mode, there is no need to check for an integer overflow
- *	as the process will run "out of memory" before.
+ *	as the process will run "out of memory" before line numbers overflow.
  */
 #ifdef	SCHILY_BUILD
 
@@ -164,8 +163,9 @@
 #include <schily/vfork.h>
 #include <schily/libport.h>
 
-#else	/* non-portable SunOS definitions BEGIN */
+#else	/* non-portable SunOS -only definitions BEGIN */
 
+#define	SCCS_DIFF		0
 #define	_FILE_OFFSET_BITS	64
 #define	_LARGEFILE_SOURCE
 #include <stdio.h>
@@ -187,10 +187,10 @@
 #include <sysexits.h>
 
 #ifndef	PROVIDER
-#define	PROVIDER	"Sun"
+#define	PROVIDER	"Schily"
 #endif
 #ifndef	VERSION
-#define	VERSION		"1.x"
+#define	VERSION		"5.08"
 #endif
 #ifndef	HOST_OS
 #define	HOST_OS		"SunOS"
@@ -228,22 +228,39 @@
 
 #define	HAVE_LARGEFILES
 #define	HAVE_CFTIME
+#define	HAVE_GETEXECNAME
+#define	HAVE_MEMCMP
+#define	HAVE_VFORK
+#define	_FOUND_STAT_NSECS_
+#define	HAVE_REALLOC_NULL
 
-#endif	/* non-portable SunOS definitions END */
+/*
+ * Found e.g. on SunOS-5.x
+ */
+#define	stat_ansecs(s)		((s)->st_atim.tv_nsec)
+#define	stat_mnsecs(s)		((s)->st_mtim.tv_nsec)
+#define	stat_cnsecs(s)		((s)->st_ctim.tv_nsec)
+
+#endif	/* non-portable SunOS -only definitions END */
 
 
 #ifdef	HAVE_LARGEFILES
 #undef	fseek
 #define	fseek		fseeko
 #undef	ftell
-#define	ftell		fetllo
+#define	ftell		ftello
 #endif
 
 #include "diff.h"
 
 #ifndef	PATH_MAX
+#ifdef	MAXPATHNAME
 #define	PATH_MAX	MAXPATHNAME
 #endif
+#endif
+#ifndef	PATH_MAX
+#define	PATH_MAX	1024
+#endif
 
 #ifdef	pdp11
 #define	D_BUFSIZ	BUFSIZ
@@ -251,10 +268,6 @@
 #define	D_BUFSIZ	(32*1024)
 #endif
 
-#ifndef	HAVE_CFTIME
-#define	cftime(buf, dcmsg, tl)	(strcpy(buf, ctime(tl)), buf[24] = 0)
-#endif
-
 /*
  * In case this is missing in /usr/include/ (e.g. SunOS-4.x)
  */
@@ -268,19 +281,27 @@ extern int	optind, opterr, optopt;
 #undef	min
 #define	min(a, b)	((a) > (b) ? (b) : (a))
 
-int pref, suff;		/* length of prefix and suffix */
-int *class;		/* will be overlaid on file[0] */
-int *member;		/* will be overlaid on file[1] */
-int *klist;		/* will be overlaid on file[0] after class */
-struct cand *clist;	/* merely a free storage pot for candidates */
-int clen = 0;
-int *J;			/* will be overlaid on class */
-off_t *ixold;		/* will be overlaid on klist */
-off_t *ixnew;		/* will be overlaid on file[1] */
+static int pref, suff;		/* length of prefix and suffix */
+static int *class;		/* will be overlaid on file[0] */
+static int *member;		/* will be overlaid on file[1] */
+static int *klist;		/* will be overlaid on file[0] after class */
+static struct cand *clist;	/* merely a free storage pot for candidates */
+static int clen = 0;
+static int camt = 0;
+static int *J;			/* will be overlaid on class */
+static off_t *ixold;		/* will be overlaid on klist */
+static off_t *ixnew;		/* will be overlaid on file[1] */
+
+#define	FUNCTION_CONTEXT_SIZE	55
+static char	lastbuf[FUNCTION_CONTEXT_SIZE];
+static int	lastline;
+static int	lastmatchline;
 
 static int	didvfork;
 static int	mbcurmax;
 
+static char	nulldev[] = "/dev/null";
+
 static void	*talloc __PR((size_t n));
 static void	*ralloc __PR((void *p, size_t n));
 	int	main __PR((int argc, char **argv));
@@ -292,7 +313,8 @@ static void	change __PR((int, int, int, int));
 static void	range __PR((int, int, char *));
 static void	fetch __PR((off_t *, int, int, int, char *, int));
 static void	dump_context_vec __PR((void));
-static void	diffdir __PR((char **));
+static void	diffdir __PR((char **, struct pdirs *));
+static void	calldiffdir __PR((char **, struct pdirs *));
 static void	setfile __PR((char **, char **, char *));
 static void	scanpr __PR((struct dir *, int, char *, char *,
 	char *, char *, char *));
@@ -309,6 +331,11 @@ static void	usage __PR((void));
 static void	initbuf __PR((FILE *, int, off_t));
 static void	resetbuf __PR((int));
 
+static int	diffreg __PR((void));
+static void	print_dstatus __PR((void));
+#ifndef	DO_SPAWN_DIFF
+static int	calldiffreg __PR((int, int));
+#endif
 static int	stone __PR((int *, int, int *, int *));
 static int	newcand __PR((int, int, int));
 static int	search __PR((int *, int, int));
@@ -327,6 +354,13 @@ static struct dir *setupdir __PR((char *));
 static off_t	ftellbuf __PR((int));
 static wint_t	wcput	__PR((wint_t));
 static wint_t	getbufwchar __PR((int, int *));
+#if !defined(HAVE_CFTIME) && defined(HAVE_STRFTIME)
+static time_t	gmtoff __PR((const time_t *clk));
+#endif
+static void	cf_time	__PR((char *s, size_t maxsize,
+				char *fmt, const time_t *clk));
+
+static char	*match_function __PR((const off_t *, int, int));
 
 
 /*
@@ -355,6 +389,11 @@ ralloc(p, n)			/* compacting reallocation */
 	void	*q;
 #if 0
 	free(p);
+#endif
+#ifndef	HAVE_REALLOC_NULL
+	if (p == NULL)
+		q = malloc(n);
+	else
 #endif
 	q = realloc(p, n);
 	if (q == NULL)
@@ -372,7 +411,6 @@ main(argc, argv)
 	int	argc;
 	char	**argv;
 {
-	int k;
 	char *argp;
 	int flag;			/* option flag read by getopt() */
 	int i, j;
@@ -390,7 +428,8 @@ main(argc, argv)
 	diffargv = argv;
 	whichtemp = 0;
 	while ((flag =
-	    getopt(argc, argv, "bitwcuefhnlrsNC:D:S:U:V(version)")) != EOF) {
+	    getopt(argc, argv,
+		    "()abBdiptwcuefhnqlrsNC:D:S:U:V(version)")) != EOF) {
 		switch (flag) {
 		case 'D':
 			opt = D_IFDEF;
@@ -399,10 +438,21 @@ main(argc, argv)
 			ifdef2 = optarg;
 			break;
 
+		case 'a':
+			aflag = 1;
+			break;
+
 		case 'b':
 			bflag = 1;
 			break;
 
+		case 'B':
+			Bflag = 1;
+			break;
+
+		case 'd':		/* -d is a no-op for GNU diff compat */
+			break;
+
 		case 'C':
 		case 'U':
 			opt = D_CONTEXT;
@@ -456,12 +506,25 @@ main(argc, argv)
 			opt = D_NREVERSE;
 			break;
 
+		case 'p':
+			pflag = 1;
+			if (opt == D_NORMAL) {
+				opt = D_CONTEXT;
+				context = 3;
+				uflag = 0;
+			}
+			break;
+
+		case 'q':
+			opt = D_BRIEF;
+			break;
+
 		case 'r':
 			rflag = 1;
 			break;
 
 		case 'S':
-			(void) strlcpy(start, optarg, sizeof (start));
+			start = optarg;
 			break;
 
 		case 's':
@@ -481,8 +544,10 @@ main(argc, argv)
 			break;
 
 		case 'V':		/* version */
-			printf("diff %s-SCCS version %s%s%s (%s-%s-%s)\n",
+			printf(gettext(
+			    "diff %s-%s version %s%s%s (%s-%s-%s)\n"),
 				PROVIDER,
+				SCCS_DIFF ? "SCCS":"diff",
 				VERSION,
 				*VDATE ? " ":"",
 				VDATE,
@@ -510,11 +575,12 @@ main(argc, argv)
 	file1 = argv[0];
 	file2 = argv[1];
 	file1ok = file2ok = 1;
+	status = 0;
 
 	if (hflag) {
 		if (opt) {
 			error(
-gettext("-h doesn't support -e, -f, -n, -c, or -I"));
+gettext("-h doesn't support -e, -f, -n, -c, -q, -u, or -D"));
 		} else {
 			diffargv[0] = "diffh";
 			(void) execv(diffh, diffargv);
@@ -534,7 +600,7 @@ gettext("-h doesn't support -e, -f, -n, -c, or -I"));
 				file1ok = 0;
 				stb1.st_mode = S_IFREG;	/* assume plain file */
 				stb1.st_size = 0;
-				input[0] = fopen("/dev/null", "r");
+				input[0] = fopen(nulldev, "r");
 				fail += 1;
 			}
 		}
@@ -546,7 +612,7 @@ gettext("-h doesn't support -e, -f, -n, -c, or -I"));
 				else
 					stb2.st_mode = S_IFREG;
 				stb2.st_size = 0;
-				input[1] = fopen("/dev/null", "r");
+				input[1] = fopen(nulldev, "r");
 				fail += 1;
 			}
 		} else if (fail == 1) {			/* file2 exists	    */
@@ -563,40 +629,44 @@ gettext("-h doesn't support -e, -f, -n, -c, or -I"));
 	}
 
 	if (strcmp(file1, "-") == 0) {
-		if (fstat(fileno(stdin), &stb1) == 0)
+		if (fstat(fileno(stdin), &stb1) == 0) {
 			stb1.st_mode = S_IFREG;
-		else {
+		} else {
 			(void) fprintf(stderr, "diff: ");
 			perror("stdin");
+			status = 2;
 			done();
 		}
 	} else if (input[0] == NULL && stat(file1, &stb1) < 0) {
 		(void) fprintf(stderr, "diff: ");
 		perror(file1);
+		status = 2;
 		done();
 	}
 
 	if (strcmp(file2, "-") == 0) {
-		if (strcmp(file1, "-") == 0)
+		if (strcmp(file1, "-") == 0) {
 			error(gettext("cannot specify - -"));
-		else {
-			if (fstat(fileno(stdin), &stb2) == 0)
+		} else {
+			if (fstat(fileno(stdin), &stb2) == 0) {
 				stb2.st_mode = S_IFREG;
-			else {
+			} else {
 				(void) fprintf(stderr, "diff: ");
 				perror("stdin");
+				status = 2;
 				done();
 			}
 		}
 	} else if (input[1] == NULL && stat(file2, &stb2) < 0) {
 		(void) fprintf(stderr, "diff: ");
 		perror(file2);
+		status = 2;
 		done();
 	}
 
 	if ((stb1.st_mode & S_IFMT) == S_IFDIR &&
 	    (stb2.st_mode & S_IFMT) == S_IFDIR) {
-		diffdir(argv);
+		diffdir(argv, (struct pdirs *)NULL);
 		done();
 	    }
 
@@ -666,7 +736,8 @@ gettext("-h doesn't support -e, -f, -n, -c, or -I"));
 
 notsame:
 	status = 1;
-	if (filebinary(input[0]) || filebinary(input[1])) {
+	if (!aflag &&
+	    (filebinary(input[0]) || filebinary(input[1]))) {
 		if (ferror(input[0]) || ferror(input[1])) {
 			(void) fprintf(stderr, "diff: ");
 			(void) fprintf(stderr, gettext("Error reading "));
@@ -682,6 +753,29 @@ notsame:
 		(void) fclose(input[1]);
 		done();
 	}
+	anychange = diffreg();
+	status = anychange;
+
+same:
+	print_dstatus();
+
+#ifdef	DEBUG
+	fprintf(stderr, "Allocated space: %ld Bytes\n", (long)sbrk(0) - oe);
+#endif
+	done();
+	/*NOTREACHED*/
+	return (0);
+}
+
+static int
+diffreg()
+{
+	int	k;
+
+	anychange = 0;
+	lastline = 0;
+	lastmatchline = 0;
+
 	prepare(0, file1);
 	prepare(1, file2);
 	prune();
@@ -698,32 +792,85 @@ notsame:
 
 	klist = (int *)talloc((slen[0] + 2) * sizeof (int));
 	clist = (struct cand *)talloc(sizeof (cand));
+	clen = 0;
+	camt = 0;
 	k = stone(class, slen[0], member, klist);
 	free((void *)member);
 	free((void *)class);
 
-	J = (int *)talloc((len[0] + 2) * sizeof (int));
+	J = (int *)ralloc(J, (len[0] + 2) * sizeof (int));
 	unravel(klist[k]);
 	free((char *)clist);
 	free((char *)klist);
 
-	ixold = (off_t *)talloc((len[0] + 2) * sizeof (off_t));
-	ixnew = (off_t *)talloc((len[1] + 2) * sizeof (off_t));
+	ixold = (off_t *)ralloc(ixold, (len[0] + 2) * sizeof (off_t));
+	ixnew = (off_t *)ralloc(ixnew, (len[1] + 2) * sizeof (off_t));
 	check();
 	output();
-	status = anychange;
 
-same:
+	return (anychange);
+}
+
+static void
+print_dstatus()
+{
 	if (opt == D_CONTEXT && anychange == 0)
 		(void) printf(gettext("No differences encountered\n"));
-#ifdef	DEBUG
-	fprintf(stderr, "Allocates space: %ld Bytes\n", (long)sbrk(0) - oe);
-#endif
-	done();
-	/*NOTREACHED*/
-	return (0);
+	else if (opt == D_BRIEF && anychange != 0)
+		(void) printf(gettext("Files %s and %s differ\n"),
+		    file1, file2);
 }
 
+#ifndef	DO_SPAWN_DIFF
+static int
+calldiffreg(f1, f2)
+	int	f1;
+	int	f2;
+{
+	int	result = anychange;
+	int	ret;
+	char	*in1 = input_file1;
+	char	*in2 = input_file2;
+
+	input_file1 = file1;
+	input_file2 = file2;
+
+	if ((input[0] = fdopen(f1, "r")) == NULL) {
+		(void) fprintf(stderr, "diff: ");
+		perror(file1);
+		status = 2;
+		done();
+	}
+	if ((input[1] = fdopen(f2, "r")) == NULL) {
+		(void) fprintf(stderr, "diff: ");
+		perror(file1);
+		status = 2;
+		done();
+	}
+
+	initbuf(input[0], 0, (off_t)0);
+	initbuf(input[1], 1, (off_t)0);
+
+#ifdef	HAVE_SETVBUF
+	setvbuf(input[0], NULL, _IOFBF, 32*1024);
+	setvbuf(input[1], NULL, _IOFBF, 32*1024);
+#endif
+	rewind(input[0]);
+	rewind(input[1]);
+
+	ret = diffreg();
+	print_dstatus();
+	fclose(input[0]);
+	fclose(input[1]);
+
+	input_file1 = in1;
+	input_file2 = in2;
+	if (ret > result)
+		result = ret;
+	return (result);
+}
+#endif
+
 static int
 stone(a, n, b, c)
 	int	*a;
@@ -776,12 +923,16 @@ newcand(x, y, pred)
 {
 	struct cand *q;
 
-	clist = (struct cand *)ralloc((void *)clist, ++clen * sizeof (cand));
-	q = clist + clen -1;
+	if (clen >= camt) {
+		camt += 64;
+		clist = (struct cand *)ralloc((void *)clist,
+		    camt * sizeof (cand));
+	}
+	q = clist + clen;
 	q->x = x;
 	q->y = y;
 	q->pred = pred;
-	return (clen - 1);
+	return (clen++);
 }
 
 static int
@@ -960,9 +1111,7 @@ skipline(f)
 	wint_t c;
 	int	mlen;
 
-	for (i = 1; (c = getbufwchar(f, &mlen)) != 0; ) {
-		if (c == '\n' || c == WEOF)
-			return (i);
+	for (i = 1; (c = getbufwchar(f, &mlen)) != '\n' && c != WEOF; ) {
 		i += mlen;
 	}
 	return (i);
@@ -983,8 +1132,8 @@ output()
 	m = len[0];
 	J[0] = 0;
 	J[m + 1] = len[1] + 1;
-	if (opt != D_EDIT)
-		for (i0 = 1; i0 <= m; i0 = i1+1) {
+	if (opt != D_EDIT) {
+		for (i0 = 1; i0 <= m; i0 = i1 + 1) {
 			while (i0 <= m && J[i0] == J[i0 - 1] + 1)
 				i0++;
 			j0 = J[i0 - 1] + 1;
@@ -994,7 +1143,9 @@ output()
 			j1 = J[i1 + 1] - 1;
 			J[i1] = j1;
 			change(i0, i1, j0, j1);
-		} else for (i0 = m; i0 >= 1; i0 = i1 - 1) {
+		}
+	} else {
+		for (i0 = m; i0 >= 1; i0 = i1 - 1) {
 			while (i0 >= 1 && J[i0] == J[i0 + 1] - 1 && J[i0] != 0)
 				i0--;
 			j0 = J[i0 + 1] - 1;
@@ -1005,6 +1156,7 @@ output()
 			J[i1] = j1;
 			change(i1, i0, j1, j0);
 		}
+	}
 	if (m == 0)
 		change(1, 0, 1, len[1]);
 	if (opt == D_IFDEF) {
@@ -1038,8 +1190,33 @@ change(a, b, c, d)
 	char	*dcmsg;
 	char	*p;
 
+#if	BUFSIZ < 40	/* 36 should be sufficient */
+	error time buffer too small
+#endif
 	if (opt != D_IFDEF && a > b && c > d)
 		return;
+	if (Bflag) {
+		int	i;
+
+		/*
+		 * On a UNIX system, enpty lines only contain a single "\n" and
+		 * thus cause a file offset difference of 1 to the next line.
+		 */
+		if (a > b) {	/* Inserted */
+			for (i = c; i <= d; i++) {
+				if ((ixnew[i] - ixnew[i - 1]) != 1)
+					goto show;
+			}
+		}
+		if (c > d) {	/* Deleted */
+			for (i = a; i <= b; i++) {
+				if ((ixold[i] - ixold[i - 1]) != 1)
+					goto show;
+			}
+		}
+		return;
+	}
+show:
 	if (anychange == 0) {
 		anychange = 1;
 		if (opt == D_CONTEXT) {
@@ -1064,28 +1241,46 @@ change(a, b, c, d)
 				dcmsg = dcgettext(NULL, "%a %b %e %T %Y",
 								LC_TIME);
 			}
-			(void) cftime(time_buf, dcmsg, &stb1.st_mtime);
-			if ((p = strchr(time_buf, '.')) != NULL) {
+			cf_time(time_buf, sizeof (time_buf),
+						dcmsg, &stb1.st_mtime);
+
+#if	defined(_FOUND_STAT_NSECS_)
+			/*
+			 * Be careful here: in the German locale, the string
+			 * contains "So. " for "Sonntag".
+			 */
+			if ((p = strchr(time_buf, '.')) != NULL &&
+			    p[1] == '0') {
 				long	ns = stat_mnsecs(&stb1);
 				if (ns < 0)
 					ns = 0;
 				sprintf(++p, "%9.9ld", ns);
 				p[9] = ' ';	/* '\0' from sprintf() */
 			}
+#endif
 			if (uflag)
 				(void) printf("--- %s	%s\n", input_file1,
 				    time_buf);
 			else
 				(void) printf("*** %s	%s\n", input_file1,
 				    time_buf);
-			(void) cftime(time_buf, dcmsg, &stb2.st_mtime);
-			if ((p = strchr(time_buf, '.')) != NULL) {
+			cf_time(time_buf, sizeof (time_buf),
+						dcmsg, &stb2.st_mtime);
+
+#if	defined(_FOUND_STAT_NSECS_)
+			/*
+			 * Be careful here: in the German locale, the string
+			 * contains "So. " for "Sonntag".
+			 */
+			if ((p = strchr(time_buf, '.')) != NULL &&
+			    p[1] == '0') {
 				long	ns = stat_mnsecs(&stb2);
 				if (ns < 0)
 					ns = 0;
 				sprintf(++p, "%9.9ld", ns);
 				p[9] = ' ';	/* '\0' from sprintf() */
 			}
+#endif
 			if (uflag)
 				(void) printf("+++ %s	%s\n", input_file2,
 				    time_buf);
@@ -1093,9 +1288,11 @@ change(a, b, c, d)
 				(void) printf("--- %s	%s\n", input_file2,
 				    time_buf);
 
-			context_vec_start = (struct context_vec *)
+			if (context_vec_start == NULL) {
+				context_vec_start = (struct context_vec *)
 					    malloc(MAX_CONTEXT *
 					    sizeof (struct context_vec));
+			}
 			if (context_vec_start == NULL)
 				error(gettext(NO_MEM_ERR));
 
@@ -1127,6 +1324,8 @@ change(a, b, c, d)
 	}
 
 	switch (opt) {
+	case D_BRIEF:
+		return;
 	case D_NORMAL:
 	case D_EDIT:
 		range(a, b, ",");
@@ -1140,9 +1339,9 @@ change(a, b, c, d)
 		(void) printf("\n");
 		break;
 	case D_NREVERSE:
-		if (a > b)
+		if (a > b) {
 			(void) printf("a%d %d\n", b, d - c + 1);
-		else {
+		} else {
 			(void) printf("d%d %d\n", a, b - a + 1);
 			if (!(c > d))
 				/* add changed lines */
@@ -1215,10 +1414,10 @@ fetch(f, a, b, filen, s, oldfile)
 		return;
 	if (opt == D_IFDEF) {
 		int oneflag = (*ifdef1 != '\0') != (*ifdef2 != '\0');
-		if (inifdef)
+		if (inifdef) {
 			(void) fprintf(stdout, "#else /* %s%s */\n",
 			    oneflag && oldfile == 1 ? "!" : "", ifdef2);
-		else {
+		} else {
 			if (oneflag) {
 				/* There was only one ifdef given */
 				endifname = ifdef2;
@@ -1238,23 +1437,42 @@ fetch(f, a, b, filen, s, oldfile)
 	}
 
 	for (i = a; i <= b; i++) {
-		(void) fseek(lb, f[i - 1], SEEK_SET);
-		initbuf(lb, filen, f[i - 1]);
+		wint_t	lastch;
+
+		if (ftellbuf(filen) != f[i - 1]) {
+			/*
+			 * Only seek in case we are not at the expected offset.
+			 * This is marginally slower with small files but 50%
+			 * faster with huge files.
+			 */
+			(void) fseek(lb, f[i - 1], SEEK_SET);
+			initbuf(lb, filen, f[i - 1]);
+		}
 		if (opt != D_IFDEF)
 			(void) prints(s);
 		col = 0;
-		while ((ch = getbufwchar(filen, &mlen)) != 0) {
-			if (ch != '\n' && ch != WEOF) {
-				if (ch == '\t' && tflag)
-					do
-						(void) putchar(' ');
-					while (++col & 7);
-				else {
-					(void) wcput(ch);
-					col++;
-				}
-			} else
-				break;
+		lastch = '\0';
+		while ((ch = getbufwchar(filen, &mlen)) != '\n' && ch != WEOF) {
+			if (ch == '\t' && tflag) {
+				do
+					(void) putchar(' ');
+				while (++col & 7);
+			} else {
+				(void) wcput(ch);
+				if (col++ == 0)
+					lastch = ch;
+			}
+		}
+		/*
+		 * When creating an "ed" script, we cannot directly append a
+		 * line that contains only ".\n". We rather enter "..\n", leave
+		 * append mode, substitute ".." by "." and re-enter append mode.
+		 */
+		if (opt == D_EDIT && col == 1 && lastch == '.' && ch == '\n') {
+			char	*cp = ".\n.\ns/.//\na";
+
+			while (*cp)
+				(void) wcput(*cp++);
 		}
 		(void) putchar('\n');
 	}
@@ -1282,7 +1500,7 @@ readhash(f, filen, str)
 	sum = 1;
 	space = 0;
 	if (!bflag && !wflag) {
-		if (iflag)
+		if (iflag) {
 			if (mbcurmax == 1) {
 				/* In this case, diff doesn't have to take */
 				/* care of multibyte characters. */
@@ -1293,8 +1511,9 @@ readhash(f, filen, str)
 							(void) fprintf(stderr,
 	gettext("Warning: missing newline at end of file %s\n"), str);
 							break;
-						} else
+						} else {
 							return (0);
+						}
 					}
 					sum += (isupper(t) ? tolower(t) : t) <<
 						(shift &= HALFMASK);
@@ -1310,14 +1529,15 @@ readhash(f, filen, str)
 							(void) fprintf(stderr,
 	gettext("Warning: missing newline at end of file %s\n"), str);
 							break;
-						} else
+						} else {
 							return (0);
+						}
 					}
 					sum += NCCHRTRAN(wt) <<
 						(shift &= HALFMASK);
 				}
 			}
-		else
+		} else {
 			/* In this case, diff doesn't have to take care of */
 			/* multibyte characters. */
 			for (shift = 0; (t = getc(f)) != '\n'; shift += 7) {
@@ -1326,11 +1546,13 @@ readhash(f, filen, str)
 						(void) fprintf(stderr,
 	gettext("Warning: missing newline at end of file %s\n"), str);
 						break;
-					} else
+					} else {
 						return (0);
+					}
 				}
 				sum += (long)t << (shift &= HALFMASK);
 			}
+		}
 	} else {
 		/* In this case, diff needs to take care of */
 		/* multibyte characters. */
@@ -1347,8 +1569,9 @@ readhash(f, filen, str)
 						(void) fprintf(stderr,
 	gettext("Warning: missing newline at end of file %s\n"), str);
 						break;
-					} else
+					} else {
 						return (0);
+					}
 				default:
 					if (space && !wflag) {
 						shift += 7;
@@ -1409,11 +1632,31 @@ dump_context_vec()
 			a = 1;
 		if (file2ok == 0)
 			b = 1;
-		(void) printf("@@ -%d,%d +%d,%d @@\n",
+		(void) printf("@@ -%d,%d +%d,%d @@",
+		    lowa > upb ? upb : 		/* Needed for -U0 */
 		    lowa - a, upb - lowa + 1,
+#ifdef	__symmetric_low__			/* othogonal but wrong */
+		    lowc > upd ? upd :
+#endif
 		    lowc - b, upd - lowc + 1);
+		if (pflag) {
+			char	*f;
+
+			f = match_function(ixold, lowa-1, 0);
+			if (f != NULL)
+				(void) printf(" %s", f);
+		}
+		(void) printf("\n");
 	} else {
-		(void) printf("***************\n*** ");
+		(void) printf("***************");
+		if (pflag) {
+			char	*f;
+
+			f = match_function(ixold, lowa-1, 0);
+			if (f != NULL)
+				(void) printf(" %s", f);
+		}
+		(void) printf("\n*** ");
 		range(lowa, upb, ",");
 		(void) printf(" ****\n");
 	}
@@ -1423,15 +1666,17 @@ dump_context_vec()
 	 * output if there were no changes to the "old" file (we'll see
 	 * the "old" lines as context in the "new" list).
 	 */
-	if (uflag)
+	if (uflag) {
 		do_output = 1;
-	else
-		for (do_output = 0; cvp <= context_vec_ptr; cvp++)
+	} else {
+		for (do_output = 0; cvp <= context_vec_ptr; cvp++) {
 			if (cvp->a <= cvp->b) {
 				cvp = context_vec_start;
 				do_output++;
 				break;
 			}
+		}
+	}
 
 	if (do_output) {
 		while (cvp <= context_vec_ptr) {
@@ -1450,7 +1695,7 @@ dump_context_vec()
 					fetch(ixnew, c, d, 1, "+", 0);
 			} else if (ch == 'd') {
 				fetch(ixold, lowa, a - 1, 0, uflag ? " " :
-					    "  ", 1);
+				    "  ", 1);
 				fetch(ixold, a, b, 0, uflag ? "-" : "- ", 1);
 			} else {
 				/* The last argument should not affect */
@@ -1460,8 +1705,9 @@ dump_context_vec()
 				if (uflag) {
 					fetch(ixold, a, b, 0, "-", 1);
 					fetch(ixnew, c, d, 1, "+", 0);
-				} else
+				} else {
 					fetch(ixold, a, b, 0, "! ", 1);
+				}
 			}
 			lowa = b + 1;
 			cvp++;
@@ -1482,12 +1728,13 @@ dump_context_vec()
 	(void) printf(" ----\n");
 
 	do_output = 0;
-	for (cvp = context_vec_start; cvp <= context_vec_ptr; cvp++)
+	for (cvp = context_vec_start; cvp <= context_vec_ptr; cvp++) {
 		if (cvp->c <= cvp->d) {
 			cvp = context_vec_start;
 			do_output++;
 			break;
 		}
+	}
 
 	if (do_output) {
 		while (cvp <= context_vec_ptr) {
@@ -1498,11 +1745,11 @@ dump_context_vec()
 			else
 				ch = (a <= b) ? 'd' : 'a';
 
-			if (ch == 'd')
+			if (ch == 'd') {
 				/* The last argument should not affect */
 				/* the behavior of fetch() */
 				fetch(ixnew, lowc, d, 1, "  ", 0);
-			else {
+			} else {
 				/* The last argument should not affect */
 				/* the behavior of fetch() */
 				fetch(ixnew, lowc, c - 1, 1, "  ", 0);
@@ -1528,8 +1775,9 @@ int	header;
 char	title[2 * BUFSIZ], *etitle;
 
 static void
-diffdir(argv)
-	char	**argv;
+diffdir(argv, pdirs)
+	char		**argv;
+	struct pdirs	*pdirs;
 {
 	struct dir *d1, *d2;
 	struct dir *dir1, *dir2;
@@ -1579,15 +1827,19 @@ diffdir(argv)
 			cmp = -1;
 		else
 			cmp = strcmp(d1->d_entry, d2->d_entry);
+
+		file1ok = file2ok = 1;
 		if (cmp < 0) {
 			if (Nflag) {
 				result = compare(d1);
 				if (result > dirstatus)
 					dirstatus = result;
-			} else if (lflag)
+			} else if (lflag) {
 				d1->d_flags |= ONLY;
-			else if (opt == D_NORMAL || opt == D_CONTEXT)
+			} else if (opt == D_NORMAL || opt == D_CONTEXT ||
+			    opt == D_BRIEF) {
 				only(d1, 1);
+			}
 			if (d1->d_entry)
 				d1++;
 			if (!Nflag && dirstatus == 0)
@@ -1607,10 +1859,11 @@ diffdir(argv)
 					dirstatus = result;
 				if (d2->d_flags & DIRECT)
 					d2->d_flags |= XDIRECT;
-			} else if (lflag)
+			} else if (lflag) {
 				d2->d_flags |= ONLY;
-			else if (opt == D_NORMAL || opt == D_CONTEXT)
+			} else if (opt == D_NORMAL || opt == D_CONTEXT) {
 				only(d2, 2);
+			}
 			if (d2->d_entry)
 				d2++;
 			if (!Nflag && dirstatus == 0)
@@ -1633,30 +1886,84 @@ diffdir(argv)
 		    file1, efile1, file2, efile2);
 	}
 	if (rflag) {
+		struct pdirs	pdir;
+
+		pdir.p_last = pdirs;
+
 		if (header && lflag)
 			(void) printf("\f");
-		for (d1 = dir1; d1->d_entry; d1++)  {
+		for (d1 = dir1; d1->d_entry; d1++) {
 			if ((d1->d_flags & DIRECT) == 0)
 				continue;
 			(void) strcpy(efile1, d1->d_entry);
 			(void) strcpy(efile2, d1->d_entry);
-			result = calldiff((char *)0);
-			if (result > dirstatus)
-				dirstatus = result;
+			pdir.p_dev1 = d1->d_dev1;
+			pdir.p_dev2 = d1->d_dev2;
+			pdir.p_ino1 = d1->d_ino1;
+			pdir.p_ino2 = d1->d_ino2;
+			calldiffdir(argv, &pdir);
 		}
 		if (Nflag) {
-			for (d2 = dir2; d2->d_entry; d2++)  {
+			for (d2 = dir2; d2->d_entry; d2++) {
 				if ((d2->d_flags & XDIRECT) == 0)
 						continue;
 				(void) strcpy(efile1, d2->d_entry);
 				(void) strcpy(efile2, d2->d_entry);
-				result = calldiff((char *)0);
-				if (result > dirstatus)
-					dirstatus = result;
+				pdir.p_dev1 = d2->d_dev1;
+				pdir.p_dev2 = d2->d_dev2;
+				pdir.p_ino1 = d2->d_ino1;
+				pdir.p_ino2 = d2->d_ino2;
+				calldiffdir(argv, &pdir);
 			}
 		}
 	}
-	status = dirstatus;
+	for (d1 = dir1; d1->d_entry; d1++)
+		free(d1->d_entry);
+	for (d2 = dir2; d2->d_entry; d2++)
+		free(d2->d_entry);
+	free(dir1);
+	free(dir2);
+	if (dirstatus > status)
+		status = dirstatus;
+}
+
+static void
+calldiffdir(argv, pdirs)
+	char		**argv;
+	struct pdirs	*pdirs;
+{
+	char	*f1 = file1;
+	char	*f2 = file2;
+	char	*ef1 = efile1;
+	char	*ef2 = efile2;
+	struct pdirs	*pd = pdirs->p_last;
+
+#ifndef	DIRCMP_DEBUG
+	/*
+	 * We do not need to compare identical directories.
+	 */
+	if (pdirs->p_dev1 == pdirs->p_dev2 &&
+	    pdirs->p_ino1 == pdirs->p_ino2)
+		return;
+#endif
+	while (pd) {
+		if (pdirs->p_dev1 == pd->p_dev1 &&
+		    pdirs->p_dev2 == pd->p_dev2 &&
+		    pdirs->p_ino1 == pd->p_ino1 &&
+		    pdirs->p_ino2 == pd->p_ino2)
+			return;
+		pd = pd->p_last;
+	}
+
+	diffdir(argv, pdirs);
+	free(file1);
+	free(file2);
+	file1 = f1;
+	file2 = f2;
+	efile1 = ef1;
+	efile2 = ef2;
+	argv[0] = file1;
+	argv[1] = file2;
 }
 
 static void
@@ -1731,6 +2038,7 @@ setupdir(cp)
 	struct dir *dp = 0, *ep;
 	struct dirent *rp;
 	int nitems;
+	int	dplen;
 	int size;
 	DIR *dirp;
 
@@ -1745,20 +2053,23 @@ setupdir(cp)
 	if (dirp == NULL) {
 		(void) fprintf(stderr, "diff: ");
 		perror(cp);
+		status = 2;
 		done();
 	}
 	nitems = 0;
+	dplen = 0;
 	dp = (struct dir *)malloc(sizeof (struct dir));
 	if (dp == 0)
 		error(gettext(NO_MEM_ERR));
 
 	while ((rp = readdir(dirp)) != NULL) {
 		ep = &dp[nitems++];
-#ifdef	__needed__
-		ep->d_reclen = rp->d_reclen;
-#endif
 		ep->d_entry = 0;
 		ep->d_flags = 0;
+		ep->d_dev1 = 0;
+		ep->d_dev2 = 0;
+		ep->d_ino1 = 0;
+		ep->d_ino2 = 0;
 		size = strlen(rp->d_name);
 		if (size > 0) {
 			ep->d_entry = (char *)malloc(size + 1);
@@ -1767,8 +2078,11 @@ setupdir(cp)
 
 			(void) strcpy(ep->d_entry, rp->d_name);
 		}
-		dp = (struct dir *)realloc((char *)dp,
-			(nitems + 1) * sizeof (struct dir));
+		if (nitems >= dplen) {
+			dplen += 64;
+			dp = (struct dir *)realloc((char *)dp,
+				(dplen + 1) * sizeof (struct dir));
+		}
 		if (dp == 0)
 			error(gettext(NO_MEM_ERR));
 	}
@@ -1804,7 +2118,10 @@ compare(dp)
 	if (stat(file1, &statb1) == -1) {
 		if (errno == ENOENT && Nflag) {
 			statb1.st_mode = 0;
+			statb1.st_dev = 0;
+			statb1.st_ino = 0;
 			result = 1;
+			file1ok = 0;
 		} else {
 			(void) fprintf(stderr, "diff: ");
 			perror(file1);
@@ -1814,7 +2131,10 @@ compare(dp)
 	if (stat(file2, &statb2) == -1) {
 		if (errno == ENOENT && Nflag) {
 			statb2.st_mode = 0;
+			statb2.st_dev = 0;
+			statb2.st_ino = 0;
 			result = 1;
+			file2ok = 0;
 		} else {
 			(void) fprintf(stderr, "diff: ");
 			perror(file2);
@@ -1827,21 +2147,25 @@ compare(dp)
 
 	if (fmt1 == S_IFREG) {
 		f1 = open(file1, O_RDONLY);
-		if (f1 < 0) {
-			(void) fprintf(stderr, "diff: ");
-			perror(file1);
-			return (2);
-		}
+	} else if (file1ok == 0) {
+		f1 = open(nulldev, O_RDONLY);
+	}
+	if ((fmt1 == S_IFREG || file1ok == 0) && f1 < 0) {
+		(void) fprintf(stderr, "diff: ");
+		perror(file1);
+		return (2);
 	}
 
 	if (fmt2 == S_IFREG) {
 		f2 = open(file2, O_RDONLY);
-		if (f2 < 0) {
-			(void) fprintf(stderr, "diff: ");
-			perror(file2);
-			(void) close(f1);
-			return (2);
-		}
+	} else if (file2ok == 0) {
+		f2 = open(nulldev, O_RDONLY);
+	}
+	if ((fmt2 == S_IFREG || file2ok == 0) && f2 < 0) {
+		(void) fprintf(stderr, "diff: ");
+		perror(file2);
+		(void) close(f1);
+		return (2);
 	}
 
 	if (result) {
@@ -1857,6 +2181,10 @@ compare(dp)
 
 			case S_IFDIR:
 				dp->d_flags = DIRECT;
+				dp->d_dev1 = statb1.st_dev;
+				dp->d_dev2 = statb2.st_dev;
+				dp->d_ino1 = statb1.st_ino;
+				dp->d_ino2 = statb2.st_ino;
 				if (lflag || opt == D_EDIT)
 					goto closem;
 				if (Nflag && rflag)
@@ -1881,13 +2209,15 @@ compare(dp)
 
 #ifdef	S_IFLNK
 			case S_IFLNK:
-				if ((i = readlink(file1, buf1, D_BUFSIZ)) == -1) {
+				if ((i = readlink(file1, buf1,
+							D_BUFSIZ)) == -1) {
 					(void) fprintf(stderr, gettext(
 					    "diff: cannot read link\n"));
 					return (2);
 				}
 
-				if ((j = readlink(file2, buf2, D_BUFSIZ)) == -1) {
+				if ((j = readlink(file2, buf2,
+							D_BUFSIZ)) == -1) {
 					(void) fprintf(stderr, gettext(
 					    "diff: cannot read link\n"));
 					return (2);
@@ -1915,9 +2245,9 @@ compare(dp)
 #endif
 			}
 		} else {
-			if (lflag)
+			if (lflag) {
 				dp->d_flags |= DIFFER;
-			else if (opt == D_NORMAL || opt == D_CONTEXT) {
+			} else if (opt == D_NORMAL || opt == D_CONTEXT) {
 /*
  * TRANSLATION_NOTE
  * The second and fourth parameters will take the gettext'ed string
@@ -1957,9 +2287,14 @@ gettext("File %s is %s while file %s is %s\n"),
 			goto notsame;
 		if (i == 0 && j == 0)
 			goto same;
+#ifdef	HAVE_MEMCMP
+		if (memcmp(buf1, buf2, i))
+			goto notsame;
+#else
 		for (j = 0; j < i; j++)
 			if (buf1[j] != buf2[j])
 				goto notsame;
+#endif
 	}
 same:
 	if (sflag == 0)
@@ -1975,7 +2310,8 @@ closem:
 	return (0);
 
 notsame:
-	if (binary(f1) || binary(f2)) {
+	if (!aflag &&
+	    (binary(f1) || binary(f2))) {
 		if (lflag)
 			dp->d_flags |= DIFFER;
 		else if (opt == D_NORMAL || opt == D_CONTEXT)
@@ -1985,16 +2321,27 @@ notsame:
 		(void) close(f1); (void) close(f2);
 		return (1);
 	}
+#ifdef	DO_SPAWN_DIFF
 	(void) close(f1); (void) close(f2);
+#endif
 	anychange = 1;
 	if (lflag) {
+#ifndef	DO_SPAWN_DIFF
+		(void) close(f1); (void) close(f2);
+#endif
 		result = calldiff(title);
 	} else {
 		if (opt == D_EDIT)
 			(void) printf("ed - %s << '-*-END-*-'\n", dp->d_entry);
 		else
 			(void) printf("%s%s %s\n", title, file1, file2);
+#ifdef	DO_SPAWN_DIFF
 		result = calldiff((char *)0);
+#else
+		stb1 = statb1;
+		stb2 = statb2;
+		result = calldiffreg(f1, f2);
+#endif
 		if (opt == D_EDIT)
 			(void) printf("w\nq\n-*-END-*-\n");
 	}
@@ -2020,16 +2367,15 @@ calldiff(wantpr)
 			error(gettext(NO_PROCS_ERR));
 
 		if (pid == 0) {
-#ifdef	set_child_standard_fds
+#ifdef	set_child_standard_fds	/* VMS */
 			set_child_standard_fds(pv[0],
 						STDOUT_FILENO,
 						STDERR_FILENO);
 #ifdef	F_SETFD
 			fcntl(pv[1], F_SETFD, 1);
 #endif
-#else
-			(void) close(0);
-			(void) dup(pv[0]);
+#else			/* ! VMS, below is the code for UNIX */
+			(void) dup2(pv[0], STDIN_FILENO);
 			(void) close(pv[0]);
 			(void) close(pv[1]);
 #endif
@@ -2039,6 +2385,7 @@ calldiff(wantpr)
 #ifdef	HAVE_VFORK
 			didvfork = 1;
 #endif
+			status = 2;
 			done();
 		}
 	}
@@ -2048,8 +2395,7 @@ calldiff(wantpr)
 
 	if (pid == 0) {
 		if (wantpr) {
-			(void) close(1);
-			(void) dup(pv[1]);
+			(void) dup2(pv[1], STDOUT_FILENO);
 			(void) close(pv[0]);
 			(void) close(pv[1]);
 		}
@@ -2062,9 +2408,10 @@ calldiff(wantpr)
 #ifdef	HAVE_VFORK
 		didvfork = 1;
 #endif
+		status = 2;
 		done();
 	}
-	if (wantpr)	{
+	if (wantpr) {
 		(void) close(pv[0]);
 		(void) close(pv[1]);
 	}
@@ -2072,10 +2419,10 @@ calldiff(wantpr)
 		continue;
 	while (wait((int *)0) != (pid_t)-1)
 		continue;
-	if ((diffstatus&0177) != 0)
-		return (2);
+	if (WIFEXITED(diffstatus))
+		return (WEXITSTATUS(diffstatus));
 	else
-		return ((diffstatus>>8) & 0377);
+		return (2);
 }
 
 #ifdef	PROTOTYPES
@@ -2273,6 +2620,9 @@ filename(pa1, pa2, st, ifile)
 	a1 = *pa1;
 	a2 = *pa2;
 
+	if (*ifile)
+		free(*ifile);
+
 	if (strcmp(*pa1, "-") == 0)
 		*ifile = strdup("-");
 	else
@@ -2287,6 +2637,12 @@ filename(pa1, pa2, st, ifile)
 
 	if ((st->st_mode & S_IFMT) == S_IFDIR) {
 		b1 = *pa1 = (char *)malloc(PATH_MAX);
+		if (b1 == (char *)NULL) {
+			(void) fprintf(stderr, gettext(
+				"no more memory - try again later\n"));
+			status = 2;
+			done();
+		}
 		while ((*b1++ = *a1++) != '\0')
 			;
 		b1[-1] = '/';
@@ -2294,6 +2650,7 @@ filename(pa1, pa2, st, ifile)
 		while ((*a1++ = *a2++) != '\0')
 			if (*a2 && *a2 != '/' && a2[-1] == '/')
 				a1 = b1;
+		free(*ifile);
 		*ifile = strdup(*pa1);
 
 		if (*ifile == (char *)NULL) {
@@ -2306,15 +2663,17 @@ filename(pa1, pa2, st, ifile)
 		if (stat(*pa1, st) < 0) {
 			(void) fprintf(stderr, "diff: ");
 			perror(*pa1);
+			status = 2;
 			done();
 		}
-	} else if ((st->st_mode & S_IFMT) == S_IFCHR)
+	} else if ((st->st_mode & S_IFMT) == S_IFCHR) {
 		*pa1 = copytemp(a1);
-	else if (a1[0] == '-' && a1[1] == 0) {
+	} else if (a1[0] == '-' && a1[1] == 0) {
 		*pa1 = copytemp(a1);	/* hack! */
 		if (stat(*pa1, st) < 0) {
 			(void) fprintf(stderr, "diff: ");
 			perror(*pa1);
+			status = 2;
 			done();
 		}
 	}
@@ -2375,12 +2734,26 @@ prepare(i, arg)
 {
 	struct line *p;
 	int j, h;
+	size_t	isize;
+
+	/*
+	 * Average line length is aprox. 35 chars.
+	 */
+	isize = (i == 0 ? stb1.st_size : stb2.st_size) / 35;
+	if (isize > 1000000)
+		isize = 1000000;
+	if (isize < 64)
+		isize = 64;
 
 	(void) fseek(input[i], (off_t)0, SEEK_SET);
-	p = (struct line *)talloc(3 * sizeof (line));
+	p = (struct line *)talloc((isize + 3) * sizeof (line));
 	for (j = 0; (h = readhash(input[i], i, arg)) != 0; ) {
-		p = (struct line *)ralloc((void *)p, (++j + 3) * sizeof (line));
-		p[j].value = h;
+		if (j >= isize) {
+			isize += 64;
+			p = (struct line *)ralloc((void *)p,
+					(isize + 3) * sizeof (line));
+		}
+		p[++j].value = h;
 	}
 	len[i] = j;
 	file[i] = p;
@@ -2392,13 +2765,13 @@ prune()
 	int i, j;
 
 	for (pref = 0; pref < len[0] && pref < len[1] &&
-			file[0][pref + 1].value == file[1][pref + 1].value;
+	    file[0][pref + 1].value == file[1][pref + 1].value;
 	    pref++)
 		;
 	for (suff = 0; (suff < len[0] - pref) &&
-			(suff < len[1] - pref) &&
-			(file[0][len[0] - suff].value ==
-			file[1][len[1] - suff].value);
+	    (suff < len[1] - pref) &&
+	    (file[0][len[0] - suff].value ==
+	    file[1][len[1] - suff].value);
 	    suff++)
 		;
 
@@ -2477,21 +2850,23 @@ static void
 usage()
 {
 	(void) fprintf(stderr, gettext(
-	"usage: diff [-bitw] [-c | -e | -f | -h | -n | -u] file1 file2\n\
-       diff [-bitw] [-C number | -U number] file1 file2\n\
-       diff [-bitw] [-D string] file1 file2\n\
-       diff [-bitw] [-c | -e | -f | -h | -n | -u] [-l] [-r] [-s] [-S name] directory1 directory2\n"));
+	"usage: diff [-abBiNptw] [-c | -e | -f | -h | -n | -q | -u] \
+file1 file2\n\
+       diff [-abBiNptw] [-C number | -U number] file1 file2\n\
+       diff [-abBiNptw] [-D string] file1 file2\n\
+       diff [-abBiNptw] [-c | -e | -f | -h | -n | -q | -u] [-l] [-r] \
+[-s] [-S name] directory1 directory2\n"));
 	status = 2;
 	done();
 }
 
 #define	NW	1024
 struct buff	{
-	FILE	*iop;	/* I/O stream */
+	FILE	*iop;			/* I/O stream */
 	char	buf[NW + MB_LEN_MAX];	/* buffer */
-	char	*ptr;	/* current pointer in the buffer */
-	int	buffered;	/* if non-zero, buffer has data */
-	off_t	offset;	/* offset in the file */
+	char	*ptr;			/* current pointer in the buffer */
+	int	buffered;		/* if non-zero, buffer has data */
+	off_t	offset;			/* offset in the file */
 };
 
 static struct buff bufwchar[2];
@@ -2659,3 +3034,149 @@ getbufwchar(filen, blen)
 		return ((wint_t)wc);
 	}
 }
+
+#if !defined(HAVE_CFTIME) && defined(HAVE_STRFTIME)
+static time_t
+gmtoff(clk)
+	const time_t	*clk;
+{
+	struct tm	local;
+	struct tm	gmt;
+	time_t		crtime;
+
+	local = *localtime(clk);
+	gmt   = *gmtime(clk);
+
+	local.tm_sec  -= gmt.tm_sec;
+	local.tm_min  -= gmt.tm_min;
+	local.tm_hour -= gmt.tm_hour;
+	local.tm_yday -= gmt.tm_yday;
+	local.tm_year -= gmt.tm_year;
+	if (local.tm_year)		/* Hit new-year limit	*/
+		local.tm_yday = local.tm_year;	/* yday = +-1	*/
+
+	crtime = local.tm_sec + 60 *
+		    (local.tm_min + 60 *
+			(local.tm_hour + 24 * local.tm_yday));
+
+	return (crtime);
+}
+#endif
+
+#define	DO2(p, n, c)	*p++ = ((char)((n)/10) + '0'); *p++ = \
+					((char)((n)%10) + '0'); *p++ = c;
+
+#define	DO2_(p, n)	*p++ = ((char)((n)/10) + '0'); *p++ = \
+					((char)((n)%10) + '0');
+
+static void
+cf_time(s, maxsize, fmt, clk)
+	char		*s;
+	size_t		maxsize;
+	char		*fmt;
+	const time_t	*clk;
+{
+#ifdef	HAVE_CFTIME
+	cftime(s, fmt, clk);
+#else
+#ifdef	HAVE_STRFTIME
+	struct	tm	*tp = localtime(clk);
+	char		*p;
+
+	strftime(s, maxsize, fmt, tp);
+	/*
+	 * HP/UX implements %z as %Z and we need to correct this...
+	 */
+	p = fmt + strlen(fmt);
+	if (*--p != 'z')
+		return;
+	p = strrchr(s, ' ');
+	if (p++) {
+		if (*p != '+' && *p != '-') {
+			register int	z;
+			register int	n;
+
+			z = gmtoff(clk) / 60;	/* seconds -> minutes */
+			if (z < 0) {
+				*p++ = '-';
+				z = -z;
+			} else {
+				*p++ = '+';
+			}
+			n = z / 60;
+			DO2_(p, n);
+			n = z % 60;
+			DO2(p, n, 0);
+		}
+	}
+#else
+	/*
+	 * This is not the correct time format, but we are not on a POSIX
+	 * platform and need to do the best we can.
+	 */
+	strlcpy(s, ctime(clk), maxsize);
+	if (maxsize > 24)
+		s[24] = '\0';
+#endif
+#endif
+}
+
+/*
+ * The next function has been imported from OpenBSD.
+ * Original author is: Otto Moerbeek <otto@drijf.net>
+ */
+#define	begins_with(s, pre)	(strncmp(s, pre, sizeof (pre)-1) == 0)
+#define	C			(char *)
+
+static char *
+match_function(f, pos, filen)
+	const off_t	*f;
+	int		pos;
+	int		filen;
+{
+	unsigned char	buf[FUNCTION_CONTEXT_SIZE];
+	size_t		nc;
+	int		last = lastline;
+	char		*state = NULL;
+	FILE		*fp = bufwchar[filen].iop;
+	off_t		off;
+
+	off = ftellbuf(filen);
+	lastline = pos;
+	while (pos > last) {
+		fseek(fp, f[pos - 1], SEEK_SET);
+		nc = f[pos] - f[pos - 1];
+		if (nc >= sizeof (buf))
+			nc = sizeof (buf) - 1;
+		nc = fread(buf, 1, nc, fp);
+		if (nc > 0) {
+			buf[nc] = '\0';
+			buf[strcspn(C buf, "\n")] = '\0';
+			if (isalpha(buf[0]) || buf[0] == '_' || buf[0] == '$') {
+				if (begins_with(C buf, "private:")) {
+					if (!state)
+						state = " (private)";
+				} else if (begins_with(C buf, "protected:")) {
+					if (!state)
+						state = " (protected)";
+				} else if (begins_with(C buf, "public:")) {
+					if (!state)
+						state = " (public)";
+				} else {
+					strlcpy(lastbuf, C buf,
+					    sizeof (lastbuf));
+
+					if (state)
+						strlcat(lastbuf, state,
+						    sizeof (lastbuf));
+					lastmatchline = pos;
+					initbuf(fp, filen, off);
+					return (lastbuf);
+				}
+			}
+		}
+		pos--;
+	}
+	initbuf(fp, filen, off);
+	return (lastmatchline > 0 ? lastbuf : NULL);
+}
