@@ -3,7 +3,7 @@
  *
  * The contents of this file are subject to the terms of the
  * Common Development and Distribution License ("CDDL"), version 1.0.
- * You may only use this file in accordance with the terms of version
+ * You may use this file only in accordance with the terms of version
  * 1.0 of the CDDL.
  *
  * A full copy of the text of the CDDL should have accompanied this
@@ -27,12 +27,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2015 J. Schilling
+ * Copyright 2006-2018 J. Schilling
  *
- * @(#)get.c	1.69 15/03/10 J. Schilling
+ * @(#)get.c	1.83 18/12/17 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)get.c 1.69 15/03/10 J. Schilling"
+#pragma ident "@(#)get.c 1.83 18/12/17 J. Schilling"
 #endif
 /*
  * @(#)get.c 1.59 06/12/12
@@ -134,7 +134,7 @@ register char *argv[];
 	 */
 #ifdef	PROTOTYPES
 	(void) bindtextdomain(NOGETTEXT("SUNW_SPRO_SCCS"),
-	   NOGETTEXT(INS_BASE "/ccs/lib/locale/"));
+	   NOGETTEXT(INS_BASE "/" SCCS_BIN_PRE "lib/locale/"));
 #else
 	(void) bindtextdomain(NOGETTEXT("SUNW_SPRO_SCCS"),
 	   NOGETTEXT("/usr/ccs/lib/locale/"));
@@ -173,7 +173,7 @@ register char *argv[];
 			}
 			no_arg = 0;
 			i = current_optind;
-		        c = getopt(argc, argv, "-r:c:ebi:x:kl:Lpsmnogta:G:w:zqdC:AFN:V(version)");
+		        c = getopt(argc, argv, "()-r:c:ebi:x:kl:Lpsmnogta:G:w:zqdC:AFN:V(version)");
 				/* this takes care of options given after
 				** file names.
 				*/
@@ -228,7 +228,11 @@ register char *argv[];
 			case 'c':
 				if (*p == 0) continue;
 				cutoffstr = p;
+#if defined(BUG_1205145) || defined(GMT_TIME)
+				if (parse_date(p,&cutoff, 0))
+#else
 				if (parse_date(p,&cutoff, PF_GMT))
+#endif
 				   fatal(gettext("bad date/time (cm5)"));
 				break;
 			case 'L':
@@ -308,7 +312,8 @@ register char *argv[];
 				break;
 
 			case 'V':		/* version */
-				printf("get %s-SCCS version %s %s (%s-%s-%s)\n",
+				printf(gettext(
+				    "get %s-SCCS version %s %s (%s-%s-%s)\n"),
 					PROVIDER,
 					VERSION,
 					VDATE,
@@ -316,7 +321,7 @@ register char *argv[];
 				exit(EX_OK);
 
 			default:
-			   fatal(gettext("Usage: get [-AbeFgkmLopst] [-l[p]] [-asequence] [-cdate-time] [-Gg-file]\n\t[-isid-list] [-rsid] [-xsid-list][ -N[bulk-spec]] file ..."));
+			   fatal(gettext("Usage: get [-AbeFgkmLopst] [-l[p]] [-asequence] [-cdate-time] [-Gg-file]\n\t[-isid-list] [-rsid] [-xsid-list][ -N[bulk-spec]] s.filename ..."));
 			}
 
 			/*
@@ -340,13 +345,21 @@ register char *argv[];
 		HADK = 1;
 	if (HADE && !logname())
 		fatal(gettext("User ID not in password file (cm9)"));
-	if (HADUCN) {					/* Parse -N args  */
-		parseN(&N);
-	}
+
 	setsig();
 	xsethome(NULL);
-	if (HADUCN && N.n_sdot && (sethomestat & SETHOME_OFFTREE))
-		fatal(gettext("-Ns. not supported in off-tree project mode"));
+	if (HADUCN) {					/* Parse -N args  */
+		/*
+		 * initN() was already called while parsing options.
+		 */
+		if (!HADP && !HADG && !HADUCG)		/* Create g-file, so */
+			N.n_flags |= N_GETI;		/* create subdirs    */
+			
+		parseN(&N);
+
+		if (N.n_sdot && (sethomestat & SETHOME_OFFTREE))
+			fatal(gettext("-Ns. not supported in off-tree project mode"));
+	}
 
 	Fflags &= ~FTLEXIT;
 	Fflags |= FTLJMP;
@@ -437,6 +450,8 @@ get(pkt, file)
 	pkt->p_cutoff = cutoff;
 	pkt->p_lfile = lfile;
 	pkt->p_enter = enter;
+#if defined(BUG_1205145) || defined(GMT_TIME)
+#else
 	if ((pkt->p_flags & PF_V6) == 0) {
 		pkt->p_flags |= PF_GMT;
 	} else if (cutoffstr != NULL) {
@@ -444,6 +459,7 @@ get(pkt, file)
 			fatal(gettext("bad date/time (cm5)"));
 		pkt->p_cutoff = cutoff;
 	}
+#endif
 	if (HADUCA)
 		pkt->p_pgmrs = (char **)Null;
 	if (Gfile[0] == 0 || !first) {
@@ -471,6 +487,8 @@ get(pkt, file)
 		fmterr(pkt);
 	finduser(pkt);
 	doflags(pkt);
+	donamedflags(pkt);
+	dometa(pkt);
 
 	if (!HADA) {
 		ser = getser(pkt);
@@ -518,7 +536,7 @@ get(pkt, file)
 			gen_lfile(pkt);
 		if (HADG)
 			goto unlock;
-		flushto(pkt, EUSERTXT, 1);
+		flushto(pkt, EUSERTXT, FLUSH_NOCOPY);
 		idsetup(pkt, gfile);
 		pkt->p_chkeof = 1;
 		pkt->p_did_id = 0;
@@ -631,7 +649,10 @@ get(pkt, file)
 		if (Gfile[0] != '\0' && exists(Gfile)) {
 			rename(Gfile, gfile);
 			if (HADO) {
+#if defined(BUG_1205145) || defined(GMT_TIME)
+#else
 				struct tm	tm;
+#endif
 				struct timespec	ts[2];
 				unsigned int	gser;
 				extern dtime_t	Timenow;
@@ -647,21 +668,35 @@ get(pkt, file)
 				 * table and converted the time stamps assuming
 				 * GMT. Fix the resulting error here.
 				 */
+#if defined(BUG_1205145) || defined(GMT_TIME)
+#else
 				if (pkt->p_flags & PF_GMT) {
 					tm = *gmtime(&ts[1].tv_sec);
 					tm.tm_isdst = -1;
 					ts[1].tv_sec = mktime(&tm);
 				}
+#endif
+				/*
+				 * As SunPro make and gmake call sccs
+				 * get when the time if s.file equals
+				 * the time stamp of the g-file, make
+				 * sure the g-file is a bit younger.
+				 */
+				if (!(gpkt.p_flags & PF_V6)) {
+					struct timespec	tn;
+
+					getnstimeofday(&tn);
+					ts[1].tv_nsec = tn.tv_nsec;
+				}
+				if (ts[1].tv_nsec <= 500000000)
+					ts[1].tv_nsec += 499999999;
+
 				utimensat(AT_FDCWD, gfile, ts, 0);
 			}
 		}
 		setuid(holduid);
 		setgid(holdgid);
 	}
-	if (pkt->p_iop) {
-		fclose(pkt->p_iop);
-		pkt->p_iop = NULL;
-	}
 unlock:
 	if (HADE) {
 		struct utsname	un;
@@ -673,7 +708,11 @@ unlock:
 		uuname = un.nodename;
 		unlockit(auxf(pkt->p_file, 'z'), getpid(), uuname);
 	}
+	sclose(pkt);
+	sfree(pkt);
 	ffreeall();
+	if (HADUCA)				/* ffreeall() killed it	*/
+		lhash_destroy();		/* need to reset it	*/
 }
 
 static void
