@@ -3,7 +3,7 @@
  *
  * The contents of this file are subject to the terms of the
  * Common Development and Distribution License ("CDDL"), version 1.0.
- * You may only use this file in accordance with the terms of version
+ * You may use this file only in accordance with the terms of version
  * 1.0 of the CDDL.
  *
  * A full copy of the text of the CDDL should have accompanied this
@@ -27,14 +27,14 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2014 J. Schilling
+ * Copyright 2006-2018 J. Schilling
  *
- * @(#)defines.h	1.76 14/08/09 J. Schilling
+ * @(#)defines.h	1.110 18/12/16 J. Schilling
  */
 #ifndef	_HDR_DEFINES_H
 #define	_HDR_DEFINES_H
 #if defined(sun)
-#pragma ident "@(#)defines.h 1.76 14/08/09 J. Schilling"
+#pragma ident "@(#)defines.h 1.110 18/12/16 J. Schilling"
 #endif
 /*
  * @(#)defines.h 1.21 06/12/12
@@ -53,7 +53,7 @@
 # include	<schily/fcntl.h>
 # include	<schily/stdio.h>
 # include	<schily/stdlib.h>
-# include	<schily/varargs.h>	/* needed for the vfprintf() prototype */
+# include	<schily/varargs.h>	/* needed for vfprintf() prototype */
 # include	<schily/unistd.h>
 # include	<schily/string.h>
 # include	<schily/standard.h>	/* define signed */
@@ -120,13 +120,20 @@ extern char *optarg;
 #define	NEED_SCHILY_PRINT	/* We need defines for js_snprintf() */
 #endif
 
+#if	(!defined(HAVE_SNPRINTF) && !defined(NO_SNPRINTF)) || \
+	defined(NEED_PRINTF_Z) && !defined(HAVE_PRINTF_Z)
+#define	NEED_SCHILY_PRINT	/* We need defines for js_snprintf() */
+#endif
+
 /*
  * Always include schily/schily.h to allow POSIX bug workarounds to be
  * implemented in schily/schily.h.
  */
 #ifdef	NEED_SCHILY_PRINT
+#ifndef	SCHILY_PRINT
 #define	SCHILY_PRINT
 #endif
+#endif
 #include	<schily/schily.h>
 
 #if SIZEOF_TIME_T == 4
@@ -165,8 +172,13 @@ extern time_t	Y1969;			/* defined in lib/comobj/src/xtzset.c */
 #define	MAX_TIME	(time_t)_YM9999	/* We currently support 4 digit years */
 #endif
 
-#define	ALIGNMENT  	(sizeof (long long))
-#define	ROUND(x,base)   (((x) + (base-1)) & ~(base-1))
+/*
+ * The epoche start for urandom() time -> rand computation:
+ */
+#define	URAND_BASE	0x50000000	/* 2012 Jul 13 11:01:20 UTC */
+
+#define	ALIGNMENT	(sizeof (long long))
+#define	ROUND(x, base)	(((x) + (base-1)) & ~(base-1))
 
 # define CTLSTR		"%c%c\n"
 
@@ -185,7 +197,7 @@ extern time_t	Y1969;			/* defined in lib/comobj/src/xtzset.c */
 # define EDELTAB	'e'	/* ^Ae the end of a delta table		    */
 
 # define BUSERNAM	'u'	/* ^Au begin list of allowed delta users    */
-# define EUSERNAM	'U'	/* ^AU end list of allowed delta users      */
+# define EUSERNAM	'U'	/* ^AU end list of allowed delta users	    */
 
 #define	NFLAGS	28
 #define	NLOWER		('z'-'a'+1)
@@ -234,18 +246,30 @@ extern	char	saveid[50];	/* defined in lib/comobj/src/logname.c */
 
 # define MINR		1		/* minimum release number */
 # define MAXR		9999		/* maximum release number */
-# define FILESIZE	MAXPATHLEN
-# define MAXLINE	BUFSIZ
-# define DEF_LINE_SIZE	128
-# undef	MAX
-# define MAX		9999
-# define DELIVER	'*'
+# define MAXL		9999		/* max level for dolist() -> maxint? */
+#ifndef	MINFBFSZ
+# define MINFBFSZ	2048	/* Mininum File Buffer Size */
+#endif
+# define FILESIZE	max(MAXPATHLEN, MINFBFSZ) /* Space for path names */
+#ifndef	MINLINESZ
+# define MINLINESZ	8192	/* Minimum Line Buffer Size */
+#endif
+# define MAXLINE	max(BUFSIZ, MINLINESZ)	/* Buffer size for various */
+# define DEF_LINE_SIZE	128		/* start size for alocated lines */
+# define DELIVER	'*'		/* MR number was "delivered" no rmdel */
 # define LOGSIZE	(33)		/* TWCP SCCS compatibility */
-# define MAXERRORLEN	(1024+MAXPATHLEN)	/* max length of SccsError buffer */
+# define MAXERRORLEN	(1024+FILESIZE)	/* max length of SccsError buffer */
 
 # define FAILPUT    fatal("fputs could not write to file (ut13)")
-# define SCCS_LOCK_ATTEMPTS	4       /* maximum number of lock   attempts  */
-# define SCCS_CREAT_ATTEMPTS	4       /* maximum number of create attempts  */
+# define SCCS_LOCK_ATTEMPTS	4	/* maximum number of lock   attempts  */
+# define SCCS_CREAT_ATTEMPTS	4	/* maximum number of create attempts  */
+
+/*
+ * The second argument for the putmeta() function:
+ */
+#define	M_INIT_PATH		1
+#define	M_URAND			2
+#define	M_ALL			0xffffffff
 
 /*
  * The third argument for the function flushto() controls whether the data
@@ -258,8 +282,21 @@ extern	char	saveid[50];	/* defined in lib/comobj/src/logname.c */
 # define FLUSH_COPY_UNTIL	2	/* Copy read lines until before match */
 
 /*
-	SCCS Internal Structures.
-*/
+ * The third argument for the function sinit() is a flag that controls the
+ * behavior. Unless SI_FORCE is used, the filename is checked to be either
+ * "s.name" or "xxx/s.name". SI_FORCE allows us to open the "x.file" for
+ * reading.
+ */
+# define SI_INIT		0	/* Check filename and init pkt	    */
+# define SI_OPEN		1	/* Check filename init pkt open file */
+# define SI_FORCE		2	/* Init pkt & open any file name    */
+
+# define SI_FORCEOPEN		(SI_OPEN|SI_FORCE)
+
+
+/*
+ *	SCCS Internal Structures.
+ */
 
 /*
  * Definitions for date+time
@@ -298,21 +335,21 @@ struct apply {
 /*
  * Definitions for a_code
  */
-#define SX_EMPTY	(0)
-#define APPLY		(1)
-#define NOAPPLY		(2)
+#define	SX_EMPTY	(0)
+#define	APPLY		(1)
+#define	NOAPPLY		(2)
 
 /*
  * Definitions for a_reason
  */
-# define IGNR		0100
-# define USER		040
-# define INCL		1
-# define EXCL		2
-# define CUTOFF		4
-# define INCLUSER	(USER | INCL)
-# define EXCLUSER	(USER | EXCL)
-# define IGNRUSER	(USER | IGNR)
+#define	IGNR		0100
+#define	USER		040
+#define	INCL		1
+#define	EXCL		2
+#define	CUTOFF		4
+#define	INCLUSER	(USER | INCL)
+#define	EXCLUSER	(USER | EXCL)
+#define	IGNRUSER	(USER | IGNR)
 
 struct queue {
 	struct queue *q_next;
@@ -323,16 +360,70 @@ struct queue {
 	char	q_user;		/* inex'ed by user */
 };
 
-#define YES	(1)
-#define NO	(2)
+#define	YES	(1)
+#define	NO	(2)
 
 struct	sid {
-	int	s_rel;
-	int	s_lev;
-	int	s_br;
-	int	s_seq;
+	int	s_rel;		/* Release			*/
+	int	s_lev;		/* Level			*/
+	int	s_br;		/* Branch			*/
+	int	s_seq;		/* Sequence (if branch != 0)	*/
 };
 
+/*
+ * String space needed for the SID.
+ *
+ * Currently s_rel is limited to the range MINR .. MAXR and as long
+ * as MAXR stays 1ith 9999, 38 chars would be enough. It is however
+ * safer to permit the whole range for SID -> string conversion.
+ */
+#define	SID_STRSIZE	45	/* 4 * 10 + 3 * '.' + '\0' = 44 */
+
+/*
+ * Parameters for the -N option:
+ */
+typedef struct Nparms {
+	char	n_comma;		/* Found -N,*			*/
+	char	n_unlink;		/* Found -N-*			*/
+	char	n_get;			/* Found -N+* or -N++*		*/
+	char	n_sdot;			/* Found -N*s.			*/
+	char	n_subd;			/* Found -NSCCS			*/
+	char	*n_prefix;		/* "s." /  "* /s." / "* /SCCS"	*/
+	char	*n_parm;		/* -N argument			*/
+	char	*n_ifile;		/* The plain g-file name	*/
+	char	*n_dir_name;		/* directory for -N		*/
+	struct sid	n_sid;		/* SID if n_get == 2		*/
+	struct timespec	n_mtime;	/* Timestamp for -i -o		*/
+	unsigned n_flags;		/* Various flags		*/
+} Nparms;
+
+/*
+ * Definitions for n_flags above
+ */
+#define	N_IFILE		0x0001		/* admin: -i has been specified  */
+#define	N_IDOT		0x0002		/* admin: -i. has been specified */
+#define	N_NFILE		0x0004		/* admin: -n has been specified  */
+#define	N_GETI		0x0008		/* get: -p has not been specified */
+
+/*
+ * Parameters for the -X option:
+ */
+typedef struct Xparms {
+	char	*x_parm;		/* -X argument			*/
+	char	*x_init_path;		/* -XGp Initial path		*/
+	urand_t	x_rand;			/* -XGr Unified random number	*/
+	unsigned x_opts;		/* Options seen			*/
+	unsigned x_flags;		/* Various flags		*/
+} Xparms;
+
+/*
+ * Definitions for x_opts / x_flags above
+ */
+#define	XO_PREPEND_FILE	0x01		/* Optimized mode for Changeset files */
+#define	XO_INIT_PATH	0x10		/* -XGp Initial path		*/
+#define	XO_URAND	0x20		/* -XGr Unified random number	*/
+
+
 struct	deltab {
 	struct	sid	d_sid;
 	int	d_serial;
@@ -410,6 +501,7 @@ struct packet {
 	int	p_glines;	/* number of lines in current gfile */
 	int	p_glnno;	/* line number of current gfile line */
 	int	p_slnno;	/* line number of current input line */
+	int	p_insser;	/* serial number which inserted current line */
 	char	p_wrttn;	/* written flag (!0 = written) */
 	char	p_keep;		/* keep switch for readmod() */
 	char	p_encoding;	/* encoding flags for data */
@@ -430,6 +522,7 @@ struct packet {
 	time_t	p_cdt;		/* date/time of newest applied delta */
 	char	*p_lfile;	/* 0 = no l-file; else ptr to l arg */
 	struct	idel *p_idel;	/* ptr to internal delta table */
+	char	**p_pgmrs;	/* ptr to table of delta users */
 	FILE	*p_stdout;	/* standard output for warnings and messages */
 	FILE	*p_gout;	/* g-file output file */
 	char	p_user;		/* !0 = user on user list */
@@ -447,7 +540,9 @@ struct packet {
 	/*
 	 * Various other s-file specific data
 	 */
-	void	(*p_enter) __PR((struct packet *pkt, int ch, int n, struct sid *sidp));
+	void	(*p_enter) __PR((struct packet *pkt,
+					int ch, int n,
+					struct sid *sidp));
 	void	(*p_escdodelt) __PR((struct packet *pkt));
 	void	(*p_fredck) __PR((struct packet *pkt));
 	char	p_cdid_mrs;	/* for cdc to check MRs */
@@ -505,10 +600,14 @@ struct	pfile	{
 };
 
 /*
- Declares for external functions in lib/cassi
-*/
-extern	char*	gf	__PR((char *));
-extern	int	sweep	__PR((int, char *, char *, int, int, int, char **, char *, char **, int (*)(char *, int, char **), int (*)(char **, char **, int)));
+ * Declares for external functions in lib/cassi
+ */
+extern	char	*gf	__PR((char *));
+extern	int	sweep	__PR((int, char *, char *, int, int, int, char **,
+				char *, char **,
+				int (*)(char *, int, char **),
+				int (*)(char **, char **,
+				int)));
 extern	int	cmrcheck __PR((char *, char *));
 extern	int	deltack __PR((char [], char *, char *, char *, char *[]));
 extern	void	cmrerror __PR((const char *));
@@ -516,34 +615,67 @@ extern	void	cmrerror __PR((const char *));
 /*
  * Declares for external variables in lib/comobj
  */
-extern	char	*sccs_insbase;
-extern	char	*Comments;
-extern	char	*Mrs;
-extern	int	Domrs;
+extern	char	*sccs_insbase;	/* Installation base dir or "/usr"	  */
+extern	char	*Comments;	/* Comments from -y option		  */
+extern	char	*Mrs;		/* List of Mr numbers from -m option	  */
+extern	int	Domrs;		/* 'v' flag was found in the histroy file */
+
+				/*
+				 * If it is not possible to retrieve "setahome",
+				 * it may be a NULL pointer. The variables
+				 * "setrhome" and "cwdprefix" are always set
+				 * from sethome().
+				 */
+extern	char	*setphome;	/* Best path to the project set home dir */
+extern	char	*setrhome;	/* Relative path to the project set home dir */
+extern	char	*setahome;	/* Absolute path to the project set home dir */
+extern	char	*cwdprefix;	/* Prefix from project set home dir to cwd */
+extern	int	homedist;	/* The # of dirs to the project set home dir */
+extern	int	setphomelen;	/* strlen(setphome) */
+extern	int	setrhomelen;	/* strlen(setrhome) */
+extern	int	setahomelen;	/* strlen(setahome) */
+extern	int	cwdprefixlen;	/* strlen(cwdprefix) */
+extern	int	sethomestat;	/* sethome() status flags */
 
 /*
- Declares for external functions in lib/comobj
-*/
+ * sethome() status flags:
+ */
+#define	SETHOME_NONE	0	/* $SET_HOME/.sccs could not be found	*/
+#define	SETHOME_OK	1	/* $SET_HOME/.sccs was found		*/
+#define	SETHOME_ABS	2	/* Using setahome is preferred		*/
+#define	SETHOME_INTREE	4	/* $SET_HOME/.sccs/data missing, SCCS in tree */
+#define	SETHOME_OFFTREE	8	/* $SET_HOME/.sccs/data found, SCCS off tree */
+#define	SETHOME_DELS_OK 16	/* $SET_HOME/.sccs/dels was found	*/
 
-extern	char*	auxf	__PR((char *, int));
+#define	SETHOME_ALL_OK	(SETHOME_OK|SETHOME_DELS_OK)
+
+#define	SETHOME_INIT()	((sethomestat & SETHOME_ALL_OK) == SETHOME_ALL_OK)
+
+/*
+ * Declares for external functions in lib/comobj
+ */
+
+extern	char	*auxf	__PR((char *, int));
 extern	void	sinit	__PR((struct packet *, char *, int));
+extern	void	sclose	__PR((struct packet *));
+extern	void	sfree	__PR((struct packet *));
 extern	char	*checkmagic __PR((struct packet *, char *));
 extern	void	setup	__PR((struct packet *, int));
 extern	void	finduser __PR((struct packet *));
 extern	void	permiss	__PR((struct packet *));
-extern	char*	sid_ab	__PR((char *, struct sid *));
-extern	char*	sid_ba	__PR((struct sid *, char *));
+extern	char	*sid_ab	__PR((char *, struct sid *));
+extern	char	*sid_ba	__PR((struct sid *, char *));
 extern	void	sidext_ab __PR((struct packet *, struct deltab *, char *));
 extern	void	sidext_v4compat_ab __PR((struct packet *, struct deltab *));
 extern	void	sidext_ba __PR((struct packet *, struct deltab *));
-extern	char*	omit_sid __PR((char *));
+extern	char	*omit_sid __PR((char *));
 extern	int	date_ab	__PR((char *, time_t *, int flags));
 extern	int	date_abz __PR((char *, dtime_t *, int flags));
-extern	char*	date_ba	__PR((time_t *, char *, int flags));
-extern	char*	date_bal __PR((time_t *, char *, int flags));
-extern	char*	date_bazl __PR((dtime_t *, char *, int flags));
+extern	char	*date_ba __PR((time_t *, char *, int flags));
+extern	char	*date_bal __PR((time_t *, char *, int flags));
+extern	char	*date_bazl __PR((dtime_t *, char *, int flags));
 extern	char	del_ab	__PR((char *, struct deltab *, struct packet *));
-extern	char*	del_ba	__PR((struct deltab *, char *, int flags));
+extern	char	*del_ba	__PR((struct deltab *, char *, int flags));
 extern	void	stats_ab __PR((struct packet *, struct stats *));
 extern	void	pf_ab	__PR((char *, struct pfile *, int));
 extern	int	getser	__PR((struct packet *));
@@ -551,15 +683,20 @@ extern	int	sidtoser __PR((struct sid *, struct packet *));
 extern	int	eqsid	__PR((struct sid *, struct sid *));
 extern	void	chksid	__PR((char *, struct sid *));
 extern	void	newsid	__PR((struct packet *, int));
-extern	void	newstats __PR((struct packet *, char *, char *)); 
+extern	void	newstats __PR((struct packet *, char *, char *));
 extern	void	condset	__PR((struct apply *, int, int));
 extern	void	dolist	__PR((struct packet *, char *, int));
 extern	void	dohist	__PR((char *));
 extern	void	doie	__PR((struct packet *, char *, char *, char *));
 extern	void	doflags	__PR((struct packet *));
+extern	void	donamedflags	__PR((struct packet *));
 extern	void	dometa	__PR((struct packet *));
-extern	struct idel *dodelt __PR((struct packet *, struct stats *, struct sid *, int));
+extern	struct idel *dodelt __PR((struct packet *,
+				struct stats *, struct sid *, int));
 extern	void	do_file __PR((char *, void (*func)(char *), int, int));
+extern	void	doget	__PR((char *afile, char *gname, int ser));
+extern	void	dogtime	__PR((struct packet *pkt, char *gfile,
+				struct timespec *mtime));
 extern	void	fmterr	__PR((struct packet *));
 /*
  * Deal with unfriendly and non POSIX compliant glibc that defines getline()
@@ -567,15 +704,17 @@ extern	void	fmterr	__PR((struct packet *));
 #undef	getline
 #define	getline	comgetline
 extern	char	*getline __PR((struct packet *));
+extern	void	grewind	__PR((struct packet *));
 extern	void	putline	__PR((struct packet *, char *));
 extern	void	putchr	__PR((struct packet *, int c));
 extern	void	putctl	__PR((struct packet *));
 extern	void	putctlnnl __PR((struct packet *));
 extern	void	putmagic __PR((struct packet *, char *));
-extern	void	putmeta	__PR((struct packet *));
-extern	char*	logname	__PR((void));
+extern	void	putmeta	__PR((struct packet *, unsigned flags));
+extern	char	*logname	__PR((void));
+extern	char	*sccs_user	__PR((char *));
 extern	int	mystrptime __PR((char *, struct tm *, int));
-extern	char*	savecmt	__PR((char *));
+extern	char	*savecmt	__PR((char *));
 extern	void	mrfixup	__PR((void));
 extern	void	xrm	__PR((struct packet *));
 extern	void	flushto	__PR((struct packet *, int, int));
@@ -590,11 +729,14 @@ extern	int	cmpdate	__PR((struct tm *, struct tm *));
 extern	void	addq	__PR((struct packet *, int, int, int, int));
 extern	void	remq	__PR((struct packet *, int));
 extern	void	setkeep	__PR((struct packet *));
-extern	void	get_Del_Date_time __PR((char *, struct deltab *, struct packet *, struct tm *));
-extern	char*	stalloc	__PR((unsigned int));
+extern	void	get_Del_Date_time __PR((char *, struct deltab *,
+					struct packet *, struct tm *));
+extern	char	*stalloc	__PR((unsigned int));
 extern	int	mosize	__PR((int y, int t));
-extern	int	gN	__PR((char *str, char **next, int num, int *digits, int *chars));
-extern	int	gNp	__PR((char *str, char **next, int num, int *digits, int *chars));
+extern	int	gN	__PR((char *str, char **next, int num,
+					int *digits, int *chars));
+extern	int	gNp	__PR((char *str, char **next, int num,
+					int *digits, int *chars));
 extern	int	gns	__PR((char *str, char **next));
 extern	int	gtz	__PR((char *str, char **next));
 extern	void	xtzset	__PR((void));
@@ -607,19 +749,44 @@ extern  void	set_init_path __PR((struct packet *, char *file, char *dir));
 extern	void	sccs_setinsbase __PR((char *));
 extern	int	sccsfatalhelp __PR((char *));
 extern	int	sccshelp __PR((FILE *, char *));
+extern	size_t	lhash_size	__PR((size_t size));
+extern	void	lhash_destroy	__PR((void));
+extern	char	*lhash_add	__PR((char *str));
+extern	char	*lhash_lookup	__PR((char *str));
+extern	void	sccs_xpg4 __PR((int val));
+extern	void	whatsetup __PR((char *w));
+extern	void	idsetup __PR((struct packet *pkt, char *gname));
+extern	char *	idsubst __PR((struct packet *pkt, char *line));
+extern	char *	getmodname __PR((void));
+extern	char *	urand_ab __PR((char *aurand, urand_t *urandp));
+extern	char *	urand_ba __PR((urand_t *urandp,
+					char *aurand, size_t aurandsize));
+extern	int	urand_valid __PR((urand_t *urandp));
+extern	void	change_ba __PR((struct packet *pkt,
+					char *cbuf, size_t cbuflen));
+extern	char *	change_ab __PR((char *cbuf, struct packet *pkt));
+extern	void	initN	__PR((Nparms *N));
+extern	void	freeN	__PR((Nparms *N));
+extern	void	parseN	__PR((Nparms *N));
+extern	char *	bulkprepare __PR((Nparms *N, char *afile));
+extern	int	parseX	__PR((Xparms *X));
+
+extern	int	opendirfd	__PR((const char *name));
+extern	int	closedirfd	__PR((int fd));
+extern	int	opencwd		__PR((void));
+extern	int	openphome	__PR((void));
+extern	BOOL	has_dotdot	__PR((const char *name));
+extern	BOOL	in_tree		__PR((const char *name));
+extern	void	make_relative	__PR((char *name));
 
 /*
  * Declares for external variables in lib/mpwlib
  */
 extern	char	SccsError[MAXERRORLEN];
-extern	char    *setrhome; 
-extern	char    *setahome; 
-extern	char    *cwdprefix; 
-extern	int     homedist; 
 
 /*
- Declares for external functions in lib/mpwlib
-*/
+ * Declares for external functions in lib/mpwlib
+ */
 
 extern	int	any	__PR((int, char *));
 #ifdef	HAVE_STRCHR
@@ -638,6 +805,7 @@ extern	char	*zero	__PR((char *, int));
 extern	void	*fmalloc __PR((unsigned));
 extern	void	ffree	__PR((void *));
 extern	void	ffreeall __PR((void));
+extern	void	*xmalloc __PR((size_t));
 extern	int	efatal	__PR((char *));
 extern	int	fatal	__PR((char *));
 extern	int	lockit	__PR((char *, int, pid_t, char *));
@@ -646,21 +814,26 @@ extern	int	mylock	__PR((char *, pid_t, char *));
 extern	int	sccs_index __PR((char *, char *));
 extern	int	imatch	__PR((char *, char *));
 extern	int	xmsg	__PR((const char *, const char *));
-extern	FILE*	fdfopen	__PR((int, int));
+extern	FILE	*fdfopen	__PR((int, int));
 extern	int	xcreat	__PR((char *, mode_t));
+extern	int	xmkdir	__PR((char *, mode_t));
+extern	int	xmkdirs	__PR((char *, mode_t));
 extern	int	xopen	__PR((char [], int));
 extern	int	xlink	__PR((const char *, const char *));
 extern	int	xunlink	__PR((const char *));
 extern	int	xpipe	__PR((int *));
 extern	void	setsig	__PR((void));
 extern	int	check_permission_SccsDir __PR((char *));
-extern  char*	get_Sccs_Comments __PR((void));
+extern  char	*get_Sccs_Comments __PR((void));
 extern	int	userexit __PR((int code));
 extern	void	*zrealloc __PR((void *ptr, size_t amt));
 extern	int	urandom	__PR((urand_t *urp));
 extern	void	tv2urand __PR((struct timeval *tvp, urand_t *urp));
 extern	void	urand2tv __PR((urand_t *urp, struct timeval *tvp));
-extern	int	sethome	__PR((void));
+extern	int	checkhome __PR((char *path));
+extern	int	sethome	__PR((char *path));
+extern	void	unsethome __PR((void));
+extern	int	xsethome __PR((char *path));
 
 #ifdef	DBG_MALLOC
 extern	void	*dbg_fmalloc __PR((unsigned, char *, int));
@@ -684,19 +857,19 @@ extern	void	set_clean_up	__PR((void (*f)(void)));
 # define VSTART		3
 
 /*
-**	The following five definitions (copy, xfopen, xfcreat, remove,
-**	USXALLOC) are taken from macros.h 1.1
-*/
+ *	The following five definitions (copy, xfopen, xfcreat, remove,
+ *	USXALLOC) are taken from macros.h 1.1
+ */
 
-# define copy(srce,dest)	cat(dest, srce, (char *)0)
-# define xfopen(file,mode)	fdfopen(xopen(file,mode),mode)
-# define xfcreat(file,mode)	fdfopen(xcreat(file,mode), O_WRONLY|O_BINARY)
+# define copy(srce, dest)	cat(dest, srce, (char *)0)
+# define xfopen(file, mode)	fdfopen(xopen(file, mode), mode)
+# define xfcreat(file, mode)	fdfopen(xcreat(file, mode), O_WRONLY|O_BINARY)
 # define remove(file)		xunlink(file)
 # define USXALLOC() \
-		char *alloc(n) {return((char *)xalloc((unsigned)n));} \
-		free(n) char *n; {xfree(n);} \
-		char *malloc(n) unsigned n; {int p; p=xalloc(n); \
-			return((char *)(p != -1?p:0));}
+		char *alloc(n) {return ((char *)xalloc((unsigned)n)); } \
+		free(n) char *n; {xfree(n); } \
+		char *malloc(n) unsigned n; {int p; p = xalloc(n); \
+			return ((char *)(p != -1?p:0)); }
 
 
 #if defined(linux) && !defined(NO_LINUX_SETVBUF_HACK) && \
@@ -706,7 +879,7 @@ extern	void	set_clean_up	__PR((void (*f)(void)));
  * Work around a performance problem in the baroque stdio implementaion on
  * Linux.
  */
-#define	setvbuf(f, buf, type, sz)     setvbuf(f, malloc(sz), type, sz)
+#define	setvbuf(f, buf, type, sz)	setvbuf(f, malloc(sz), type, sz)
 
 static inline int xfclose __PR((FILE *f));
 static inline int xfclose(f)
