@@ -1,4 +1,8 @@
-h16234
+h26596
+s 00001/00001/00057
+d D 1.3 15/06/03 00:06:43 joerg 3 2
+c ../common/test-common -> ../../common/test-common
+e
 s 00003/00001/00055
 d D 1.2 14/08/26 20:06:34 joerg 2 1
 c Zerstoerung der Checksumme hatte auch "V6" -> "V3" gewandelt
@@ -19,7 +23,12 @@
 # admin-hz.sh:  Tests for the -h and -z options of "admin".
 
 # Import common functions & definitions.
+D 3
 . ../common/test-common
+E 3
+I 3
+. ../../common/test-common
+E 3
 
 g=new.txt
 s=s.$g
