@@ -1,4 +1,8 @@
-h45791
+h56147
+s 00001/00001/00057
+d D 1.2 15/06/03 00:06:44 joerg 2 1
+c ../common/test-common -> ../../common/test-common
+e
 s 00058/00000/00000
 d D 1.1 10/04/29 02:05:14 joerg 1 0
 c date and time created 10/04/29 02:05:14 by joerg
@@ -15,7 +19,12 @@
 # body.sh:  Testing for :GB: keyword of prs.
 
 # Import common functions & definitions.
+D 2
 . ../common/test-common
+E 2
+I 2
+. ../../common/test-common
+E 2
 
 remove s.1 p.1 z.1 1 command.log DESC s.foo p.foo z.foo 
 
