@@ -1,4 +1,12 @@
-h16946
+h47043
+s 00002/00002/00025
+d D 1.3 15/06/03 00:06:44 joerg 3 2
+c ../common/test-common -> ../../common/test-common
+e
+s 00001/00001/00026
+d D 1.2 15/06/01 23:55:23 joerg 2 1
+c ../../testutils/ -> ${SRCROOT}/tests/testutils/
+e
 s 00027/00000/00000
 d D 1.1 10/04/29 02:05:14 joerg 1 0
 c date and time created 10/04/29 02:05:14 by joerg
@@ -15,13 +23,24 @@
 # default.sh:  Tests for prt no involving several deltas.
 
 # Import common functions & definitions.
+D 3
 . ../common/test-common
 . ../common/need-prt
+E 3
+I 3
+. ../../common/test-common
+. ../../common/need-prt
+E 3
 
 s=s.testfile
 
 remove $s
+D 2
 ../../testutils/uu_decode --decode < testfile.uue || miscarry could not uudecode testfile.uue.
+E 2
+I 2
+${SRCROOT}/tests/testutils/uu_decode --decode < testfile.uue || miscarry could not uudecode testfile.uue.
+E 2
 
 # XXX: the IGNORE in the followig lines is because of the warning message we
 #      get about the excluded deltas feature not being fully tested.
